@@ -39,6 +39,7 @@ class BoVector3
     BoVector3()  { reset(); }
     BoVector3(GLfloat x, GLfloat y, GLfloat z)  { set(x, y, z); }
     BoVector3(const GLfloat* data) { set(data[0], data[1], data[2]); }
+    BoVector3(const BoVector3& v) { set(v[0], v[1], v[2]); }
     ~BoVector3() {}
 
     /**
@@ -64,7 +65,7 @@ class BoVector3
      **/
     inline void set(GLfloat x, GLfloat y, GLfloat z)
     {
-      mData[0] = x;  mData[1] = y;  mData[2] = z; 
+      mData[0] = x;  mData[1] = y;  mData[2] = z;
     }
     /**
      * @overload
@@ -172,6 +173,11 @@ class BoVector3
 
     /**
      * @return The dot product of this vector with itself, i.e. (v * v).
+     *
+     * The dot product is also equivalent to the square of the @ref length. This
+     * can be important sometimes, as it might be sufficient to use the square
+     * of the length (which is calculated very fast) instead of the actual
+     * length (which needs a call to sqrt()).
      **/
     inline float dotProduct() const
     {
@@ -435,6 +441,7 @@ class BoVector4
     BoVector4()  { reset(); };
     BoVector4(GLfloat x, GLfloat y, GLfloat z, GLfloat w)  { set(x, y, z, w); };
     BoVector4(const GLfloat* data) { set(data[0], data[1], data[2], data[3]); }
+    BoVector4(const BoVector4& v) { set(v[0], v[1], v[2], v[3]); }
     ~BoVector4() {};
 
     /**
@@ -909,6 +916,14 @@ class BoMatrix
      **/
     static void debugMatrix(const GLfloat* matrix);
 
+    /**
+     * See @ref loadMatrix
+     **/
+    inline void operator=(const BoMatrix& m)
+    {
+      loadMatrix(m);
+    }
+
   private:
     /**
      * Used by @ref toGluLookAt. Extract the up vector from the two row vectors
@@ -1253,7 +1268,7 @@ class Bo3dTools
      * is completely in the frustum.
      *
      * @return 0 if the sphere is not in the frustum at all, 1 if it is
-     * partially in the frsutum and 2 if the complete sphere is in the frustum.
+     * partially in the frustum and 2 if the complete sphere is in the frustum.
      **/
     static int sphereCompleteInFrustum(const float* viewFrustum, const BoVector3&, float radius);
 
@@ -1320,6 +1335,30 @@ class Bo3dTools
      **/
     static void enableReadDepthBufferWorkaround(float _1_0_depthValue);
     static void disableReadDepthBufferWorkaround();
+
+    /**
+     * Check whether there is any OpenGL error and output an error to the
+     * konsole if there is one. The error state is cleared then (since @ref
+     * glGetError is used).
+     *
+     * @param error If non-NULL this is set to the error (if any) or to GL_NO_ERROR
+     * @param errorString If non-NULL this is set to the error string describing
+     * @p error, or to an empty string if there is no error. @ref gluErrorString
+     * is used for this string.
+     * @param errorName If non-NULL this is set to the name of the enum that describes
+     * @p error, or an empty string if there is no error.
+     **/
+    static bool checkError(GLenum* error = 0, QString* errorString = 0, QString* errorName = 0);
+
+    /**
+     * @return OpenGL blending function given in QString.
+     * str can be one of "GL_SRC_APLHA", "GL_ONE_MINUS_SRC_ALPHA", "GL_ONE",
+     *  "GL_ZERO", "GL_DST_COLOR", "GL_ONE_MINUS_DST_COLOR", "GL_DST_APLHA",
+     *  "GL_ONE_MINUS_DST_ALPHA", "GL_SRC_ALPHA_SATURATE", "GL_SRC_COLOR",
+     *  "GL_ONE_MINUS_SRC_COLOR". If str is not one of these, GL_INVALID_ENUM
+     *  is returned
+     **/
+    static GLenum string2GLBlendFunc(const QString& str);
 };
 
 
