@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,9 +29,12 @@ class Player;
 class Unit;
 class BosonCanvas;
 class BoSelection;
+class BoSpecificAction;
+class BoFontInfo;
 
 class KPlayer;
 class QDomElement;
+template<class T> class QPtrList;
 
 /**
  * Since boson is able to provide different displays ("views") of the same map
@@ -91,8 +94,11 @@ public:
 	 **/
 	void setLocalPlayer(Player* player);
 
-	/*+
-	 * Set the canvas for all displays
+	/**
+	 * Make the displaymanager use this canvas (e.g. the selections need
+	 * this).
+	 *
+	 * WARNING: this is NOT applied to the displays!
 	 **/
 	void setCanvas(BosonCanvas* canvas);
 
@@ -103,7 +109,6 @@ public:
 	BosonBigDisplayBase* splitActiveDisplayHorizontal();
 
 	void addChatMessage(const QString& text);
-	void mapChanged();
 
 	/**
 	 * See @ref BosonBigDisplayInputBase::unlockAction
@@ -121,6 +126,8 @@ public:
 	 **/
 	double fps() const;
 
+	QPtrList<BosonBigDisplayBase>* displayList();
+
 public slots:
 	/**
 	 * Scroll the active display
@@ -128,6 +135,11 @@ public slots:
 	 **/
 	void slotScroll(int direction);
 
+	void slotRotateLeft();
+	void slotRotateRight();
+	void slotZoomIn();
+	void slotZoomOut();
+
 	/**
 	 * Select the specified group to the active display.
 	 * @param number The group to be selected. Must be in range 0..9 where 1
@@ -150,13 +162,6 @@ public slots:
 	 **/
 	void slotClearGroup(int number);
 
-	/**
-	 * Game mode only: specifies the action (move, attack, mine, ...) that
-	 * will be used for the current selection when the player clicks on the
-	 * map the next time.
-	 **/
-	void slotUnitAction(int action);
-
 	/**
 	 * Editor mode only: specifies the unitType that will be placed on the
 	 * map when the user clicks the next time
@@ -164,10 +169,10 @@ public slots:
 	void slotPlaceUnit(unsigned long int unitType, Player* owner);
 
 	/**
-	 * Editor mode only: specifies the tile/cell that will be placed on the
+	 * Editor mode only: specifies the texture values that will be placed on the
 	 * map when the user clicks the next time
 	 **/
-	void slotPlaceCell(int tile);
+	void slotPlaceGround(unsigned int textureCount, unsigned char* alpha);
 
 	/**
 	 * Editor mode only: delete all currently selected units.
@@ -213,12 +218,15 @@ public slots:
 	 **/
 	void slotActiveSelectSingleUnit(Unit*);
 
-	void slotSetDebugMapCoordinates(bool);
-	void slotSetDebugShowCellGrid(bool);
-	void slotSetDebugMatrices(bool);
-	void slotSetDebugItemWorks(bool);
-	void slotSetDebugCamera(bool);
-	void slotSetDebugRenderCounts(bool);
+	void slotAction(const BoSpecificAction&);
+
+	void slotUpdateOpenGLSettings();
+
+	void slotChangeFont(const BoFontInfo& font);
+
+	void slotSetGrabMovie(bool grab);
+
+	void slotShowLight0Widget();
 
 signals:
 	/**
@@ -255,6 +263,8 @@ protected:
 	BoBox* findBox(BosonBigDisplayBase*) const;
 	void recreateLayout();
 
+	void grabMovieFrame();
+
 protected slots:
 	void slotMakeActiveDisplay(BosonBigDisplayBase*);
 	void slotChangeViewport(BosonBigDisplayBase* display, const QPoint& topLeft, const QPoint& topRight, const QPoint& bottomLeft, const QPoint& bottomRight);
