@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,9 @@
 #include "bomaterial.h"
 #include "bomemorytrace.h"
 #include "bolodbuilder.h"
+#include "bomeshrenderermanager.h"
+#include "bomeshrenderer.h"
+#include "bosonmodel.h"
 
 #include <qptrlist.h>
 #include <qcolor.h>
@@ -31,8 +34,11 @@
 #include <GL/gl.h>
 
 #define AB_DEBUG_1 0
+#define USE_OCCLUSION_CULLING 0 // experimental occlusion culling. this actually makes rendering SLOWER !
 
-extern unsigned int glstat_item_faces, glstat_item_vertices, glstat_terrain_faces, glstat_terrain_vertices;
+#if USE_OCCLUSION_CULLING
+#include <GL/glext.h>
+#endif
 
 class BoundingObject
 {
@@ -117,8 +123,6 @@ public:
 		// have bad bounding boxes - they could easily be smaller.
 		// but that would be harder to code...
 		float v[3];
-		BoMatrix matrix; // TODO use first frame matrix.
-		mesh->calculateMaxMin(&matrix); // probably redundant! we need to ensure that these values are final!
 
 		v[0] = mesh->maxX();
 		v[1] = mesh->maxY();
@@ -180,6 +184,14 @@ public:
 	}
 };
 
+/**
+ * Helper class thaz provides information about whether two faces are adjacent or
+ * not. This is used by some GL_TRIANGLE_STRIP experiments.
+ *
+ * Since STRIPs are basically not applicable for us, this class can probably be
+ * ignored completely. It isn't removed so that we can continue to do some
+ * experiments
+ **/
 class BoAdjacentDataBase
 {
 public:
@@ -248,6 +260,19 @@ private:
 };
 
 
+/**
+ * This class takes care about whether (and how) faces are connected.
+ *
+ * In GL_TRIANGLES rendering mode, simply @ref addNodes is called meaning the
+ * faces are in a simple linear linked list. No special connection takes place.
+ *
+ * When you want to use GL_TRIANGLE_STRIP on the other hand, you will use @ref
+ * connectNodes instead. adjacent faces get connected to a strip then.
+ *
+ * Note that since we do not support GL_TRIANGLE_STRIP (that code is here for
+ * experiments only! we will probably never support them!) only @ref addNodes is
+ * used here.
+ **/
 class BoFaceConnector
 {
 public:
@@ -257,7 +282,7 @@ public:
 	static void disconnectNodes(const QPtrList<BoFaceNode>& allNodes, BoFaceNode*& first)
 	{
 		QPtrListIterator<BoFaceNode> it(allNodes);
-		for (; 	it.current(); ++it) {
+		for (; it.current(); ++it) {
 			it.current()->setNext(0);
 			it.current()->setPrevious(0);
 		}
@@ -292,6 +317,20 @@ public:
 		return GL_TRIANGLES;
 	}
 
+	// AB: there are several problems concerning GL_TRIANGLE_STRIP in boson.
+	// one of them is that it is very hard to create such a strip from our
+	// models - most meshes don't contain stripable data, so we would have
+	// to insert quite a lot of "dummy" vertices (which is more difficult to
+	// code).
+	// Also strips cannot have vertex normals, i.e. we can't have smooth (i
+	// hope thats the correct word now) surfaces with them. Strips must use
+	// face normals (we can provide one normal per vertex only and in strips
+	// every new vertex is a new face).
+	// More problems occur with backface culling. It is possible to create
+	// strips that work with backface culling, but it is more difficult to
+	// code (again).
+	// I think the speed gain would not be worth the effort (and I believe
+	// more problems would come up, especially concerning backface culling)
 	/**
 	 * @return The OpenGL-type of what has been created. That is
 	 * GL_TRIANGLE_STRIP on success and @ref addNodes on failure.
@@ -556,19 +595,14 @@ BoFace& BoFace::operator=(const BoFace& face)
 {
  setPointIndex(face.pointIndex());
  setSmoothGroup(face.smoothGroup());
-#if BOMESH_USE_1_NORMAL_PER_FACE
- mNormals[0] = face.mNormals[0];
-#else
  mNormals[0] = face.mNormals[0];
  mNormals[1] = face.mNormals[1];
  mNormals[2] = face.mNormals[2];
-#endif
  return *this;
 }
 
 
 
-#define BO_NODE_BYTE_SIZE ((1 + 1 + 1 + 3) * 4) // 2 pointer, one int, one 3-int array, each 4 bytes
 BoFaceNode::BoFaceNode(const BoFace* face)
 {
  init();
@@ -577,7 +611,6 @@ BoFaceNode::BoFaceNode(const BoFace* face)
 
 BoFaceNode::~BoFaceNode()
 {
- boMem->subBytes(BO_NODE_BYTE_SIZE);
 }
 
 void BoFaceNode::setPrevious(BoFaceNode* previous)
@@ -625,7 +658,6 @@ void BoFaceNode::init()
  mPrevious = 0;
  mRelevantPoint = -1;
  mFace = 0;
- boMem->addBytes(BO_NODE_BYTE_SIZE);
 }
 
 bool BoFaceNode::isAdjacent(BoFaceNode* f1, BoFaceNode* f2)
@@ -677,6 +709,11 @@ int BoFaceNode::findPointIndex(int index) const
  * initially allocated), so you have to allocate all points that you may need
  * right from the beginning.
  *
+ * Note: a "point" is a collection of multiple values, at least the vertex and
+ * usually also the texel. E.g. a mesh could have two points which are both at
+ * (0,0,0) (i.e. the same vertex). But since they have different texels - say
+ * (0,0) and (0,1) - they are two different points.
+ *
  * @short points collection for @ref BoMesh
  * @author Andreas Beckermann <b_mann@gmx.de>
  **/
@@ -695,20 +732,40 @@ public:
 		boMem->freeFloatArray(mAllocatedPoints);
 		mPoints = 0;
 	}
+
+	/**
+	 * A point is an array describing .. a "point" :) It consists of at
+	 * least the vertex and usually also of the texel. It could also include
+	 * the normal or other data (depending on how we use these data later).
+	 * @return The size of a point
+	 **/
 	inline static int pointSize()
 	{
 		// 3 vertex components, 2 texel components
 		return (3 + 2);
 	}
+
+	/**
+	 * @return The position of the vertex in a point (see @ref pointSize)
+	 **/
 	inline static int vertexPos()
 	{
 		return 0;
 	}
+	/**
+	 * @return The position of a texel in a point (see @ref pointSize)
+	 **/
 	inline static int texelPos()
 	{
 		return 3;
 	}
 
+	/**
+	 * Set the coordinates (i.e. the vertex) of the point at @p index to @p
+	 * vertex.
+	 *
+	 * This method is safe - it won't crash on invalid @p index values.
+	 **/
 	void setVertex(unsigned int index, const BoVector3& vertex)
 	{
 		BO_CHECK_NULL_RET(mPoints);
@@ -721,6 +778,12 @@ public:
 		mPoints[index * pointSize() + vertexPos() + 1] = vertex[1];
 		mPoints[index * pointSize() + vertexPos() + 2] = vertex[2];
 	}
+
+	/**
+	 * Set the texel of the point at @p index to @p texel.
+	 *
+	 * This method is safe - it won't crash on invalid @p index values.
+	 **/
 	void setTexel(unsigned int index, const BoVector3& texel)
 	{
 		BO_CHECK_NULL_RET(mPoints);
@@ -732,7 +795,16 @@ public:
 		mPoints[index * pointSize() + texelPos() + 0] = texel[0];
 		mPoints[index * pointSize() + texelPos() + 1] = texel[1];
 	}
-	inline BoVector3 vertex(unsigned int p) const
+
+	/**
+	 * @return The vertex of the point at @p p.
+	 *
+	 * This method is safe - it won't crash on invalid @p index values.
+	 *
+	 * Note that this method is somewhat slow for using it in mesh
+	 * rendering. It creates a @ref BoVector3 object that is returned then.
+	 **/
+	BoVector3 vertex(unsigned int p) const
 	{
 		if (!mPoints) {
 			boError() << k_funcinfo << "no points allocated" << endl;
@@ -746,6 +818,24 @@ public:
 		return BoVector3(&mPoints[p * pointSize() + vertexPos()]);
 	}
 
+	BoVector3 texel(unsigned int p) const
+	{
+		if (!mPoints) {
+			boError() << k_funcinfo << "no points allocated" << endl;
+			return BoVector3();
+		}
+		if (p >= points()) {
+			boError() << k_funcinfo << "invalid point " << p
+					<< " max=" << points() - 1 << endl;
+			return BoVector3();
+		}
+		BoVector3 tex;
+		tex.setX(mPoints[p * pointSize() + texelPos() + 0]);
+		tex.setY(mPoints[p * pointSize() + texelPos() + 1]);
+		tex.setZ(0.0f);  // not used
+		return tex;
+	}
+
 	inline unsigned int points() const
 	{
 		return mPointCount;
@@ -771,6 +861,10 @@ public:
 		return mPointsMovedBy;
 	}
 
+	/**
+	 * Make sure that we can store @p points in this class. You should not
+	 * call this twice!
+	 **/
 	void allocatePoints(int points)
 	{
 		if (mAllocatedPoints) {
@@ -801,6 +895,22 @@ public:
 		}
 	}
 
+	/**
+	 * This is the tricky part of this class (and the main reason for it
+	 * being a separate class!)
+	 *
+	 * Move all points in this class from the internally allocated array to
+	 * @p array. The firs point in the internal array will be at @p index of
+	 * @p array.
+	 *
+	 * This can be used to make sure that <em>all</em> points of a model
+	 * (not only mesh, but the entire model!) are in a single array. This is
+	 * required for many optimizations, that is why we do it after loading
+	 * all meshes.
+	 *
+	 * The internal array is freed once all points got moved to their new
+	 * location.
+	 **/
 	unsigned int movePoints(float* array, int index)
 	{
 		unsigned int pointsMoved = 0;
@@ -816,7 +926,7 @@ public:
 		}
 		pointsMoved = points();
 
-		#warning FIXME: memory fragmentation
+		// FIXME: memory fragmentation
 		// we allocate a lot of floats and free them later (moving the values to
 		// a single array). this will probably lead to quite some memory
 		// fragmentation. we should change this design.
@@ -855,18 +965,19 @@ BoMeshLOD::BoMeshLOD()
  mPointsCache = 0;
  mPointsCacheCount = 0;
  d->mAllNodes.setAutoDelete(true);
- mDisplayList = 0;
+ mMeshRendererMeshLODData = 0;
 
  mType = GL_TRIANGLES;
 }
 
 BoMeshLOD::~BoMeshLOD()
 {
+ if (mMeshRendererMeshLODData) {
+	boWarning(100) << "meshrenderer forgot to delete meshLOD data" << endl;
+ }
+ delete mMeshRendererMeshLODData;
  d->mAllNodes.clear();
  delete[] mPointsCache;
- if (mDisplayList) {
-	glDeleteLists(mDisplayList, 1);
- }
  delete d;
 }
 
@@ -1052,6 +1163,13 @@ void BoMeshLOD::createPointCache()
  }
 }
 
+void BoMeshLOD::setMeshRendererMeshLODData(BoMeshRendererMeshLODData* data)
+{
+ delete mMeshRendererMeshLODData;
+ mMeshRendererMeshLODData = data;
+}
+
+
 
 
 class BoMeshPrivate
@@ -1085,6 +1203,12 @@ public:
 	BoundingObject* mBoundingObject;
 
 	QString mName;
+
+	// these two make sure that we won't call setVertices()/setTexels()
+	// twice, as that might cause trouble (e.g. with calculateMaxMin() and
+	// methods that already used the old max/min values)
+	bool mVerticesApplied;
+	bool mTexelsApplied;
 };
 
 BoMesh::BoMesh(unsigned int faces, const QString& name)
@@ -1092,6 +1216,8 @@ BoMesh::BoMesh(unsigned int faces, const QString& name)
  init();
 
  d->mName = name;
+ d->mVerticesApplied = false;
+ d->mTexelsApplied = false;
 
  BoMeshLOD* lod = levelOfDetail(0);
  BO_CHECK_NULL_RET(lod);
@@ -1100,6 +1226,10 @@ BoMesh::BoMesh(unsigned int faces, const QString& name)
 
 BoMesh::~BoMesh()
 {
+ if (mMeshRendererMeshData) {
+	boWarning(100) << "meshrenderer forgot to delete mesh data" << endl;
+ }
+ delete mMeshRendererMeshData;
  for (unsigned int i = 0; i < d->mLODCount; i++) {
 	delete d->mLODs[i];
  }
@@ -1126,6 +1256,8 @@ void BoMesh::init()
  d->mLODs = new BoMeshLOD*[1];
  d->mLODs[0] = new BoMeshLOD;
  d->mLODCount = 1;
+
+ mMeshRendererMeshData = 0;
 }
 
 int BoMesh::pointSize()
@@ -1143,11 +1275,6 @@ int BoMesh::texelPos()
  return BoMeshPoints::texelPos();
 }
 
-unsigned int BoMesh::defaultLodCount()
-{
- return 5;
-}
-
 void BoMesh::setMaterial(BoMaterial* mat)
 {
  d->mMaterial = mat;
@@ -1215,9 +1342,31 @@ void BoMesh::allocatePoints(unsigned int points)
  d->mMeshPoints.allocatePoints(points);
 }
 
-void BoMesh::setVertex(unsigned int index, const BoVector3& vertex)
+void BoMesh::setVertices(const QValueVector<BoVector3>& vertices)
 {
- d->mMeshPoints.setVertex(index, vertex);
+ if (d->mVerticesApplied) {
+	// calling this twice might cause trouble with e.g. calculateMaxMin()
+	// and especially with classes/methods that already used the previous
+	// max/min values. so this is not allowed.
+	//
+	// note that we might one day allow to _add_ vertices (BoLODBuilder
+	// might need that). But they should not change the default LOD (and
+	// are therefore irrelevant for max/min and similar things)
+	boError() << k_funcinfo << "called twice. this is not allowed" << endl;
+	return;
+ }
+ if (d->mMeshPoints.points() != vertices.count()) {
+	boError() << k_funcinfo << "vertices count (" << vertices.count()
+			<< ") does not match allocated points (" << d->mMeshPoints.points()
+			<< ")" << endl;
+	return;
+ }
+ for (unsigned int i = 0; i < vertices.count(); i++) {
+	d->mMeshPoints.setVertex(i, vertices[i]);
+ }
+ d->mVerticesApplied = true;
+
+ calculateMaxMin();
 }
 
 BoVector3 BoMesh::vertex(unsigned int p) const
@@ -1248,9 +1397,25 @@ BoVector3 BoMesh::vertex(unsigned int face, unsigned int i, unsigned int _lod) c
  return vertex(f->pointIndex()[i] - d->mMeshPoints.pointsMovedBy());
 }
 
-void BoMesh::setTexel(unsigned int index, const BoVector3& texel)
+void BoMesh::setTexels(const QValueVector<BoVector3>& texels)
 {
- d->mMeshPoints.setTexel(index, texel);
+ if (d->mTexelsApplied) {
+	boError() << k_funcinfo << "called twice. this is not allowed" << endl;
+	return;
+ }
+ if (texels.count() == 0) {
+	return;
+ }
+ if (d->mMeshPoints.points() != texels.count()) {
+	boError() << k_funcinfo << "texel count (" << texels.count()
+			<< ") does not match allocated points (" << d->mMeshPoints.points()
+			<< ")" << endl;
+	return;
+ }
+ for (unsigned int i = 0; i < texels.count(); i++) {
+	d->mMeshPoints.setTexel(i, texels[i]);
+ }
+ d->mTexelsApplied = true;
 }
 
 void BoMesh::calculateNormals()
@@ -1295,7 +1460,6 @@ void BoMesh::calculateNormals(unsigned int _lod)
  //  it's normal is it's face's normal
  for (unsigned int face = 0; face < lod->facesCount(); face++) {
 	if (lod->face(face)->smoothGroup()) {
-#if !BOMESH_USE_1_NORMAL_PER_FACE
 		// Face is in some smoothing group. Use smooth shading
 		for (int p = 0; p < 3; p++) {
 			BoVector3 normal;
@@ -1311,117 +1475,34 @@ void BoMesh::calculateNormals(unsigned int _lod)
 			normal.normalize();
 			lod->setNormal(face, p, normal);
 		}
-#else
-		lod->setNormal(face, -1, facenormals[face]);
-#endif
 	} else {
 		// Face is not in any smoothing group. Use flat shading
 		lod->setNormal(face, -1, facenormals[face]);
 	}
  }
+ delete[] facenormals;
 }
 
-void BoMesh::renderMesh(const QColor* teamColor, unsigned int _lod)
+void BoMesh::renderMesh(const BoMatrix* matrix, const QColor* teamColor, unsigned int _lod)
 {
- if (_lod >= lodCount()) {
-	_lod = lodCount() - 1;
+ BoMeshRenderer* renderer = BoMeshRendererManager::manager()->currentRenderer();
+ if (!renderer) {
+	BO_NULL_ERROR(renderer);
+	return;
+ }
+ if (!matrix) {
+	BO_NULL_ERROR(matrix);
+	return;
  }
  BoMeshLOD* lod = levelOfDetail(_lod);
- if (!lod) {
-	BO_NULL_ERROR(lod);
+ if (lod && lod->pointsCacheCount() == 0) {
+	// nothing to render at this lod. avoid the multmatrix
 	return;
  }
- unsigned int* pointsCache = lod->pointsCache();
- unsigned int pointsCacheCount = lod->pointsCacheCount();
- int type = lod->type();
- BoFaceNode* nodes = lod->nodes();
-
- if (!nodes) {
-	// nothing to do.
-	return;
- }
-
-
- bool resetColor = false; // needs to be true after we changed the current color
-
- // AB: we have *lots* of faces! in numbers the maximum i found
- // so far (only a short look) was about 25 toplevel nodes and
- // rarely child nodes. sometimes 2 child nodes or so - maybe 10
- // per model (if at all).
- // but we have up to (short look only) 116 faces *per node*
- // usually it's about 10-20 faces (minimum) per node!
- //
- // so optimization should happen here - if possible at all...
-
- BoMaterial::activate(material());
- if (!material() && isTeamColor()) {
-	if (teamColor) {
-		glPushAttrib(GL_CURRENT_BIT);
-		glColor3ub(teamColor->red(), teamColor->green(), teamColor->blue());
-		resetColor = true;
-	}
- }
-
-#define USE_OCCLUSION_CULLING 0
-#if USE_OCCLUSION_CULLING
- if (checkVisible())
-#endif
- {
-//	boDebug() << k_funcinfo << "not culled" << endl;
-	if (!pointsCache || pointsCacheCount == 0) {
-		boError() << k_funcinfo << "no point cache!" << endl;
-	} else {
-		glBegin(type);
-
-		BoFaceNode* node = nodes;
-		while (node) {
-			const BoFace* face = node->face();
-			const int* points = face->pointIndex();
-#if BOMESH_USE_1_NORMAL_PER_FACE
-			// here we assume that the same normal is used for all
-			// vertices of this face. this is the default.
-			glNormal3fv(face->normal(0).data());
-			glArrayElement(points[0]);
-			glArrayElement(points[1]);
-			glArrayElement(points[2]);
-#else
-			glNormal3fv(face->normal(0).data());
-			glArrayElement(points[0]);
-			glNormal3fv(face->normal(1).data());
-			glArrayElement(points[1]);
-			glNormal3fv(face->normal(2).data());
-			glArrayElement(points[2]);
-#endif
-
-			glstat_item_vertices += 3;
-			glstat_item_faces++;
-
-			node = node->next();
-		}
-
-		glEnd();
-
-		// reset the normal...
-		// (better solution: don't enable light when rendering
-		// selection rect!)
-		const float n[] = { 0.0f, 0.0f, 1.0f };
-		glNormal3fv(n);
-	}
- }
-
-
- if (resetColor) {
-	// we need to reset the color (mainly for the placement preview)
-	glPopAttrib();
-	resetColor = false;
- }
-
- // we need this currently, because of the selection rects. we should avoid
- // this.
- // maybe place BoMaterial::deactivate() into SelectBox ?
- if (material()) {
-	material()->deactivate();
- }
+ glPushMatrix();
+ glMultMatrixf(matrix->data());
+ renderer->renderMesh(teamColor, this, _lod);
+ glPopMatrix();
 }
 
 void BoMesh::renderBoundingObject()
@@ -1465,42 +1546,6 @@ void BoMesh::renderVertexPoints(unsigned int _lod)
  glEnd();
 }
 
-// there MUST be a valid context set already!!
-void BoMesh::loadDisplayList(const QColor* teamColor, bool reload)
-{
- for (unsigned int i = 0; i < d->mLODCount; i++) {
-	loadDisplayList(d->mLODs[i], teamColor, reload);
- }
-}
-
-void BoMesh::loadDisplayList(BoMeshLOD* lod, const QColor* teamColor, bool reload)
-{
- BO_CHECK_NULL_RET(lod);
-
- if (!lod->displayList()) {
-	lod->setDisplayList(glGenLists(1));
-	if (lod->displayList() == 0) {
-		boError() << k_funcinfo << "NULL display list generated" << endl;
-		return;
-	}
- } else {
-	if (reload) {
-		// it is important that the list is deleted, but the
-		// newly generated list must have the *same* number as
-		// it had before!
-		glDeleteLists(lod->displayList(), 1);
-	} else {
-		boWarning() << k_funcinfo << "mesh was already loaded!" << endl;
-		return;
-	}
- }
- boDebug(100) << k_funcinfo << endl;
-
- glNewList(lod->displayList(), GL_COMPILE);
- renderMesh(teamColor);
- glEndList();
-}
-
 unsigned int BoMesh::points() const
 {
  return d->mMeshPoints.points();
@@ -1516,33 +1561,6 @@ void BoMesh::setIsTeamColor(bool c)
  d->mIsTeamColor = c;
 }
 
-GLuint BoMesh::displayList(unsigned int lod) const
-{
- BoMeshLOD* l = levelOfDetail(lod);
- if (!l) {
-	boError(100) << k_funcinfo << "NULL lod " << lod << endl;
-	return 0;
- }
- return l->displayList();
-}
-
-unsigned int BoMesh::movePoints(float* array, int index)
-{
- // move all points to the new array
- unsigned int pointsMoved = d->mMeshPoints.movePoints(array, index);
-
- // now we fix the (point-)indices of all faces in all LODs.
- for (unsigned int i = 0; i < d->mLODCount; i++) {
-	if (!d->mLODs[i]) {
-		boError(100) << k_funcinfo << "NULL LOD " << i << endl;
-		continue;
-	}
-	d->mLODs[i]->movePointIndices(index);
- }
-
- return pointsMoved;
-}
-
 void BoMesh::createPointCache()
 {
  for (unsigned int i = 0; i < d->mLODCount; i++) {
@@ -1552,24 +1570,17 @@ void BoMesh::createPointCache()
  }
 }
 
-void BoMesh::calculateMaxMin(const BoMatrix* matrix)
+void BoMesh::calculateMaxMin()
 {
  // NOT time critical! (called on startup only)
- if (!matrix) {
-	BoMatrix m;
-	calculateMaxMin(&m);
-	return;
- }
- BoVector3 v1 = vertex(0);
- BoVector3 v;
- matrix->transform(&v, &v1);
+ BoVector3 v = vertex(0);
  d->mMinZ = v.z();
  d->mMaxZ = v.z();
  d->mMinX = v.x();
  d->mMaxX = v.x();
  d->mMinY = v.y();
  d->mMaxY = v.y();
- for (unsigned int i = 0; i < points(); i++) {
+ for (unsigned int i = 1; i < points(); i++) {
 	v = vertex(i);
 	if (v.x() < d->mMinX) {
 		d->mMinX = v.x();
@@ -1624,6 +1635,7 @@ bool BoMesh::checkVisible()
  if (!d->mBoundingObject) {
 	return false;
  }
+#if USE_OCCLUSION_CULLING
  GLboolean result;
 #if 1
  glDisable(GL_TEXTURE_2D);
@@ -1649,6 +1661,9 @@ bool BoMesh::checkVisible()
  glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
 #endif
  return result;
+#else // USE_OCCLUSION_CULLING
+ return true;
+#endif // USE_OCCLUSION_CULLING
 }
 
 void BoMesh::computeBoundingObject()
@@ -1660,13 +1675,11 @@ void BoMesh::computeBoundingObject()
  d->mBoundingObject = builder.generateBoundingObject(this);
 }
 
-void BoMesh::generateLOD()
+void BoMesh::generateLOD(unsigned int LODCount, const BosonModel* model)
 {
- // how many LODs should be generated. maybe use a parameter for this?
- // these numbers can be used by renderMesh() (or indirectly by renderFrame())
- // to choose which LOD should be rendered at. 0 is always the default, i.e.
- // render all points and faces.
- unsigned int LODCount = defaultLodCount(); // must be at least 1, as we have at leas the full-detailed version
+ // LODCount must ALWAYS be greate or equal to 1, as 0 is the default (full
+ // detailed) LOD, which must always be present.
+
 
  unsigned int oldCount = d->mLODCount;
  if (LODCount < oldCount) {
@@ -1678,11 +1691,16 @@ void BoMesh::generateLOD()
 	LODCount = 1;
 	boWarning(100) << k_funcinfo << "must create at least one LOD" << endl;
  }
+ if (oldCount == LODCount) {
+	boDebug(100) << k_funcinfo << "old LOD count == new LOD count. nothing to do." << endl;
+	return;
+ }
  BoMeshLOD** lod = new BoMeshLOD*[LODCount];
  for (unsigned int i = 0; i < oldCount; i++) {
 	lod[i] = d->mLODs[i];
  }
  if (oldCount == 0) {
+	boError() << k_funcinfo << "old LOD count is 0?! must be at least 1! will probably crash!" << endl;
 	lod[0] = 0;
  }
  boDebug(100) << k_funcinfo << "lods=" << LODCount
@@ -1700,6 +1718,53 @@ void BoMesh::generateLOD()
  d->mLODCount = LODCount;
 
  BoLODBuilder builder(this, lod[0]);
+
+ // fill the builder with data about the model
+ if (model->frame(0)) {
+	// WARNING we operate on frame 0 only - this might cause trouble, when
+	// meshes are getting larger (by scaling them) in later frames!
+	BoFrame* frame0 = model->frame(0);
+	float maxModelSurface = 0.0f;
+	float modelVolume = 0.0f;
+	float volume = 0.0f;
+	float maxSurface = 0.0f; // max surface of _this_ mesh
+	float largestMeshVolume = 0.0f;
+	float largestMeshSurface = 0.0f;
+
+	float depthMultiplier = frame0->depthMultiplier(); // height in z direction
+
+	maxModelSurface = model->width() * model->height();
+	modelVolume = model->width() * model->height() * depthMultiplier;
+	maxModelSurface = QMAX(maxModelSurface, model->width() * depthMultiplier);
+	maxModelSurface = QMAX(maxModelSurface, model->height() * depthMultiplier);
+
+	float minX, minY, minZ;
+	float maxX, maxY, maxZ;
+	for (unsigned int i = 0; i < frame0->meshCount(); i++) {
+		const BoMesh* mesh = frame0->mesh(i);
+		const BoMatrix* matrix = frame0->matrix(i);
+		mesh->getBoundingBox(*matrix, &minX, &maxX, &minY, &maxY, &minZ, &maxZ);
+		float lengthX = maxX - minX;
+		float lengthY = maxY - minY;
+		float lengthZ = maxZ - minZ;
+		float s = lengthX * lengthY;
+		s = QMAX(s, lengthX * lengthZ);
+		s = QMAX(s, lengthY * lengthZ);
+		float v = lengthX * lengthY * lengthZ;
+		if (mesh == this) {
+			maxSurface = s;
+			volume = v;
+		}
+		largestMeshVolume = QMAX(largestMeshVolume, v);
+		largestMeshSurface = QMAX(largestMeshSurface, s);
+	}
+
+
+	builder.setModelData(volume, maxSurface, modelVolume, maxModelSurface, largestMeshVolume, largestMeshSurface);
+ } else {
+	BO_NULL_ERROR(model->frame(0));
+ }
+
  for (unsigned int i = oldCount; i < LODCount; i++) {
 	QValueList<BoFace> faces = builder.generateLOD(i);
 	if (faces.count() > 0) {
@@ -1746,3 +1811,81 @@ const QString& BoMesh::name() const
  return d->mName;
 }
 
+unsigned int BoMesh::movePoints(float* array, int index)
+{
+ // move all points to the new array
+ unsigned int pointsMoved = d->mMeshPoints.movePoints(array, index);
+
+ // now we fix the (point-)indices of all faces in all LODs.
+ for (unsigned int i = 0; i < d->mLODCount; i++) {
+	if (!d->mLODs[i]) {
+		boError(100) << k_funcinfo << "NULL LOD " << i << endl;
+		continue;
+	}
+	d->mLODs[i]->movePointIndices(index);
+ }
+
+ return pointsMoved;
+}
+
+void BoMesh::setMeshRendererMeshData(BoMeshRendererMeshData* data)
+{
+ delete mMeshRendererMeshData;
+ mMeshRendererMeshData = data;
+}
+
+void BoMesh::getBoundingBox(float* _minX, float* _maxX, float* _minY, float* _maxY, float* _minZ, float* _maxZ) const
+{
+ *_minX = minX();
+ *_minY = minY();
+ *_minZ = minZ();
+ *_maxX = maxX();
+ *_maxY = maxY();
+ *_maxZ = maxZ();
+}
+
+void BoMesh::getBoundingBox(BoVector3* vertices) const
+{
+ float minX, minY, minZ;
+ float maxX, maxY, maxZ;
+ getBoundingBox(&minX, &maxX, &minY, &maxY, &minZ, &maxZ);
+ vertices[0].set(minX, minY, minZ);
+ vertices[1].set(minX, minY, maxZ);
+ vertices[2].set(minX, maxY, minZ);
+ vertices[3].set(minX, maxY, maxZ);
+ vertices[4].set(maxX, minY, minZ);
+ vertices[5].set(maxX, minY, maxZ);
+ vertices[6].set(maxX, maxY, minZ);
+ vertices[7].set(maxX, maxY, maxZ);
+}
+
+void BoMesh::getBoundingBox(const BoMatrix& matrix, float* minX, float* maxX, float* minY, float* maxY, float* minZ, float* maxZ) const
+{
+ BoVector3 _vertices[8];
+ BoVector3 vertices[8];
+ getBoundingBox(_vertices);
+ for (int i = 0 ; i < 8; i++) {
+	matrix.transform(&vertices[i], &_vertices[i]);
+ }
+ *minX = *maxX = vertices[0].x();
+ *minY = *maxY = vertices[0].y();
+ *minZ = *maxZ = vertices[0].z();
+ for (int i = 1; i < 7; i++) {
+	if (vertices[i].x() < *minX) {
+		*minX = vertices[i].x();
+	} else if (vertices[i].x() > *maxX) {
+		*maxX = vertices[i].x();
+	}
+	if (vertices[i].y() < *minY) {
+		*minY = vertices[i].y();
+	} else if (vertices[i].y() > *maxY) {
+		*maxY = vertices[i].y();
+	}
+	if (vertices[i].z() < *minZ) {
+		*minZ = vertices[i].z();
+	} else if (vertices[i].z() > *maxZ) {
+		*maxZ = vertices[i].z();
+	}
+ }
+}
+
