@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,21 +26,12 @@ class BoMesh;
 class BoMaterial;
 class BoAdjacentDataBase;
 class BoMeshLOD;
+class BoMeshRendererMeshData;
+class BoMeshRendererMeshLODData;
+class BosonModel;
 class QColor;
 
-// AB: there are two different ways for normals: store one normal per face
-// ("surface normal") or store one normal per vertex, i.e. 3 per face ("vertex
-// normal").
-// the "surface normal" is said to produce objects that appear "flat", whereas
-// the "vertex normals" are used for "curved" objects. so vertex normals would
-// be nicer, but take three times as much memory.
-// i seriously think that we don't need vertex normals. our camera rarely moves
-// *that* close that you can see whether the mesh appears slightly more flat or
-// not.
-// atm we don't have code for vertex normals anyway.
-//
-// use 0 here to use vertex normals.
-#define BOMESH_USE_1_NORMAL_PER_FACE 0
+template<class T> class QValueVector;
 
 class BoFace
 {
@@ -78,33 +69,20 @@ public:
 	 **/
 	void setAllNormals(const BoVector3 normal)
 	{
-#if BOMESH_USE_1_NORMAL_PER_FACE
-		setNormal(0, normal);
-#else
 		setNormal(0, normal);
 		setNormal(1, normal);
 		setNormal(2, normal);
-#endif
 	}
 	void setNormal(unsigned int i, const BoVector3 normal)
 	{
-#if BOMESH_USE_1_NORMAL_PER_FACE
-		i = 0;
-#else
 		// values > 2 are not allowed
 		i = i % 3;
-#endif
 		mNormals[i] = normal;
 	}
 	inline const BoVector3& normal(unsigned int i) const
 	{
-#if BOMESH_USE_1_NORMAL_PER_FACE
-		Q_UNUSED(i);
-		return mNormals[0];
-#else
 		// we don't do i = i % 3; as of performance reasons
 		return mNormals[i];
-#endif
 	}
 
 	void setSmoothGroup(unsigned long int group)
@@ -121,11 +99,7 @@ private:
 
 	unsigned long int mSmoothGroup;
 
-#if BOMESH_USE_1_NORMAL_PER_FACE
-	BoVector3 mNormals[1];
-#else
 	BoVector3 mNormals[3];
-#endif
 };
 
 /**
@@ -268,15 +242,6 @@ public:
 	void setFace(int index, const BoFace& face);
 	const BoFace* face(unsigned int f) const;
 
-	void setDisplayList(GLuint list)
-	{
-		mDisplayList = list;
-	}
-	inline GLuint displayList() const
-	{
-		return mDisplayList;
-	}
-
 	/**
 	 * @param vertex The vertex in the face this normal applies to. This
 	 * must be 0..2 or -1 for all vertices.
@@ -304,6 +269,13 @@ public:
 	void connectNodes();
 	void addNodes();
 
+	inline BoMeshRendererMeshLODData* meshRendererMeshLODData() const
+	{
+		return mMeshRendererMeshLODData;
+	}
+
+	void setMeshRendererMeshLODData(BoMeshRendererMeshLODData* data);
+
 private:
 	BoMeshLODPrivate* d;
 
@@ -311,13 +283,13 @@ private:
 	BoFaceNode* mNodes;
 	int mType;
 
+	BoMeshRendererMeshLODData* mMeshRendererMeshLODData;
+
 	// the list of points in the final order (after connectNodes() or
 	// addNodes() was called). iterating through nodes() is equalivent (for
 	// some modes the BoFaceNode::relevantPoint() will have to be used though)
 	unsigned int* mPointsCache;
 	unsigned int mPointsCacheCount;
-
-	GLuint mDisplayList;
 };
 
 
@@ -345,12 +317,6 @@ public:
 
 	static int texelPos();
 
-	/**
-	 * @return How many LOD levels mesh has by default
-	 */
-	static unsigned int defaultLodCount();
-
-
 	/**
 	 * Prepare to load the points, i.e. allocate memory for them. You can
 	 * set them using @ref setVertex and @ref setTexel.
@@ -392,24 +358,33 @@ public:
 
 	/**
 	 * You must call @ref allocatePoints before calling this!
-	 * @param index The index of the vertex in the vertex pool (relative to
-	 * this mesh). It must be < @ref points.
 	 *
-	 * Note that this changes the vertex at @p index for all LODs that
-	 * reference that index!
+	 * This methods sets the list of available vertices for this mesh. The
+	 * faces (see @ref BoFace) may reference these.
+	 *
+	 * You are not allowed to call this twice!
+	 * -> Some pre-parsing may occur at this point already (or on demand
+	 *  after this point), and other methods/classes that use the generated
+	 *  data might not know about later changes.
+	 * @param vertices A value vector containing all vertices. The size of the
+	 * vector must match exactly the size that was allocated using @ref
+	 * allocatePoints.
 	 **/
-	void setVertex(unsigned int index, const BoVector3&);
+	// AB: if BoLODBuilder ever needs to add points/vertices, we should add
+	// a addVertex() method, which _appends_ a vertex to the internal array.
+	// existing vertices should not be removed.
+	void setVertices(const QValueVector<BoVector3>& vertices);
 
 	void calculateNormals();
 
 	/**
-	 * The third coordinate is discarded.
-	 * @param index See @ref setVertex.
+	 * Just like @ref setVertices, just for texels. All rules for the
+	 * @ref setVertices are valid here, too, but a zero size vector is
+	 * valid.
 	 *
-	 * Note that this changes the texel at @p index for all LODs that
-	 * reference that index!
+	 * Note that the third component of every texel is discarded.
 	 **/
-	void setTexel(unsigned int index, const BoVector3&);
+	void setTexels(const QValueVector<BoVector3>& texels);
 
 	/**
 	 * Try to connect all faces in the mesh, so that we can use
@@ -471,7 +446,7 @@ public:
 	 **/
 	const QString& name() const;
 
-	void renderMesh(const QColor* color, unsigned int lod = 0);
+	void renderMesh(const BoMatrix* matrix, const QColor* color, unsigned int lod = 0);
 
 	/**
 	 * Render the bounding object (usually a mesh) of this mesh
@@ -485,15 +460,15 @@ public:
 	 **/
 	void renderVertexPoints(unsigned int lod = 0);
 
-	void loadDisplayList(const QColor* teamColor, bool reload = false);
-	GLuint displayList(unsigned int lod) const;
-
-
 	/**
 	 * Create a BoVector3 at index @p p from the vertex pool.
 	 *
 	 * @param p The index of the vertex in the vertex pool. Must
 	 * be < @ref points.
+	 *
+	 * See also @ref BosonModel::vertex, which uses indices that are
+	 * <em>global</em> to the model (as in @ref BoFace::pointIndex), while
+	 * this method uses indices that are local to this mesh.
 	 **/
 	BoVector3 vertex(unsigned int p) const;
 
@@ -503,16 +478,6 @@ public:
 	 **/
 	unsigned int points() const;
 
-	/**
-	 * Calculate values for @ref maxZPoint and similar functions. This needs
-	 * to get called whenever the values might change!
-	 *
-	 * @param matrix The matrix of the first frame. Note that the max/min
-	 * values can change when the frames changes, but we will always use the
-	 * values from the _first_ frame only.
-	 **/
-	void calculateMaxMin(const BoMatrix* matrix);
-
 	/**
 	 * @return The maximal x value in this mesh. Call @ref calculateMaxMin
 	 * before you use this
@@ -549,6 +514,45 @@ public:
 	 **/
 	float minZ() const;
 
+	/**
+	 * Return the bounding box in the specified parameters.
+	 *
+	 * The bounding box consists of the 8 points that are made by the
+	 * possible combinations of the min/max x/y/z values.
+	 *
+	 * Note that the bounding box is axis aligned in all cases and might
+	 * therefore be bigger than necessary.
+	 *
+	 * This method is equivalent to calling @ref minX, @ref maxX, ... for
+	 * all specified parameters.
+	 **/
+	void getBoundingBox(float* minX, float* maxX, float* minY, float* maxY, float* minZ, float* maxZ) const;
+
+	/**
+	 * @overloaded
+	 * This version behaves just like the above method, but it returns all
+	 * combinations of min/max x/y/z in @p vertices. These 8 vertices are
+	 * all vertices of the bounding box.
+	 * @param vertices Must be an array of 8. The vertices of the bounding
+	 * box are returned here.
+	 **/
+	void getBoundingBox(BoVector3* vertices) const;
+
+	/**
+	 * Return the bounding box after transforming the mesh by @p matrix. See
+	 * the above version for more information.
+	 *
+	 * Note that the returned bounding box is still axis aligned and
+	 * therefore is probably bigger than necessary!
+	 *
+	 * Also note that this bounding box is retrieved by transforming the
+	 * bounding box returned by the above method by @p matrix, and therefore
+	 * is probably <em>much</em> larger than necessary. For better results
+	 * you should transform all vertices (see @ref vertex) by @ref matrix
+	 * and find the min/max values from all resulting vertices.
+	 **/
+	void getBoundingBox(const BoMatrix& matrix, float* minX, float* maxX, float* minY, float* maxY, float* minZ, float* maxZ) const;
+
 	/**
 	 * Compute a bounding object (usually a box) for the mesh.
 	 **/
@@ -556,11 +560,20 @@ public:
 
 	/**
 	 * Called by @ref BosonModel to generate LODs for the mesh.
+	 * @param lodCount How many levels of detail will be generated. Must be
+	 * at least 1 and must be the same value for all meshes in the model.
 	 **/
-	void generateLOD();
+	void generateLOD(unsigned int lodCount, const BosonModel* parent);
 
 	unsigned int facesCount(unsigned int lod) const;
 
+	inline BoMeshRendererMeshData* meshRendererMeshData() const
+	{
+		return mMeshRendererMeshData;
+	}
+
+	void setMeshRendererMeshData(BoMeshRendererMeshData* data);
+
 protected:
 	// this is meant to check whether the something on the screen will
 	// change if we draw this mesh now.
@@ -584,13 +597,70 @@ protected:
 	void calculateNormals(unsigned int lod);
 
 	void setNormal(unsigned int face, int vertex, const BoVector3& normal);
-	void loadDisplayList(BoMeshLOD* lod, const QColor* teamColor, bool reload = false);
+
+	/**
+	 * Calculate values for @ref maxZPoint and similar functions.
+	 *
+	 * Called by @ref setVertices
+	 **/
+	void calculateMaxMin();
 
 private:
 	void init();
+	friend class BoMeshRenderer;
 
 private:
 	BoMeshPrivate* d;
+	BoMeshRendererMeshData* mMeshRendererMeshData;
+};
+
+
+/**
+ * Same as @ref BoMeshRendererModelData, but this stores data for the @ref
+ * BoMesh.
+ *
+ * See also @ref BoMesh::meshRendererMeshData and @ref
+ * BoMeshRenderer::initMeshData.
+ *
+ * If you are not sure whether your data belongs to BoMeshRendererMeshData or to
+ * @ref BoMeshRendererMeshLODData, try @ref BoMeshRendererMeshLODData first.
+ * There you will have less trouble retrieving certain data (such as normals).
+ * @short Simple storage class for @ref BoMesh and @ref BoMeshRenderer
+ * @author Andreas Beckermann <b_mann@gmx.de>
+ **/
+class BoMeshRendererMeshData
+{
+public:
+	BoMeshRendererMeshData()
+	{
+	}
+	virtual ~BoMeshRendererMeshData()
+	{
+	}
+};
+
+/**
+ * Same as @ref BoMeshRendererModelData, but this stores data for the @ref
+ * BoMeshLOD.
+ *
+ * See also @ref BoMesh::meshRendererMeshData and @ref
+ * BoMeshRenderer::initMeshLODData.
+ *
+ * You will most probably want to store your mesh relevant data in this class
+ * (e.g. vertex buffer objects), if you are not sure whether you need @ref
+ * BoMeshRendererMeshData or BoMeshRendererLODData, try this one first.
+ * @short Simple storage class for @ref BoMeshLOD and @ref BoMeshRenderer
+ * @author Andreas Beckermann <b_mann@gmx.de>
+ **/
+class BoMeshRendererMeshLODData
+{
+public:
+	BoMeshRendererMeshLODData()
+	{
+	}
+	virtual ~BoMeshRendererMeshLODData()
+	{
+	}
 };
 
 #endif
