@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,9 +23,10 @@
 #include "speciestheme.h"
 #include "bosonconfig.h"
 #include "bosonmodel.h"
+#include "bosonmodeltextures.h"
 #include "bomesh.h"
 #include "bomaterial.h"
-#include "bosonglfont.h"
+#include "bosonfont/bosonglfont.h"
 #include "bosonprofiling.h"
 #include "unitproperties.h"
 #include "kgamemodeldebug.h"
@@ -35,6 +36,11 @@
 #include "boapplication.h"
 #include "bocamera.h"
 #include "bocamerawidget.h"
+#include "bomaterialwidget.h"
+#include "bolight.h"
+#include "bomeshrenderermanager.h"
+#include "boglstatewidget.h"
+#include "info/boinfo.h"
 
 #include <kcmdlineargs.h>
 #include <kaboutdata.h>
@@ -44,6 +50,7 @@
 #include <kstandarddirs.h>
 #include <kdialogbase.h>
 #include <kcolordialog.h>
+#include <kmessagebox.h>
 
 #include <qtimer.h>
 #include <qhbox.h>
@@ -53,12 +60,17 @@
 #include <qinputdialog.h>
 
 #include <GL/glu.h>
+#include <math.h>
+#include <stdlib.h>
 
 #define NEAR 1.0
 #define FAR 100.0
 
 #define GL_UPDATE_TIMER 40
 
+#define BORENDER_DEFAULT_LIGHT_ENABLED true
+#define BORENDER_DEFAULT_MATERIALS_ENABLED true
+
 static const char *description =
     I18N_NOOP("Rendering tester for Boson");
 
@@ -120,10 +132,14 @@ ModelPreview::ModelPreview(QWidget* parent) : BosonGLWidget(parent)
  mDisallowPlacement = false;
  mWireFrame = false;
  mConstruction = false;
+ mRenderAxis = false;
+ mRenderGrid = false;
 
  mDefaultFont = 0;
 
  mCamera = new BoCamera;
+ mLight = 0;
+
 
  connect(this, SIGNAL(signalFovYChanged(float)), this, SLOT(slotFovYChanged(float)));
  connect(this, SIGNAL(signalFrameChanged(int)), this, SLOT(slotFrameChanged(int)));
@@ -134,28 +150,61 @@ ModelPreview::ModelPreview(QWidget* parent) : BosonGLWidget(parent)
  boConfig->addDynamicEntry(new BoConfigBoolEntry(boConfig, "ShowVertexPoints", true));
  boConfig->addDynamicEntry(new BoConfigUIntEntry(boConfig, "VertexPointSize", 3));
  boConfig->addDynamicEntry(new BoConfigColorEntry(boConfig, "BoRenderBackgroundColor", Qt::black));
+ boConfig->addDynamicEntry(new BoConfigDoubleEntry(boConfig, "GridUnitSize", 0.1));
 }
 
 ModelPreview::~ModelPreview()
 {
  qApp->setGlobalMouseTracking(false);
  resetModel();
+ BoMeshRendererManager::manager()->unsetCurrentRenderer();
  delete mUpdateTimer;
  delete mMouseMoveDiff;
+ delete mLight;
+ delete mCamera;
 }
 
 void ModelPreview::initializeGL()
 {
+ if (isInitialized()) {
+	return;
+ }
+ static bool recursive = false;
+ if (recursive) {
+	return;
+ }
+ recursive = true;
  makeCurrent();
+ delete mDefaultFont;
  mDefaultFont = new BosonGLFont(QString::fromLatin1("fixed"));
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glShadeModel(GL_FLAT);
  glDisable(GL_DITHER);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
- glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+
+ delete mLight;
+ mLight = new BoLight;
+ if (mLight->id() < 0) {
+	boError() << k_funcinfo << "light NOT created" << endl;
+	delete mLight;
+	mLight = 0;
+ } else {
+	BoVector4 lightDif(1.0f, 1.0f, 1.0f, 1.0f);
+	BoVector4 lightAmb(0.5f, 0.5f, 0.5f, 1.0f);
+	BoVector3 lightPos(-6000.0, 3000.0, 10000.0);
+	mLight->setAmbient(lightAmb);
+	mLight->setDiffuse(lightDif);
+	mLight->setSpecular(lightDif);
+	mLight->setDirectional(true);
+	mLight->setEnabled(true);
+ }
+
  setUpdatesEnabled(false);
  mUpdateTimer->start(GL_UPDATE_TIMER);
+
+
+ recursive = false;
 }
 
 void ModelPreview::resizeGL(int w, int h)
@@ -174,26 +223,91 @@ void ModelPreview::paintGL()
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
 
- camera()->applyCameraToScene();
-
  QColor background = boConfig->colorValue("BoRenderBackgroundColor", Qt::black);
  glClearColor((GLfloat)background.red() / 255.0f, (GLfloat)background.green() / 255.0f, background.blue() / 255.0f, 0.0f);
 
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glColor3ub(255, 255, 255);
 
+ camera()->applyCameraToScene();
+
+ if (mRenderGrid) {
+	renderGrid();
+ }
+ if (mRenderAxis) {
+	renderAxii();
+ }
+
+ if (boConfig->useLight()) {
+	glEnable(GL_LIGHTING);
+	glEnable(GL_COLOR_MATERIAL);
+	glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
+	glEnable(GL_NORMALIZE);
+ }
+
  renderModel();
  renderMeshSelection();
-// renderGrid();
+
+ glDisable(GL_LIGHTING);
+ glDisable(GL_NORMALIZE);
 
  glDisable(GL_DEPTH_TEST);
 
  renderText();
+}
 
- GLenum e = glGetError();
- if (e != GL_NO_ERROR) {
-	boError() << k_funcinfo << "OpenGL error: " << (int)e << endl;
- }
+void ModelPreview::renderAxii()
+{
+ glPushAttrib(GL_ENABLE_BIT);
+ glDisable(GL_TEXTURE_2D);
+ glDisable(GL_BLEND);
+ glColor3ub(255, 0, 0);
+
+ // FIXME: we _cannot_ use really high numbers (has strange effects on the lines
+ // - if you make the numbers high enough, they even won't appear at all)
+ // i guess we should get the size of the bounding box of the model, add a
+ // certain value to that and use that size as maximum.
+ // -> the axii would be big enough for the model then
+ // 10.0 is high enough for us atm, as we scale every model to 1.0 anyway.
+ float max = 10.0f;
+ glBegin(GL_LINES);
+	glVertex3f(-max, 0.0f, 0.0f);
+	glVertex3f(max, 0.0f, 0.0f);
+
+	glVertex3f(0.0f, -max, 0.0f);
+	glVertex3f(0.0f, max, 0.0f);
+
+	glVertex3f(0.0f, 0.0f, -max);
+	glVertex3f(0.0f, 0.0f, max);
+
+	const float offset = 1.1f;
+	const float distance = 0.1f; // distance of the letter to the line
+
+	// "X"
+	glVertex3f(offset, distance, 0.0f);
+	glVertex3f(offset + 0.666f, distance + 1.0f, 0.0f);
+	glVertex3f(offset, distance + 1.0f, 0.0f);
+	glVertex3f(offset + 0.666f, distance, 0.0f);
+
+	// "Y"
+	glVertex3f(distance, offset + 1.0f, 0.0f);
+	glVertex3f(distance + 0.5f, offset + 0.5f, 0.0f);
+	glVertex3f(distance + 0.5f, offset + 0.5f, 0.0f);
+	glVertex3f(distance + 1.0f, offset + 1.0f, 0.0f);
+	glVertex3f(distance + 0.5f, offset + 0.5f, 0.0f);
+	glVertex3f(distance + 0.5f, offset, 0.0f);
+
+	// "Z"
+	glVertex3f(0.0f, distance + 1.0f, -offset);
+	glVertex3f(0.0f, distance + 1.0f, -offset - 1.0f);
+	glVertex3f(0.0f, distance + 1.0f, -offset - 1.0f);
+	glVertex3f(0.0f, distance, -offset);
+	glVertex3f(0.0f, distance, -offset);
+	glVertex3f(0.0f, distance, -offset - 1.0f);
+ glEnd();
+
+ glColor3ub(255, 255, 255);
+ glPopAttrib();
 }
 
 void ModelPreview::renderModel(int mode)
@@ -202,6 +316,8 @@ void ModelPreview::renderModel(int mode)
 	return;
  }
 
+ glPushAttrib(GL_ENABLE_BIT | GL_POLYGON_BIT); // doesnt include client states
+
  if (mWireFrame) {
 	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  } else {
@@ -223,7 +339,7 @@ void ModelPreview::renderModel(int mode)
 	if (f) {
 		if (mPlacementPreview) {
 			glEnable(GL_BLEND);
-			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); // default anyway - redundant call
 			GLubyte r, g, b, a;
 			a = PLACEMENTPREVIEW_ALPHA;
 			r = 255;
@@ -237,18 +353,17 @@ void ModelPreview::renderModel(int mode)
 			glColor4ub(r, g, b, a);
 		}
 
-		// FIXME: this isn't good here...
-		mModel->enablePointer();
-
 		if (mode == GL_SELECT) {
 			glDisable(GL_BLEND);
 			glDisable(GL_TEXTURE_2D);
 		}
+		BosonModel::startModelRendering();
+		mModel->prepareRendering();
 		f->renderFrame(0, mCurrentLOD, mode);
+		BosonModel::stopModelRendering();
 		if (mPlacementPreview) {
 			// AB: do not reset the actual color - if it will get
 			// used it will be set again anyway.
-			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 			glColor4ub(255, 255, 255, 255);
 		}
 	} else {
@@ -256,159 +371,48 @@ void ModelPreview::renderModel(int mode)
 	}
  }
 
- glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
+ glPopAttrib();
  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
- glDisable(GL_CULL_FACE);
- glDisable(GL_TEXTURE_2D);
- glDisable(GL_DEPTH_TEST);
-
 }
 
 void ModelPreview::renderGrid()
 {
- if (!haveModel()) {
-	return;
- }
+ glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_TEXTURE_2D);
  glEnable(GL_DEPTH_TEST);
- float w = mModel->width();
- float h = mModel->height();
- if ((float)((int)w) != w) {
-	w = (float)((int)w + 1);
+ float size = 2.0f;
+ if (mModel) {
+	size = fmaxf(mModel->width(), mModel->height());
  }
- if ((float)((int)h) != h) {
-	h = (float)((int)h + 1);
- }
- if ((int)w % 2 != 0) {
-	w += 1.0f;
- }
- if ((int)h % 2 != 0) {
-	h += 1.0f;
- }
- // depth doesn't have to be mod 2 == 0. we render from bottom (0.0) to
- // top (depth), not from -depth/2 to depth/2
- float depth = 1.0f;
+ size = ceilf(size);
+ glColor3ub(0, 255, 0);
+ const float unit = (float)boConfig->doubleValue("GridUnitSize");
 
- const float xyz = 0.05f; // FIXME: name
+ // the XY plane
+ int lines = (int)((2 * size) / unit);
+ // fix rounding errors...
+ if (unit * lines < 2 * size) {
+	lines++;
+ }
  glBegin(GL_LINES);
-	// bottom
-	glVertex3f(-(w / 2), -(h / 2 + xyz), 0.0f);
-	glVertex3f(-(w / 2), (h / 2 + xyz), 0.0f);
+	// AB: we dont use for (float x = -size; x <= size; x +=unit) due to
+	// rounding errors
+	for (int i = 0; i <= lines; i++) {
+		float x = -size + i * unit;
+		glVertex3f(x, -size, 0.0f);
+		glVertex3f(x, size, 0.0f);
 
-	glVertex3f(-(w / 2 + xyz), -(h / 2), 0.0f);
-	glVertex3f((w / 2 + xyz), -(h / 2), 0.0f);
-
-	glVertex3f((w / 2), -(h / 2 + xyz), 0.0f);
-	glVertex3f((w / 2), (h / 2 + xyz), 0.0f);
-
-	glVertex3f(-(w / 2 + xyz), (h / 2), 0.0f);
-	glVertex3f((w / 2 + xyz), (h / 2), 0.0f);
-
-	// top
-	glVertex3f(-(w / 2), -(h / 2 + xyz), depth);
-	glVertex3f(-(w / 2), (h / 2 + xyz), depth);
-
-	glVertex3f(-(w / 2 + xyz), -(h / 2), depth);
-	glVertex3f((w / 2 + xyz), -(h / 2), depth);
-
-	glVertex3f((w / 2), -(h / 2 + xyz), depth);
-	glVertex3f((w / 2), (h / 2 + xyz), depth);
-
-	glVertex3f(-(w / 2 + xyz), (h / 2), depth);
-	glVertex3f((w / 2 + xyz), (h / 2), depth);
-
-	const float marker = 0.025f;
-	for (float i = -(w / 2) + 1.0f; i < w / 2; i += 1.0f) {
-		glVertex3f(i, (h / 2 - marker), 0.0f);
-		glVertex3f(i, (h / 2 + marker), 0.0f);
-
-		glVertex3f(i, -(h / 2 - marker), 0.0f);
-		glVertex3f(i, -(h / 2 + marker), 0.0f);
-
-		glVertex3f(i, -(h / 2 - marker), depth);
-		glVertex3f(i, -(h / 2 + marker), depth);
-
-		glVertex3f(i, (h / 2 - marker), depth);
-		glVertex3f(i, (h / 2 + marker), depth);
+		float y = -size + i * unit;
+		glVertex3f(-size, y, 0.0f);
+		glVertex3f(size, y, 0.0f);
 	}
-	for (float i = -(h / 2) + 1.0f; i < h / 2; i += 1.0f) {
-		glVertex3f((h / 2 - marker), i, 0.0f);
-		glVertex3f((h / 2 + marker), i, 0.0f);
-
-		glVertex3f(-(h / 2 - marker), i, 0.0f);
-		glVertex3f(-(h / 2 + marker), i, 0.0f);
-
-		glVertex3f(-(h / 2 - marker), i, depth);
-		glVertex3f(-(h / 2 + marker), i, depth);
-
-		glVertex3f((h / 2 - marker), i, depth);
-		glVertex3f((h / 2 + marker), i, depth);
-	}
-
-	// bottom-to-top
-	glVertex3f((w / 2), -(h / 2), -xyz);
-	glVertex3f((w / 2), -(h / 2), depth + xyz);
-
-	glVertex3f((w / 2), (h / 2), -xyz);
-	glVertex3f((w / 2), (h / 2), depth + xyz);
-
-	glVertex3f(-(w / 2), (h / 2), -xyz);
-	glVertex3f(-(w / 2), (h / 2), depth + xyz);
-
-	glVertex3f(-(w / 2), -(h / 2), -xyz);
-	glVertex3f(-(w / 2), -(h / 2), depth + xyz);
-
-	// TODO: markers for depth, just like for w and h above.
-	// not yet important, as depth is 1.0 always
  glEnd();
 
- // mark the origin
- // (AB: maybe use a different color? if yes then use that for z-axis, too,
- // which is rendered above!
- glEnable(GL_LINE_STIPPLE);
- glLineStipple(1, 0x00FF);
- glBegin(GL_LINES);
-	glVertex3f(0.0f, -(h / 2 + xyz), 0.0f);
-	glVertex3f(0.0f, (h / 2 + xyz), 0.0f);
 
-	glVertex3f(-(w / 2 + xyz), 0.0f, 0.0f);
-	glVertex3f((w / 2 + xyz), 0.0f, 0.0f);
+ glColor3ub(255, 255, 255);
+ glPopAttrib();
 
-	glVertex3f(0.0f, -(h / 2 + xyz), depth);
-	glVertex3f(0.0f, (h / 2 + xyz), depth);
-
-	glVertex3f(-(w / 2 + xyz), 0.0f, depth);
-	glVertex3f((w / 2 + xyz), 0.0f, depth);
-
- glEnd();
- glDisable(GL_LINE_STIPPLE);
-
-#if 0
- glListBase(mDefaultFont->displayList());
-
- glRasterPos3f(0.0f, 0.0f, 0.0f);
- glTranslatef((w / 2), 0.0f, 0.0f);
- mDefaultFont->renderString(QString::fromLatin1("X-Axis"));
- glTranslatef(-(w / 2), 0.0f, 0.0f);
-
- glRasterPos3f(0.0f, 0.0f, 0.0f);
- glTranslatef(0.0f, (h / 2), 0.0f);
- mDefaultFont->renderString(QString::fromLatin1("Y-Axis"));
- glTranslatef(0.0f, -(h / 2), 0.0f);
-
- glRasterPos3f(0.0f, 0.0f, 0.0f);
- glTranslatef(0.0f, 0.0f, depth / 2);
- mDefaultFont->renderString(QString::fromLatin1("Z-Axis"));
- glTranslatef(0.0f, 0.0f, -depth / 2);
-
- glRasterPos3f(0.0f, 0.0f, 0.0f);
-#endif
- // TODO: write texts to every axis: "X-Axis", "Y-Axis", "Z-Axis". Also numbers
- // indicating the units would be nice, i.e. -w/2,-h/2,0.0 at the start of an
- // axis and w/2,h/2,depth at the end of an axis.
- // we cannot easily do that with the code above (BosonGLFont), as normal fonts
- // can't be rotated,scaled,... we need to use textured fonts.
 }
 
 void ModelPreview::renderMeshSelection()
@@ -440,6 +444,7 @@ void ModelPreview::renderMeshSelection()
  if (!matrix) {
 	return;
  }
+ glPushAttrib(GL_POLYGON_BIT | GL_ENABLE_BIT | GL_CURRENT_BIT);
  glPushMatrix();
  glMultMatrixf(matrix->data());
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
@@ -447,18 +452,17 @@ void ModelPreview::renderMeshSelection()
  glDisable(GL_TEXTURE_2D);
  mesh->renderBoundingObject();
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
- glColor3ub(0, 255, 0);
 
  if (boConfig->boolValue("ShowVertexPoints")) {
+	glColor3ub(0, 255, 0);
 	float size = (float)boConfig->uintValue("VertexPointSize", 3);
 	glPointSize(size);
 	glDisable(GL_DEPTH_TEST);
 	mesh->renderVertexPoints();
-	glEnable(GL_DEPTH_TEST);
  }
 
- glEnable(GL_TEXTURE_2D);
  glPopMatrix();
+ glPopAttrib();
  glColor3ub(255, 255, 255);
 }
 
@@ -474,7 +478,7 @@ void ModelPreview::renderText()
  glPushMatrix();
  glLoadIdentity();
 
- glListBase(mDefaultFont->displayList());
+ mDefaultFont->begin();
  const int border = 5;
  int y = height() - border;
 
@@ -584,6 +588,7 @@ void ModelPreview::setModel(BosonModel* model)
  mModel = model;
  if (mModel) {
 	emit signalMaxFramesChanged(mModel->frames() - 1);
+	emit signalMaxLODChanged(mModel->lodCount() - 1);
  }
 }
 
@@ -694,7 +699,7 @@ int ModelPreview::pickObject(const QPoint& cursor)
  for (int i = 0; i < hits; i++) {
 	unsigned int names = buffer[pos];
 	pos++;
-	unsigned int minZ = buffer[pos];
+	unsigned int minZ = buffer[pos]; // note: depth is [0;1] and multiplied by 2^32-1
 	pos++;
 //	unsigned int maxZ = buffer[pos];
 	pos++;
@@ -714,6 +719,7 @@ int ModelPreview::pickObject(const QPoint& cursor)
 //		boDebug() << k_funcinfo << m->name() << endl;
 		if (minZ < closestZ) {
 			mesh = name;
+			closestZ = minZ;
 		}
 	} else {
 		boWarning() << k_funcinfo << "no such mesh: " << name << endl;
@@ -825,6 +831,8 @@ void ModelPreview::wheelEvent(QWheelEvent* e)
 	// FIXME: this should be disallowed!
 	orientation = BoVector3(1.0f, 0.0f, 0.0f);
  }
+ // TODO: we should check whether all meshes are still in front of the NEAR plane!
+
  eye.add(orientation * -delta);
  lookAt.add(orientation * -delta);
  updateCamera(eye, lookAt, up);
@@ -858,8 +866,8 @@ void ModelPreview::slotLODChanged(int l)
 		emit signalLODChanged(0);
 		return;
 	}
-	if ((unsigned int)l + 1 > BosonModel::defaultLodCount()) {
-		emit signalLODChanged(BosonModel::defaultLodCount() - 1);
+	if ((unsigned int)l + 1 > mModel->lodCount()) {
+		emit signalLODChanged(mModel->lodCount() - 1);
 		return;
 	}
   }
@@ -903,11 +911,23 @@ void ModelPreview::hideMesh(unsigned int mesh, bool hide)
 	}
 	f->setHidden(mesh, hide);
  }
- if (mesh == mSelectedMesh) {
+ if (isSelected(mesh)) {
 	selectMesh(-1);
  }
 }
 
+bool ModelPreview::isSelected(unsigned int mesh) const
+{
+ // AB: one day we will use a list of selected meshes
+ if (mSelectedMesh < 0) {
+	return false;
+ }
+ if ((unsigned int)mSelectedMesh == mesh) {
+	return true;
+ }
+ return false;
+}
+
 void ModelPreview::slotHideSelectedMesh()
 {
  if (!haveModel()) {
@@ -982,16 +1002,27 @@ RenderMain::RenderMain()
  connect(mPreview, SIGNAL(signalCameraChanged()), mConfig, SLOT(slotCameraChanged()));
  mConfig->setCamera(mPreview->camera());
 
+ mPreview->initGL();
+ mLightWidget = new BoLightCameraWidget(0, true);
+ mLightWidget->hide();
+ mLightWidget->setLight(mPreview->light(), mPreview->context());
+
+ mMaterialWidget = new BoMaterialWidget(0);
+ mMaterialWidget->hide();
+
 
  connectBoth(mConfig, mPreview, SIGNAL(signalFovYChanged(float)), SLOT(slotFovYChanged(float)));
  connectBoth(mConfig, mPreview, SIGNAL(signalFrameChanged(int)), SLOT(slotFrameChanged(int)));
  connectBoth(mConfig, mPreview, SIGNAL(signalLODChanged(int)), SLOT(slotLODChanged(int)));
  connect(mConfig, SIGNAL(signalResetDefaults()), mPreview, SLOT(slotResetView()));
  connect(mPreview, SIGNAL(signalMaxFramesChanged(int)), mConfig, SLOT(slotMaxFramesChanged(int)));
+ connect(mPreview, SIGNAL(signalMaxLODChanged(int)), mConfig, SLOT(slotMaxLODChanged(int)));
  connect(mConfig, SIGNAL(signalPlacementPreviewChanged(bool)), mPreview, SLOT(slotPlacementPreviewChanged(bool)));
  connect(mConfig, SIGNAL(signalDisallowPlacementChanged(bool)), mPreview, SLOT(slotDisallowPlacementChanged(bool)));
  connect(mConfig, SIGNAL(signalWireFrameChanged(bool)), mPreview, SLOT(slotWireFrameChanged(bool)));
  connect(mConfig, SIGNAL(signalConstructionChanged(bool)), mPreview, SLOT(slotConstructionChanged(bool)));
+ connect(mConfig, SIGNAL(signalRenderAxisChanged(bool)), mPreview, SLOT(slotRenderAxisChanged(bool)));
+ connect(mConfig, SIGNAL(signalRenderGridChanged(bool)), mPreview, SLOT(slotRenderGridChanged(bool)));
  connect(mConfig, SIGNAL(signalHideMesh()), mPreview, SLOT(slotHideSelectedMesh()));
  connect(mConfig, SIGNAL(signalHideOthers()), mPreview, SLOT(slotHideUnSelectedMeshes()));
  connect(mConfig, SIGNAL(signalUnHideAll()), mPreview, SLOT(slotUnHideAllMeshes()));
@@ -1170,7 +1201,21 @@ void RenderMain::slotDebugModels()
 	QValueList<const UnitProperties*> prop = it.current()->allUnits();
 	QValueList<const UnitProperties*>::Iterator propIt;
 	for (propIt = prop.begin(); propIt != prop.end(); ++propIt) {
-		QString file = (*propIt)->unitPath() + SpeciesTheme::unitModelFile();
+		QStringList fileNames = SpeciesTheme::unitModelFiles();
+		bool found = false;
+		QString file;
+		for (QStringList::Iterator fit = fileNames.begin(); fit != fileNames.end(); ++fit) {
+			if (KStandardDirs::exists((*propIt)->unitPath() + *fit)) {
+				file = *fit;
+				found = true;
+				break;
+			}
+		}
+		if (!found) {
+			boError() << k_funcinfo << "Cannot find model file file for " << (*propIt)->typeId() << endl;
+			continue;
+		}
+		file = (*propIt)->unitPath() + file;
 		models->addFile(file, QString("%1/%2").arg(theme->identifier()).arg((*propIt)->name()));
 	}
 
@@ -1217,6 +1262,18 @@ void RenderMain::slotVertexPointSize()
  }
 }
 
+void RenderMain::slotGridUnitSize()
+{
+ bool ok = false;
+ double size = boConfig->doubleValue("GridUnitSize", 0.1);
+ size = QInputDialog::getDouble(i18n("Grid unit size"),
+		i18n("Grid unit size"),
+		size, 0.0, 1.0, 2, &ok, this);
+ if (ok) {
+	boConfig->setDoubleValue("GridUnitSize", size);
+ }
+}
+
 void RenderMain::slotShowVertexPoints(bool s)
 {
  boConfig->setBoolValue("ShowVertexPoints", s);
@@ -1308,12 +1365,16 @@ void RenderMain::changeUnit(SpeciesTheme* s, const UnitProperties* prop)
  BO_CHECK_NULL_RET(prop);
  // TODO: check/uncheck the menu items!
 
+ mMaterialWidget->clearMaterials();
+ mMaterialWidget->hide();
  mPreview->load(s, prop);
 }
 
 void RenderMain::changeObject(SpeciesTheme* s, const QString& objectModel)
 {
  BO_CHECK_NULL_RET(s);
+ mMaterialWidget->clearMaterials();
+ mMaterialWidget->hide();
  mPreview->loadObjectModel(s, objectModel);
 }
 
@@ -1359,14 +1420,30 @@ void RenderMain::initKAction()
  (void)new KAction(i18n("Vertex point size..."), 0, this,
 		SLOT(slotVertexPointSize()),
 		actionCollection(), "options_vertex_point_size");
+ (void)new KAction(i18n("Grid unit size..."), 0, this,
+		SLOT(slotGridUnitSize()),
+		actionCollection(), "options_grid_unit_size");
  (void)new KAction(i18n("Background color..."), 0, this,
 		SLOT(slotBackgroundColor()),
 		actionCollection(), "options_background_color");
+ (void)new KAction(i18n("Light..."), 0, this, SLOT(slotShowLightWidget()),
+		actionCollection(), "options_light"); // AB: actually this is NOT a setting
+ (void)new KAction(i18n("Materials..."), 0, this, SLOT(slotShowMaterialsWidget()),
+		actionCollection(), "options_materials"); // AB: actually this is NOT a setting
+
 
  (void)new KAction(i18n("Debug &Models"), 0, this, SLOT(slotDebugModels()),
 		actionCollection(), "debug_models");
  (void)new KAction(i18n("Debug &Species"), 0, this, SLOT(slotDebugSpecies()),
 		actionCollection(), "debug_species");
+ (void)new KAction(i18n("Show OpenGL states"), KShortcut(), this,
+		SLOT(slotShowGLStates()), actionCollection(),
+		"debug_show_opengl_states");
+ (void)new KAction(i18n("&Reload model textures"), KShortcut(), this,
+		SLOT(slotReloadModelTextures()), actionCollection(), "debug_lazy_reload_model_textures");
+ (void)new KAction(i18n("Reload &meshrenderer plugin"), KShortcut(), this,
+		SLOT(slotReloadMeshRenderer()), actionCollection(),
+		"debug_lazy_reload_meshrenderer");
 
  createGUI(locate("data", "boson/borenderui.rc"));
 }
@@ -1390,6 +1467,53 @@ void RenderMain::uncheckAllBut(KAction* action)
  }
 }
 
+void RenderMain::slotShowLightWidget()
+{
+ mLightWidget->show();
+}
+
+void RenderMain::slotShowMaterialsWidget()
+{
+ mMaterialWidget->clearMaterials();
+ if (!mPreview || !mPreview->model()) {
+	return;
+ }
+ BosonModel* model = mPreview->model();
+ for (unsigned int i = 0; i < model->materialCount(); i++) {
+	mMaterialWidget->addMaterial(model->material(i));
+ }
+ mMaterialWidget->show();
+}
+
+void RenderMain::slotShowGLStates()
+{
+ boDebug() << k_funcinfo << endl;
+ BoGLStateWidget* w = new BoGLStateWidget(0, 0, WDestructiveClose);
+ w->show();
+}
+
+void RenderMain::slotReloadMeshRenderer()
+{
+ bool unusable = false;
+ bool r = BoMeshRendererManager::manager()->reloadPlugin(&unusable);
+ if (r) {
+	return;
+ }
+ boError() << "meshrenderer reloading failed" << endl;
+ if (unusable) {
+	KMessageBox::sorry(this, i18n("Reloading meshrenderer failed, library is now unusable. quitting."));
+	exit(1);
+ } else {
+	KMessageBox::sorry(this, i18n("Reloading meshrenderer failed but library should still be usable"));
+ }
+}
+
+void RenderMain::slotReloadModelTextures()
+{
+ BO_CHECK_NULL_RET(BosonModelTextures::modelTextures());
+ BosonModelTextures::modelTextures()->reloadTextures();
+}
+
 PreviewConfig::PreviewConfig(QWidget* parent) : QWidget(parent)
 {
  bool slider = false; // FIXME: there seems to be a bug with the slider. when you slide between e.g. -1.0->0.0 or so then there seem to be recursive setValue() calls which block the UI for some seconds
@@ -1410,7 +1534,7 @@ PreviewConfig::PreviewConfig(QWidget* parent) : QWidget(parent)
 
  mLOD = new KIntNumInput(this);
  mLOD->setLabel(i18n("LOD"));
- mLOD->setRange(0, BosonModel::defaultLodCount() - 1);
+ mLOD->setRange(0, 0);
  connect(mLOD, SIGNAL(valueChanged(int)), this, SIGNAL(signalLODChanged(int)));
  topLayout->addWidget(mLOD);
  topLayout->addStretch(1);
@@ -1445,6 +1569,26 @@ PreviewConfig::PreviewConfig(QWidget* parent) : QWidget(parent)
  connect(mConstruction, SIGNAL(toggled(bool)), this, SIGNAL(signalConstructionChanged(bool)));
  topLayout->addWidget(mConstruction);
 
+ mRenderAxis = new QCheckBox(i18n("Render axis"), this);
+ connect(mRenderAxis, SIGNAL(toggled(bool)), this, SIGNAL(signalRenderAxisChanged(bool)));
+ topLayout->addWidget(mRenderAxis);
+
+ mRenderGrid = new QCheckBox(i18n("Render grid"), this);
+ connect(mRenderGrid, SIGNAL(toggled(bool)), this, SIGNAL(signalRenderGridChanged(bool)));
+ topLayout->addWidget(mRenderGrid);
+
+ mEnableLight = new QCheckBox(i18n("Enable Light"), this);
+ connect(mEnableLight, SIGNAL(toggled(bool)), this, SLOT(slotEnableLightChanged(bool)));
+ topLayout->addWidget(mEnableLight);
+ mEnableLight->setChecked(BORENDER_DEFAULT_LIGHT_ENABLED);
+ slotEnableLightChanged(mEnableLight->isChecked());
+
+ mEnableMaterials = new QCheckBox(i18n("Enable Materials"), this);
+ connect(mEnableMaterials, SIGNAL(toggled(bool)), this, SLOT(slotEnableMaterialsChanged(bool)));
+ topLayout->addWidget(mEnableMaterials);
+ mEnableMaterials->setChecked(BORENDER_DEFAULT_MATERIALS_ENABLED);
+ slotEnableMaterialsChanged(mEnableMaterials->isChecked());
+
  mCameraWidget = new BoCameraWidget(this, "bocamerawidget");
  topLayout->addWidget(mCameraWidget);
  mCameraWidget->show();
@@ -1481,6 +1625,16 @@ void PreviewConfig::slotMeshSelected(int mesh)
  }
 }
 
+void PreviewConfig::slotEnableLightChanged(bool e)
+{
+ boConfig->setUseLight(e);
+}
+
+void PreviewConfig::slotEnableMaterialsChanged(bool e)
+{
+ boConfig->setUseMaterials(e);
+}
+
 int main(int argc, char **argv)
 {
  KAboutData about("borender",
@@ -1594,6 +1748,11 @@ int main(int argc, char **argv)
 
 
  args->clear();
+
+ // the VBO meshrenderer (which isn't used by borender by default!) requires
+ // this call. otherwise we get an undefined reference on plugin loading.
+ // I have _no_ idea why this is required, as it's basically a noop.
+ (void)BoInfo::boInfo();
  return app.exec();
 }
 
