@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ class BoItemList;
 class BosonItem;
 class BoPixmapRenderer;
 class BoLight;
+class BoFontInfo;
 
 class KGameChat;
 class KGameIO;
@@ -145,6 +146,16 @@ private:
 	bool mAltButton;
 };
 
+class BoLineVisualization
+{
+  public:
+    BoLineVisualization()  { color.set(1.0f, 1.0f, 1.0f, 1.0f); timeout = 60; pointsize = 1.0f; }
+    QValueList<BoVector3> points;
+    BoVector4 color;
+    int timeout;
+    float pointsize;
+};
+
 /**
  * @author Andreas Beckermann <b_mann@gmx.de>
  **/
@@ -178,10 +189,14 @@ public:
 	 **/
 	void setActive(bool a);
 
+	void addLineVisualization(BoLineVisualization v);
+	void advanceLineVisualization();
+
 	BosonCursor* cursor() const { return mCursor; }
 	void setCursor(BosonCursor* c) { mCursor = c; }
 	BoSelection* selection() const { return mSelection; }
 
+	void setFont(const BoFontInfo& font);
 	QImage screenShot();
 
 	/**
@@ -238,8 +253,6 @@ public:
 
 	double fps() const;
 
-	void mapChanged();
-
 	Player* localPlayer() const;
 	PlayerIO* localPlayerIO() const;
 
@@ -277,6 +290,7 @@ public:
 	BoLight* newLight();
 	void removeLight(int id);
 
+	void updateOpenGLSettings();
 	void changeGroundRenderer(int renderer);
 
 	/**
@@ -310,7 +324,7 @@ public slots:
 
 	/**
 	 * In case the unit has been destroyed make sure that it's removed from
-	 * the local selection. 
+	 * the local selection.
 	 *
 	 * Currently this does not do anything else, but we might add some
 	 * functionality in the future
@@ -342,6 +356,10 @@ protected slots:
 
 	void slotMouseIODestroyed();
 
+	void slotInitMiniMapFogOfWar();
+
+	void slotAddLineVisualization(const QValueList<BoVector3>& points, const BoVector4& color, float pointSize, int timeout, float zOffset);
+
 protected:
 	/**
 	 * Here the defined action for a wheel event should happen. See
@@ -410,6 +428,12 @@ protected:
 	void createRenderItemList();
 
 	void renderItems();
+	/**
+	 * @param isFlying If TRUE, the provided z value is always used, which
+	 * is expected to be higher than any cells. Otherwise the z value of
+	 * every cell is used.
+	 **/
+	void renderPathLines(QValueList<QPoint>& path, bool isFlying = false, float z = 0.05f);
 	void renderCursor();
 	void renderSelectionRect();
 	void renderPlacementPreview();
@@ -418,16 +442,28 @@ protected:
 	 **/
 	void renderText();
 
+	void renderMiniMap();
+
 	/**
 	 * Called by @ref paintGL only to render the cells on the screen
 	 **/
 	void renderCells();
 
 	/**
-	 * Called by @ref paintGL only to render the particles on the screen
+	 * Called by @ref paintGL only to render the particle effects on the screen
 	 **/
 	void renderParticles();
 
+	/**
+	 * Called by @ref paintGL only to render the fog effects on the screen
+	 **/
+	void renderFog();
+
+	/**
+	 * Called by @ref paintGL only to render the fade effects on the screen
+	 **/
+	void renderFadeEffects();
+
 	/**
 	 * @param y The <em>top</em> of the text to-be rendered. See also @ref
 	 * BosonGLFont::renderText
