@@ -318,6 +318,8 @@ public:
 	bool deactivateWeaponSounds() const { return mDeactivateWeaponSounds->value(); }
 	void setUseLOD(bool use) { mUseLOD->setValue(use); }
 	bool useLOD() const { return mUseLOD->value(); }
+	void setUseVBO(bool use) { mUseVBO->setValue(use); }
+	bool useVBO() const { return mUseVBO->value(); }
 
 	void setUnitSoundActivated(UnitSoundEvent e, bool activated);
 	bool unitSoundActivated(UnitSoundEvent e) const;
@@ -501,6 +503,9 @@ public:
 	void setDebugMapCoordinates(bool debug) { mDebugMapCoordinates = debug; }
 	bool debugMapCoordinates() const { return mDebugMapCoordinates; }
 
+	void setDebugPFData(bool debug) { mDebugPFData = debug; }
+	bool debugPFData() const { return mDebugPFData; }
+
 	void setDebugShowCellGrid(bool debug) { mDebugShowCellGrid = debug; }
 	bool debugShowCellGrid() const { return mDebugShowCellGrid; }
 
@@ -516,12 +521,18 @@ public:
 	void setDebugBoundingBoxes(bool debug) { mDebugBoundingBoxes = debug; }
 	bool debugBoundingBoxes() const { return mDebugBoundingBoxes; }
 
+	unsigned int defaultLodCount() const { return mDefaultLodCount; }
+	void setDefaultLodCount(unsigned int c) { mDefaultLodCount = c; }
+
 	void setDebugFPS(bool debug) { mDebugFPS = debug; }
 	bool debugFPS() const { return mDebugFPS; }
 
 	void setShowResources(bool show) { mShowResources = show; }
 	bool showResources() const { return mShowResources; }
 
+	void setEnableColormap(bool enable) { mEnableColormap = enable; }
+	bool enableColormap() const { return mEnableColormap; }
+
 public:
 	void save(bool editor = false, KConfig* conf = 0);
 	void reset(KConfig* conf = 0);
@@ -579,6 +590,7 @@ private:
 	BoConfigIntEntry* mToolTipCreator;
 	BoConfigIntEntry* mGameLogInterval;
 	BoConfigBoolEntry* mUseLOD;
+	BoConfigBoolEntry* mUseVBO;
 
 	// NOT stored to config file!
 	bool mDisableSound;
@@ -588,6 +600,7 @@ private:
 	bool mWireFrames;
 	bool mDebugOpenGLMatrices;
 	bool mDebugMapCoordinates;
+	bool mDebugPFData;
 	bool mDebugShowCellGrid;
 	bool mDebugItemWorkStatistics;
 	bool mDebugOpenGLCamera;
@@ -595,6 +608,8 @@ private:
 	bool mDebugBoundingBoxes;
 	bool mDebugFPS;
 	bool mShowResources;
+	bool mEnableColormap;
+	unsigned int mDefaultLodCount;
 };
 
 #endif
