@@ -28,17 +28,10 @@
 #include "bosondata.h"
 #include "bosontexturearray.h"
 
-#include <qtimer.h>
-#include <qdatetime.h>
 #include <qdatastream.h>
 #include <qimage.h>
-#include <qvaluevector.h>
 #include <qdom.h>
 
-#include <kapplication.h>
-#include <kglobal.h>
-#include <kstandarddirs.h>
-
 #define BOSONMAP_VERSION 0x01 // current version
 
 
@@ -261,9 +254,8 @@ bool BoTexMap::importTexMap(const QImage* img)
  boDebug() << k_funcinfo << endl;
 
  for (unsigned int y = 0; y < height(); y++) {
-	QRgb* line = (QRgb*)img->scanLine(y);
 	for (unsigned int x = 0; x < width(); x++) {
-		QRgb pixel = line[x];
+		QRgb pixel = img->pixel(x, y);
 		setTexMapAlpha(0, x, y, qRed(pixel));
 		setTexMapAlpha(1, x, y, qGreen(pixel));
 		setTexMapAlpha(2, x, y, qBlue(pixel));
@@ -271,9 +263,8 @@ bool BoTexMap::importTexMap(const QImage* img)
  }
  if (useAlpha) {
 	for (unsigned int y = 0; y < height(); y++) {
-		QRgb* line = (QRgb*)img->scanLine(y);
 		for (unsigned int x = 0; x < width(); x++) {
-			QRgb pixel = line[x];
+			QRgb pixel = img->pixel(x, y);
 			setTexMapAlpha(3, x, y, qAlpha(pixel));
 		}
 	}
@@ -325,6 +316,7 @@ BosonMap::BosonMap(QObject* parent) : QObject(parent)
 BosonMap::~BosonMap()
 {
  delete[] mCells;
+ delete mColorMap;
  delete mNormalMap;
  delete mHeightMap;
  delete mTexMap;
@@ -335,6 +327,7 @@ void BosonMap::init()
 {
  d = new BosonMapPrivate;
  mCells = 0;
+ mColorMap = 0;
  mHeightMap = 0;
  mNormalMap = 0;
  mGroundTheme = 0;
@@ -360,6 +353,7 @@ bool BosonMap::createNewMap(unsigned int width, unsigned int height, BosonGround
 
  mHeightMap = new BoHeightMap(width + 1, height + 1);
  mTexMap = new BoTexMap(theme->textureCount(), width + 1, height + 1);
+ mTexMap->fill(0);
  bool ret = generateCellsFromTexMap();
  if (!ret) {
 	boError() << k_funcinfo << "unable to generate cells from texmap" << endl;
@@ -396,69 +390,91 @@ bool BosonMap::createNewMap(unsigned int width, unsigned int height, BosonGround
 	return ret;
  }
 
+ // check whether map was loaded successfully
+ if (!mGroundTheme || !mCells || !mHeightMap || !mTexMap) {
+	boError() << k_funcinfo << "map was not loaded correctly" << endl;
+	return false;
+ }
+ if (mGroundTheme != theme) {
+	boError() << k_funcinfo << "incorrect theme loaded: " << mGroundTheme << " != " << theme << endl;
+	return false;
+ }
+ if (this->width() != width) {
+	boError() << k_funcinfo << "invalid width loaded" << endl;
+	return false;
+ }
+ if (this->height() != height) {
+	boError() << k_funcinfo << "invalid height loaded" << endl;
+	return false;
+ }
+ if (mTexMap->textureCount() != mGroundTheme->textureCount()) {
+	boError() << k_funcinfo << "texmap has invalid texture count for this groundtheme!" << endl;
+	return false;
+ }
+
  return ret;
 }
 
 bool BosonMap::loadMapFromFile(const QByteArray& mapXML)
 {
- boDebug() << k_funcinfo << endl;
+ boDebug(270) << k_funcinfo << endl;
  QDomDocument doc(QString::fromLatin1("BosonMap"));
  QString errorMsg;
  int line, column;
  if (!doc.setContent(QString(mapXML), &errorMsg, &line, &column)) {
-	boError() << k_funcinfo << "unable to load from mapXML (error in line " << line << ", column " << column << ", msg=" << errorMsg << ")" << endl;
+	boError(270) << k_funcinfo << "unable to load from mapXML (error in line " << line << ", column " << column << ", msg=" << errorMsg << ")" << endl;
 	return false;
  }
  QDomElement root = doc.documentElement();
  if (root.isNull()) {
-	boError() << k_funcinfo << "no root element" << endl;
+	boError(270) << k_funcinfo << "no root element" << endl;
 	return false;
  }
  if (!root.hasAttribute("Version")) {
-	boError() << k_funcinfo << "missing attribute: Version" << endl;
+	boError(270) << k_funcinfo << "missing attribute: Version" << endl;
 	return false;
  }
  int version = root.attribute("Version").toInt();
  if (version != BOSONMAP_VERSION) {
-	boError() << k_funcinfo << "invalid version " << version << " - need: " << BOSONMAP_VERSION << endl;
+	boError(270) << k_funcinfo << "invalid version " << version << " - need: " << BOSONMAP_VERSION << endl;
 	return false;
  }
  if (!root.hasAttribute(QString::fromLatin1("GroundTheme"))) {
-	boError() << k_funcinfo << "no GroundTheme attribute in root element" << endl;
+	boError(270) << k_funcinfo << "no GroundTheme attribute in root element" << endl;
 	return false;
  }
  QDomElement geometry = root.namedItem(QString::fromLatin1("Geometry")).toElement();
  if (geometry.isNull()) {
-	boError() << k_funcinfo << "no geometry element" << endl;
+	boError(270) << k_funcinfo << "no geometry element" << endl;
 	return false;
  }
  if (!geometry.hasAttribute(QString::fromLatin1("Width"))) {
-	boError() << k_funcinfo << "no Width attribute in Geometry element" << endl;
+	boError(270) << k_funcinfo << "no Width attribute in Geometry element" << endl;
 	return false;
  }
  if (!geometry.hasAttribute(QString::fromLatin1("Height"))) {
-	boError() << k_funcinfo << "no Height attribute in Geometry element" << endl;
+	boError(270) << k_funcinfo << "no Height attribute in Geometry element" << endl;
 	return false;
  }
  bool ok = false;
  QString groundTheme = root.attribute(QString::fromLatin1("GroundTheme"));
  int width = geometry.attribute(QString::fromLatin1("Width")).toInt(&ok);
  if (!ok || width < 0) {
-	boError() << k_funcinfo << "width is not a valid number" << endl;
+	boError(270) << k_funcinfo << "width is not a valid number" << endl;
 	return false;
  }
  int height = geometry.attribute(QString::fromLatin1("Height")).toInt(&ok);
  if (!ok || height < 0) {
-	boError() << k_funcinfo << "height is not a valid number" << endl;
+	boError(270) << k_funcinfo << "height is not a valid number" << endl;
 	return false;
  }
 
  if (!loadMapGeo((unsigned int)width, (unsigned int)height)) {
-	boError() << k_funcinfo << "Could not load map geo" << endl;
+	boError(270) << k_funcinfo << "Could not load map geo" << endl;
 	return false;
  }
  if (!loadGroundTheme(groundTheme)) {
-	boError() << k_funcinfo << "Could not load the ground theme" << endl;
+	boError(270) << k_funcinfo << "Could not load the ground theme" << endl;
 	return false;
  }
  return true;
@@ -492,6 +508,10 @@ bool BosonMap::loadCompleteMap(QDataStream& stream)
 	boError() << k_funcinfo << "Could not load map cells" << endl;
 	return false;
  }
+ // should this be done somewhere else?
+ boDebug() << k_funcinfo << "creating colormap" << endl;
+ createColorMap();
+ boDebug() << k_funcinfo << "created colormap" << endl;
  return true;
 }
 
@@ -513,6 +533,9 @@ bool BosonMap::loadMapGeo(unsigned int width, unsigned int height)
  mHeightMap = 0;
  delete mNormalMap;
  mNormalMap = 0;
+ boDebug() << k_funcinfo << "deleting colormap" << endl;
+ delete mColorMap;
+ mColorMap = 0;
  delete mTexMap;
  mTexMap = 0;
 
@@ -600,6 +623,10 @@ bool BosonMap::loadHeightMapImage(const QByteArray& heightMapBuffer)
  }
  boDebug() << k_funcinfo << "loading real height map" << endl;
  QImage map(heightMapBuffer);
+ if (map.isNull()) {
+	boError() << k_funcinfo << "received an invalid image buffer - null image" << endl;
+	return false;
+ }
  if (!map.isGrayscale()) {
 	// we load a valid height map (i.e. a dummy height map) but still return
 	// false for error checking.
@@ -619,30 +646,16 @@ bool BosonMap::loadHeightMapImage(const QByteArray& heightMapBuffer)
 	loadHeightMapImage(QByteArray());
 	return false;
  }
- int increment = 1;
- if (map.bytesPerLine() > map.width() + 1) {
-	// QT doesn't save images as grayscale. it returns bytesPerLine() ==
-	// width() (i.e. 8bits per pixel) for all *actual* grayscale images
-	// and bytesPerLine() == width() * 4 for grayscale images that Qt
-	// has created (i.e. RGB/RGBA images). we only use grayscale here, that
-	// means red=blue=green component. so we can just pick the first and
-	// skip all (including alpha) other values.
-	increment = 4;
- }
-
 
  BoHeightMap heightMap(width() + 1, height() + 1);
  for (unsigned int y = 0; y < height() + 1; y++) {
 	// AB: warning: from Qt docs: "If you are accessing 16-bpp image data,
 	// you must handle endianness yourself."
 	// do we have to care about this? (since we are using 16bpp)
-	// AB: we use 32bpp (qt doesnt support 16bpp on X11)
-	// AB: hmm accordint to "file" we use 6bpp only... ok thats easier
-	// then :)
-	int imageX = 0;
-	unsigned char* line = map.scanLine(y);
-	for (unsigned int x = 0; x < width() + 1; x++, imageX += increment) {
-		heightMap.setHeightAt(x, y, BoHeightMap::pixelToHeight(line[imageX]));
+	// AB: the comment about endianness might be obsolete here as it applies
+	// to scanLine(), but we use pixel() instead now!
+	for (unsigned int x = 0; x < width() + 1; x++) {
+		heightMap.setHeightAt(x, y, BoHeightMap::pixelToHeight(qRed(map.pixel(x, y))));
 	}
  }
  QByteArray buffer;
@@ -730,7 +743,6 @@ QByteArray BosonMap::saveMapToFile()
 	return false;
  }
 
- // TODO: add a version attribute?
  QDomDocument doc(QString::fromLatin1("BosonMap"));
  QDomElement root = doc.createElement(QString::fromLatin1("BosonMap"));
  root.setAttribute(QString::fromLatin1("Version"), BOSONMAP_VERSION);
@@ -821,7 +833,7 @@ QByteArray BosonMap::saveHeightMapImage()
 	boDebug() << k_funcinfo << "real height map" << endl;
 	// AB: this *might* be correct, but i am not sure about this. (02/11/22)
 	for (int y = 0; y < image.height(); y++) {
-		uint* p = (uint*)image.scanLine(y);
+		uint* p = (uint*)image.scanLine(y); // AB: maybe use setPixel() instead, due to endianness
 		for (int x = 0; x < image.width(); x++) {
 			float value = mHeightMap->heightAt(x, y);
 			int v = BoHeightMap::heightToPixel(value);
@@ -866,7 +878,7 @@ QByteArray BosonMap::saveTexMapImage(unsigned int texture)
 	boDebug() << k_funcinfo << "real texmap" << endl;
 	// AB: this *might* be correct, but i am not sure about this. (02/11/22)
 	for (int y = 0; y < image.height(); y++) {
-		uint* p = (uint*)image.scanLine(y);
+		uint* p = (uint*)image.scanLine(y); // AB: maybe use setPixel() instead, due to endianness
 		for (int x = 0; x < image.width(); x++) {
 			int v = (int)texMapAlpha(texture, x, y);
 			*p = qRgb(v, v, v);
@@ -1043,7 +1055,7 @@ void BosonMap::fill(unsigned int texture)
  mTexMap->fill(texture);
 }
 
-float BosonMap::cellAverageHeight(int x, int y)
+float BosonMap::cellAverageHeight(int x, int y) const
 {
  Cell* c = cell(x, y);
  if (!c) {
@@ -1184,11 +1196,11 @@ void BosonMap::slotChangeTexMap(int x, int y, unsigned int texCount, unsigned in
 	return;
  }
  BO_CHECK_NULL_RET(groundTheme());
- if (x < 0 || (uint)x >= width()) {
+ if (x < 0 || (uint)x > width()) {
 	boError() << k_funcinfo << "invalid x coordinate: " << x << endl;
 	return;
  }
- if (y < 0 || (uint)y >= height()) {
+ if (y < 0 || (uint)y > height()) {
 	boError() << k_funcinfo << "invalid y coordinate: " << y << endl;
 	return;
  }
@@ -1376,6 +1388,9 @@ void BosonMap::recalculateNormalsInRect(int x1, int y1, int x2, int y2)
 		}
 		n.scale(1.0f / count);
 		n.normalize();
+		// Cells will be drawn with y-coordinate reversed, so to make lighting look
+		//  correct, we need to reverse normal's y-component here as well.
+		n.setY(-n.y());
 		mNormalMap->setNormalAt(x, y, n);
 	}
  }
@@ -1390,3 +1405,28 @@ int BosonMap::mapFileFormatVersion()
  return BOSONMAP_VERSION;
 }
 
+void BosonMap::createColorMap()
+{
+ boDebug() << k_funcinfo << "" << endl;
+ if (mColorMap) {
+	// Old normal map should already be deleted
+	boWarning() << k_funcinfo << "Old color map not deleted!" << endl;
+	delete mColorMap;
+	mColorMap = 0;
+ }
+
+ boDebug() << k_funcinfo << "creating map" << endl;
+ mColorMap = new BoColorMap(width(), height());
+
+ boDebug() << k_funcinfo << "initing map" << endl;
+ // Init colormap to gray
+ unsigned char color[] = { 128, 128, 128 };
+ for (unsigned int y = 0; y < height(); y++) {
+	for (unsigned int x = 0; x < width(); x++) {
+		mColorMap->setColorAt(x, y, color);
+	}
+ }
+ boDebug() << k_funcinfo << "done" << endl;
+}
+
+
