@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -135,18 +135,6 @@ void BosonSaveLoad::initBoson()
 		d->mBoson, SIGNAL(signalLoadExternalStuffFromXML(const QDomElement&)));
  connect(this, SIGNAL(signalSaveExternalStuffAsXML(QDomElement&)),
 		d->mBoson, SIGNAL(signalSaveExternalStuffAsXML(QDomElement&)));
-
-#if 0
- connect(this, SIGNAL(signalLoadingPlayersCount(int)),
-		d->mBoson, SIGNAL(signalLoadingPlayersCount(int)));
- connect(this, SIGNAL(signalLoadingPlayer(int)),
-		d->mBoson, SIGNAL(signalLoadingPlayer(int)));
- connect(this, SIGNAL(signalLoadingType(int)),
-		d->mBoson, SIGNAL(signalLoadingType(int)));
-
- connect(this, SIGNAL(signalLoadPlayerData(Player*)),
-		d->mBoson, SIGNAL(signalLoadPlayerData(Player*)));
-#endif
 }
 
 void BosonSaveLoad::systemAddPlayer(KPlayer* p)
@@ -222,7 +210,7 @@ bool BosonSaveLoad::loadXMLDoc(QDomDocument* doc, const QString& xml)
  return true;
 }
 
-bool BosonSaveLoad::saveToFiles(QMap<QString, QByteArray>& files, Player* localPlayer)
+bool BosonSaveLoad::saveToFiles(QMap<QString, QByteArray>& files)
 {
  boDebug() << k_funcinfo << endl;
  if (!files.isEmpty()) {
@@ -250,7 +238,7 @@ bool BosonSaveLoad::saveToFiles(QMap<QString, QByteArray>& files, Player* localP
  }
 
  boProfiling->start(BosonProfiling::SavePlayersToXML);
- QByteArray playersXML = savePlayersAsXML(localPlayer);
+ QByteArray playersXML = savePlayersAsXML();
  boProfiling->stop(BosonProfiling::SavePlayersToXML);
  if (playersXML.isNull()) {
 	return false;
@@ -278,10 +266,10 @@ bool BosonSaveLoad::saveToFiles(QMap<QString, QByteArray>& files, Player* localP
  return true;
 }
 
-bool BosonSaveLoad::savePlayFieldToFiles(QMap<QString, QByteArray>& files, Player* localPlayer)
+bool BosonSaveLoad::savePlayFieldToFiles(QMap<QString, QByteArray>& files)
 {
  // first we save a complete game.
- bool ret = saveToFiles(files, localPlayer);
+ bool ret = saveToFiles(files);
  if (!ret) {
 	boError() << k_funcinfo << "saving failed" << endl;
 	return ret;
@@ -290,10 +278,10 @@ bool BosonSaveLoad::savePlayFieldToFiles(QMap<QString, QByteArray>& files, Playe
  return ret;
 }
 
-bool BosonSaveLoad::saveToFile(Player* localPlayer, const QString& file)
+bool BosonSaveLoad::saveToFile(const QString& file)
 {
  QMap<QString, QByteArray> files;
- if (!saveToFiles(files, localPlayer)) {
+ if (!saveToFiles(files)) {
 	boError() << k_funcinfo << "saving failed" << endl;
 	return false;
  }
@@ -347,7 +335,7 @@ bool BosonSaveLoad::saveToFile(const QMap<QString, QByteArray>& files, const QSt
 	boError() << k_funcinfo << "no descriptionXML found" << endl;
 	return false;
  }
- BSGFile f(file, false);
+ BPFFile f(file, false);
  if (!f.writeFile(QString::fromLatin1("kgame.xml"), QString(kgameXML))) {
 	boError() << k_funcinfo << "Could not write kgame.xml to " << file << endl;
 	return false;
@@ -408,7 +396,7 @@ QCString BosonSaveLoad::saveKGameAsXML()
  return doc.toCString();
 }
 
-QCString BosonSaveLoad::savePlayersAsXML(Player* localPlayer)
+QCString BosonSaveLoad::savePlayersAsXML()
 {
  QDomDocument doc(QString::fromLatin1("Players"));
  QDomElement root = doc.createElement(QString::fromLatin1("Players"));
@@ -416,7 +404,7 @@ QCString BosonSaveLoad::savePlayersAsXML(Player* localPlayer)
 
  if (!d->mBoson) {
 	BO_NULL_ERROR(d->mBoson);
-	return doc.toCString();
+	return QCString();
  }
 
  KGame::KGamePlayerList* list = d->mBoson->playerList();
@@ -428,22 +416,11 @@ QCString BosonSaveLoad::savePlayersAsXML(Player* localPlayer)
 	QDomElement element = doc.createElement(QString::fromLatin1("Player"));
 	if (!((Player*)p)->saveAsXML(element)) {
 		boError() << k_funcinfo << "Unable to save player " << p->id() << endl;
-		continue;
+		return QCString();
 	}
 	root.appendChild(element);
  }
 
- // save the ID of the local player
- if (localPlayer) {
-	root.setAttribute(QString::fromLatin1("LocalPlayerId"), (unsigned int)localPlayer->id());
- } else {
-	// this might be the case for network games.
-	// (when a player enters the game it is saved on the ADMIN and loaded on
-	// the new client).
-	// But currently we don't use XML for that. Maybe we will never do.
-	boWarning() << k_funcinfo << "NULL local player" << endl;
- }
-
  return doc.toCString();
 }
 
@@ -479,17 +456,17 @@ QCString BosonSaveLoad::saveExternalAsXML()
 bool BosonSaveLoad::loadNewGame(const QByteArray& playersXML, const QByteArray& canvasXML)
 {
  // AB: nearly all code copied from loadFromFile().
- boDebug() << k_funcinfo << endl;
+ boDebug(270) << k_funcinfo << endl;
  d->mLoadingStatus = LoadingInProgress;
 
   if (playersXML.isEmpty()) {
-	boError() << k_funcinfo << "Empty playersXML" << endl;
+	boError(270) << k_funcinfo << "Empty playersXML" << endl;
 	addLoadError(SaveLoadError::LoadBSGFileError, i18n("empty file: players.xml"));
 	d->mLoadingStatus = BSGFileError;
 	return false;
  }
  if (canvasXML.isEmpty()) {
-	boError() << k_funcinfo << "Empty canvasXML" << endl;
+	boError(270) << k_funcinfo << "Empty canvasXML" << endl;
 	addLoadError(SaveLoadError::LoadBSGFileError, i18n("empty file: canvas.xml"));
 	d->mLoadingStatus = BSGFileError;
 	return false;
@@ -498,22 +475,9 @@ bool BosonSaveLoad::loadNewGame(const QByteArray& playersXML, const QByteArray&
  if (!loadPlayersFromXML(playersXML)) {
 	return false;
  }
- boDebug() << k_funcinfo << d->mBoson->playerCount() << " players loaded" << endl;
- emit signalLoadingType(BosonLoadingWidget::ReceiveMap);
+ boDebug(270) << k_funcinfo << d->mBoson->playerCount() << " players loaded" << endl;
 
-
- // Load player data (e.g. unit configs, unit models, ...
- // TODO: since we use XML for savegames now we should be able to include this
- // to regular startup code in BosonStarting!
- KPlayer* p;
- int current = 0;
- for (p = d->mBoson->playerList()->first(); p; p = d->mBoson->playerList()->next(), current++) {
-	emit signalLoadingPlayer(current);
-	emit signalLoadPlayerData((Player*)p);
-	emit signalLoadingType(BosonLoadingWidget::LoadSavedUnits);
- }
-
- boDebug() << k_funcinfo << "loading units" << endl;
+ boDebug(270) << k_funcinfo << "loading units" << endl;
 
  // Load canvas (items - i.e. units and shots)
  if (!loadCanvasFromXML(canvasXML)) {
@@ -524,71 +488,9 @@ bool BosonSaveLoad::loadNewGame(const QByteArray& playersXML, const QByteArray&
  return true;
 }
 
-#if 0
-bool BosonSaveLoad::loadVersionFromXML(const QString& xml)
-{
- // AB: this takes a kgame.xml. maybe we should split that file up into
- // kgame.xml and version.xml?
- boDebug() << k_funcinfo << endl;
- QDomDocument doc(QString::fromLatin1("Boson"));
- if (!loadXMLDoc(&doc, xml)) {
-	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Parsing error in kgame.xml"));
-	d->mLoadingStatus = InvalidXML;
-	return false;
- }
- QDomElement root = doc.documentElement();
- bool ok = false;
- unsigned int version = root.attribute(QString::fromLatin1("Version")).toUInt(&ok);
- if (!ok) {
-	boError() << k_funcinfo << "savegame version not a valid number: " << root.attribute(QString::fromLatin1("Version")) << endl;
-	addLoadError(SaveLoadError::LoadInvalidXML, i18n("savegame version in kgame.xml is not a valid number"));
-	d->mLoadingStatus = InvalidXML;
-	return false;
- }
- if (version != latestSavegameVersion()) {
-	boError() << "savegame version " << version << " not supported (latest is " << latestSavegameVersion() << ")" << endl;
-	addLoadError(SaveLoadError::LoadInvalidXML, i18n("savegame version %1 is not supported").arg(version));
-	d->mLoadingStatus = InvalidVersion;
-	return false;
- }
- return true;
-}
-#endif
-
-#if 0
-bool BosonSaveLoad::loadKGameFromXML(const QString& xml)
-{
- boDebug() << k_funcinfo << endl;
- QDomDocument doc(QString::fromLatin1("Boson"));
- if (!loadXMLDoc(&doc, xml)) {
-	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Parsing error in kgame.xml"));
-	d->mLoadingStatus = InvalidXML;
-	return false;
- }
- QDomElement root = doc.documentElement();
-
- // load the datahandler
- QDomElement handler = root.namedItem(QString::fromLatin1("DataHandler")).toElement();
- if (handler.isNull()) {
-	boError() << k_funcinfo << "No DataHandler tag found" << endl;
-	addLoadError(SaveLoadError::LoadInvalidXML, i18n("No DataHandler tag found in kgame.xml"));
-	d->mLoadingStatus = InvalidXML;
-	return false;
- }
- BosonCustomPropertyXML propertyXML;
- propertyXML.removeProperty(handler, KGamePropertyBase::IdGameStatus); // just in case - should not be there at all
- if (!propertyXML.loadFromXML(handler, d->mBoson->dataHandler())) {
-	boError() << k_funcinfo << "unable to load KGame data handler" << endl;
-	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Error while loading DataHandler of kgame.xml"));
-	d->mLoadingStatus = InvalidXML;
-	return false;
- }
- return true;
-}
-#endif
-
 bool BosonSaveLoad::loadPlayersFromXML(const QString& playersXML)
 {
+ boDebug(270) << k_funcinfo << endl;
  QDomDocument doc(QString::fromLatin1("Players"));
  if (!loadXMLDoc(&doc, playersXML)) {
 	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Parsing error in players.xml"));
@@ -596,36 +498,22 @@ bool BosonSaveLoad::loadPlayersFromXML(const QString& playersXML)
 	return false;
  }
  QDomElement root = doc.documentElement();
- if (!root.hasAttribute(QString::fromLatin1("LocalPlayerId"))) {
-	boError(260) << k_funcinfo << "missing attribute: LocalPlayerId" << endl;
-	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Missing attribute LocalPlayerId in players.xml"));
-	d->mLoadingStatus = InvalidXML;
-	return false;
- }
- bool ok = false;
- unsigned int localId = root.attribute(QString::fromLatin1("LocalPlayerId")).toUInt(&ok);
- if (!ok) {
-	boError(260) << k_funcinfo << "invalid LocalPlayerId: " << root.attribute(QString::fromLatin1("LocalPlayerId")) << endl;
-	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Invalid value for LocalPlayerId in players.xml"));
-	d->mLoadingStatus = InvalidXML;
-	return false;
- }
  QDomNodeList list = root.elementsByTagName(QString::fromLatin1("Player"));
  if (list.count() < 1) {
-	boError(260) << k_funcinfo << "no Player tags in file" << endl;
+	boError(270) << k_funcinfo << "no Player tags in file" << endl;
 	addLoadError(SaveLoadError::LoadInvalidXML, i18n("No Player Tag in players.xml"));
 	d->mLoadingStatus = InvalidXML;
 	return false;
  }
- Player* localPlayer = 0;
  for (unsigned int i = 0; i < d->mBoson->playerCount(); i++) {
 	Player* p = (Player*)d->mBoson->playerList()->at(i);
 	QDomElement player;
 	for (unsigned int j = 0; j < list.count() && player.isNull(); j++) {
 		QDomElement e = list.item(j).toElement();
+		bool ok = false;
 		unsigned int id = e.attribute(QString::fromLatin1("Id")).toUInt(&ok);
 		if (!ok) {
-			boError() << k_funcinfo << "missing or invalid Id attribute for Player tag " << j << endl;
+			boError(270) << k_funcinfo << "missing or invalid Id attribute for Player tag " << j << endl;
 			continue;
 		}
 		if (p->id() != id) {
@@ -634,26 +522,32 @@ bool BosonSaveLoad::loadPlayersFromXML(const QString& playersXML)
 		player = e;
 	}
 	if (player.isNull()) {
-		boError() << k_funcinfo << "no Player tag found for player with id " << p->id() << endl;
+		boError(270) << k_funcinfo << "no Player tag found for player with id " << p->id() << endl;
 		return false;
 	}
-	if (p->id() == localId) {
-		localPlayer = p;
+	if (i == d->mBoson->playerList()->count() - 1) {
+		boDebug(270) << k_funcinfo << "loading neutral player" << endl;
+		if (!player.hasAttribute("IsNeutral")) {
+			boError(270) << k_funcinfo << "file format error: missing IsNeutral attribute for neutral player" << endl;
+			return false;
+		}
+		bool ok = false;
+		if (player.attribute("IsNeutral").toUInt(&ok) != 1) {
+			boError(270) << k_funcinfo << "IsNeutral attribute must be 1, if present!" << endl;
+			return false;
+		} else if (!ok) {
+			boError(270) << k_funcinfo << "invalid IsNeutral attribute (must be 1)!" << endl;
+			return false;
+		}
 	}
 	p->loadFromXML(player);
  }
- if (!localPlayer) {
-	boWarning(260) << k_funcinfo << "local player NOT found" << endl;
-	addLoadError(SaveLoadError::LoadPlayersError, i18n("No local player found"));
- }
-// d->mBoson->setLocalPlayer(localPlayer);// AB: do we need this?
-
  return true;
 }
 
 bool BosonSaveLoad::loadCanvasFromXML(const QString& xml)
 {
- boDebug() << k_funcinfo << endl;
+ boDebug(270) << k_funcinfo << endl;
  QDomDocument doc(QString::fromLatin1("Canvas"));
  if (!loadXMLDoc(&doc, xml)) {
 	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Parsing error in canvas.xml"));
@@ -673,7 +567,7 @@ bool BosonSaveLoad::loadCanvasFromXML(const QString& xml)
 
 bool BosonSaveLoad::loadExternalFromXML(const QString& xml)
 {
- boDebug() << k_funcinfo << endl;
+ boDebug(260) << k_funcinfo << endl;
  // Load external stuff (camera)
  QDomDocument doc(QString::fromLatin1("External"));
  if (!loadXMLDoc(&doc, xml)) {
@@ -704,7 +598,6 @@ bool BosonSaveLoad::convertSaveGameToPlayField(QMap<QString, QByteArray>& files)
 	return false;
  }
  QDomElement playersRoot = playersDoc.documentElement();
- playersRoot.removeAttribute("LocalPlayerId");
  QDomNodeList playerList = playersRoot.elementsByTagName("Player");
  if (playerList.count() == 0) {
 	boError() << k_funcinfo << "no players in game" << endl;
