@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,17 +18,17 @@
 */
 
 #include "bosonwidgetbase.h"
+#include "bosonwidgetbase.moc"
 
 #include "defines.h"
-#include "bosonminimap.h"
 #include "bosoncanvas.h"
 #include "boson.h"
+#include "bosonsaveload.h"
 #include "player.h"
 #include "unit.h"
 #include "bosonmessage.h"
 #include "bosonplayfield.h"
 #include "bosonmap.h"
-#include "bosonscenario.h"
 #include "bosonconfig.h"
 #include "bosoncursor.h"
 #include "bodisplaymanager.h"
@@ -37,12 +37,21 @@
 #include "editorbigdisplayinput.h"
 #include "boselection.h"
 #include "global.h"
-#include "top.h"
 #include "bodebug.h"
 #include "bosonprofiling.h"
 #include "optionsdialog.h"
-#include "commandframe/bosoncommandframe.h"
-#include "sound/bosonmusic.h"
+#include "boaction.h"
+#include "bosonlocalplayerinput.h"
+#include "bosoncomputerio.h"
+#include "bosonmodeltextures.h"
+#include "commandframe/bosoncommandframebase.h"
+#include "sound/bosonaudiointerface.h"
+#include "script/bosonscript.h"
+#include "bosonwidgets/bogamechat.h"
+#include "bosonpath.h"
+#include "bomeshrenderermanager.h"
+#include "bogroundrenderermanager.h"
+#include "boglstatewidget.h"
 
 #include <kapplication.h>
 #include <klocale.h>
@@ -51,7 +60,9 @@
 #include <kpopupmenu.h>
 #include <kgame/kgamedebugdialog.h>
 #include <kgame/kgamepropertyhandler.h>
-#include <kgame/kgamechat.h>
+#include <klineedit.h>
+#include <kdockwidget.h>
+#include <kmessagebox.h>
 
 #include <qlayout.h>
 #include <qptrlist.h>
@@ -62,8 +73,9 @@
 #include <qfileinfo.h>
 #include <qdatastream.h>
 #include <qdom.h>
+#include <qimage.h>
 
-#include "bosonwidgetbase.moc"
+#include <stdlib.h>
 
 #define ID_DEBUG_KILLPLAYER 0
 #define ID_DEBUG_ADD_10000_MINERALS 1
@@ -79,17 +91,20 @@ public:
 	BosonWidgetBasePrivate()
 	{
 		mCommandFrame = 0;
-		mCommandFrameDock = 0;
-
 		mChat = 0;
-		mChatDock = 0;
+
+		mActionDebugPlayers = 0;
+		mActionZoom = 0;
+		mActionChat = 0;
+		mActionCmdFrame = 0;
+
+		mScript = 0;
+
+		mCanvas = 0;
 	}
 
 	BosonCommandFrameBase* mCommandFrame;
-	KDockWidget* mCommandFrameDock;
-
-	KGameChat* mChat;
-	KDockWidget* mChatDock;
+	BoGameChatWidget* mChat;
 
 	KActionMenu* mActionDebugPlayers;
 	KSelectAction* mActionZoom;
@@ -99,37 +114,40 @@ public:
 	QPtrDict<KPlayer> mPlayers; // needed for debug only
 
 	bool mInitialized;
+
+	BosonScript* mScript;
+
+	BosonCanvas* mCanvas;
 };
 
-BosonWidgetBase::BosonWidgetBase(TopWidget* top, QWidget* parent)
-    : QWidget( parent, "BosonWidgetBase" ), KXMLGUIClient(/*FIXME: clientParent!*/)
+BosonWidgetBase::BosonWidgetBase(QWidget* parent)
+    : QWidget( parent, "BosonWidgetBase" )
 {
  d = new BosonWidgetBasePrivate;
  d->mInitialized = false;
- mTop = top;
 
- mMiniMap = 0;
  mDisplayManager = 0;
  mCursor = 0;
  mLocalPlayer = 0;
-
 }
 
 BosonWidgetBase::~BosonWidgetBase()
 {
  boDebug() << k_funcinfo << endl;
+ if (factory()) {
+	// remove the bosonwidget-specific menus from the XML GUI (menubar,
+	// toolbar, ...)
+	factory()->removeClient(this);
+ }
  if (mDisplayManager) {
 	// we do NOT delete the display manager here
 	setDisplayManager(0);
  }
  d->mPlayers.clear();
- if (mTop && mTop->factory()) {
-	mTop->factory()->removeClient(this);
- }
- delete d->mCommandFrameDock;
- delete d->mChatDock;
 
  delete mCursor;
+ delete d->mCommandFrame;
+ delete d->mChat;
 
  delete d;
  boDebug() << k_funcinfo << "done" << endl;
@@ -148,16 +166,18 @@ void BosonWidgetBase::setDisplayManager(BoDisplayManager* displayManager)
 	}
 	mDisplayManager->reparent(this, QPoint(0, 0));
 	mDisplayManager->show();
+
+	changeToConfigCursor();
  }
 }
 
 BosonCanvas* BosonWidgetBase::canvas() const
 {
- return mTop->canvas();
+ return d->mCanvas;
 }
 
 #include <kstandarddirs.h> //locate()
-void BosonWidgetBase::init()
+void BosonWidgetBase::init(KDockWidget* chatDock, KDockWidget* commandFrameDock)
 {
  // NOTE: order of init* methods is very important here, so don't change it,
  //  unless you know what you're doing!
@@ -165,9 +185,8 @@ void BosonWidgetBase::init()
 	return;
  }
  d->mInitialized = true;
- initMiniMap();
- initChat();
- initCommandFrame();
+ initChat(chatDock);
+ initCommandFrame(commandFrameDock);
  initDisplayManager();
 
  initConnections();
@@ -184,69 +203,23 @@ void BosonWidgetBase::init()
 // setFocus(); // nonsense, since its still hidden
 
  initPlayersMenu();
-}
 
-void BosonWidgetBase::initMap()
-{
- boDebug() << k_funcinfo << endl;
-
- if (!boGame->playField()) {
-	boError() << k_funcinfo << "NULL playfield" << endl;
-	return;
- }
- BosonMap* map = boGame->playField()->map();
- canvas()->setMap(map);
- minimap()->setMap(map);
- minimap()->initMap();
- displayManager()->mapChanged();
-// boGame->setPlayField(playField()); // already done on startup in BosonStarting
-
- // AB: note that this meets the name "initMap" only slightly. We can't do this
- // when players are initialized, as the map must be known to them once we start
- // loading the units (for *loading* games)
- for (unsigned int i = 0; i < boGame->playerCount(); i++) {
-	boDebug() << "init map for player " << i << endl;
-	Player* p = (Player*)boGame->playerList()->at(i);
-	if (p) {
-		p->initMap(map, boGame->gameMode());
-	}
- }
-
-}
-
-void BosonWidgetBase::initMiniMap()
-{
- mMiniMap = new BosonMiniMap(0);
- minimap()->hide();
- minimap()->setBackgroundOrigin(WindowOrigin);
-
- connect(canvas(), SIGNAL(signalUnitMoved(Unit*, float, float)),
-		minimap(), SLOT(slotMoveUnit(Unit*, float, float)));
- connect(canvas(), SIGNAL(signalUnitRemoved(Unit*)),
-		minimap(), SLOT(slotUnitDestroyed(Unit*)));
+ BosonScript::setGame(boGame);
 }
 
 void BosonWidgetBase::initConnections()
 {
- connect(canvas(), SIGNAL(signalUnitRemoved(Unit*)),
-		this, SLOT(slotRemoveUnit(Unit*)));
-
- connect(boGame, SIGNAL(signalAddUnit(Unit*, int, int)),
-		canvas(), SLOT(slotAddUnit(Unit*, int, int))); 
- connect(boGame, SIGNAL(signalAddUnit(Unit*, int, int)),
-		this, SLOT(slotAddUnit(Unit*, int, int)));
-
  connect(boGame, SIGNAL(signalLoadExternalStuff(QDataStream&)),
-		this, SLOT(slotLoadExternalStuff(QDataStream&))); 
+		this, SLOT(slotLoadExternalStuff(QDataStream&)));
  connect(boGame, SIGNAL(signalSaveExternalStuff(QDataStream&)),
-		this, SLOT(slotSaveExternalStuff(QDataStream&))); 
+		this, SLOT(slotSaveExternalStuff(QDataStream&)));
  connect(boGame, SIGNAL(signalLoadExternalStuffFromXML(const QDomElement&)),
-		this, SLOT(slotLoadExternalStuffFromXML(const QDomElement&))); 
+		this, SLOT(slotLoadExternalStuffFromXML(const QDomElement&)));
  connect(boGame, SIGNAL(signalSaveExternalStuffAsXML(QDomElement&)),
-		this, SLOT(slotSaveExternalStuffAsXML(QDomElement&))); 
+		this, SLOT(slotSaveExternalStuffAsXML(QDomElement&)));
 
- connect(boGame, SIGNAL(signalAddChatSystemMessage(const QString&, const QString&)),
-		this, SLOT(slotAddChatSystemMessage(const QString&, const QString&)));
+ connect(boGame, SIGNAL(signalAddChatSystemMessage(const QString&, const QString&, const Player*)),
+		this, SLOT(slotAddChatSystemMessage(const QString&, const QString&, const Player*)));
 }
 
 void BosonWidgetBase::initDisplayManager()
@@ -254,20 +227,14 @@ void BosonWidgetBase::initDisplayManager()
 #warning do NOT do these connections here!
  // dont do the connect()s here, as some objects might not be deleted and
  // therefore we do the same connect twice if an endgame() occurs!
- connect(cmdFrame(), SIGNAL(signalAction(int)),
-		displayManager(), SLOT(slotUnitAction(int)));
  connect(mDisplayManager, SIGNAL(signalSelectionChanged(BoSelection*)),
-		d->mCommandFrame, SLOT(slotSelectionChanged(BoSelection*)));
- connect(mDisplayManager, SIGNAL(signalChangeActiveViewport(
-		const QPoint&, const QPoint&, const QPoint&, const QPoint&)),
-		minimap(), SLOT(slotMoveRect(
-		const QPoint&, const QPoint&, const QPoint&, const QPoint&)));
- connect(minimap(), SIGNAL(signalReCenterView(const QPoint&)),
-		mDisplayManager, SLOT(slotReCenterActiveDisplay(const QPoint&)));
+		cmdFrame(), SLOT(slotSelectionChanged(BoSelection*)));
  connect(cmdFrame(), SIGNAL(signalSelectUnit(Unit*)),
 		mDisplayManager, SLOT(slotActiveSelectSingleUnit(Unit*)));
  connect(boGame, SIGNAL(signalAdvance(unsigned int, bool)),
 		mDisplayManager, SLOT(slotAdvance(unsigned int, bool)));
+ connect(boGame, SIGNAL(signalAdvance(unsigned int, bool)),
+		this, SLOT(slotAdvance(unsigned int, bool)));
 
  displayManager()->setLocalPlayer(localPlayer()); // this does nothing.
 
@@ -275,27 +242,18 @@ void BosonWidgetBase::initDisplayManager()
 		mDisplayManager, SLOT(slotUnitChanged(Unit*)));
 }
 
-void BosonWidgetBase::addInitialDisplay()
-{
- initBigDisplay(displayManager()->addInitialDisplay());
 
- // we need to add the display first (in order to create a valid GL context) and
- // then load the cursor
- slotChangeCursor(boConfig->cursorMode(), boConfig->cursorDir());
-}
-
-void BosonWidgetBase::initChat()
+void BosonWidgetBase::initChat(KDockWidget* chatDock)
 {
  // note: we can use the chat widget even for editor mode, e.g. for status
  // messages!
- d->mChatDock = mTop->createDockWidget("chat_dock", 0, 0, i18n("Chat"));
- d->mChatDock->setEnableDocking(KDockWidget::DockTop | KDockWidget::DockBottom);
- d->mChatDock->setDockSite(KDockWidget::DockNone);
- d->mChat = new KGameChat(boGame, BosonMessage::IdChat, d->mChatDock);
- d->mChatDock->setWidget(d->mChat);
+ d->mChat = new BoGameChatWidget(chatDock, "chatwidget", boGame, BosonMessage::IdChat);
+ connect(d->mChat->chatWidget(), SIGNAL(signalScriptCommand(const QString&)),
+		this, SLOT(slotRunScriptLine(const QString&)));
+ chatDock->setWidget(d->mChat);
 
- connect(d->mChatDock, SIGNAL(iMBeingClosed()), this, SLOT(slotChatDockHidden()));
- connect(d->mChatDock, SIGNAL(hasUndocked()), this, SLOT(slotChatDockHidden()));
+ connect(chatDock, SIGNAL(iMBeingClosed()), this, SLOT(slotChatDockHidden()));
+ connect(chatDock, SIGNAL(hasUndocked()), this, SLOT(slotChatDockHidden()));
 }
 
 void BosonWidgetBase::initPlayer()
@@ -306,8 +264,6 @@ void BosonWidgetBase::initPlayer()
 	return;
  }
 
- setLocalPlayerRecursively(localPlayer());
-
  connect(localPlayer(), SIGNAL(signalPropertyChanged(KGamePropertyBase*, KPlayer*)),
 		this, SLOT(slotPlayerPropertyChanged(KGamePropertyBase*, KPlayer*)));
 
@@ -320,8 +276,30 @@ void BosonWidgetBase::initPlayer()
 void BosonWidgetBase::initGameMode()//FIXME: rename! we don't have a difference to initEditorMode anymore. maybe just initGame() or so??
 {
  BO_CHECK_NULL_RET(displayManager());
+
+ // Init all bigdisplays
+ QPtrListIterator<BosonBigDisplayBase> it(*displayManager()->displayList());
+ while (it.current()) {
+	initBigDisplay(it.current());
+	++it;
+ }
+
  initLayout();
  startScenarioAndGame();
+
+ initScripts();
+
+#ifdef PATHFINDER_TNG
+ // FIXME: this isn't correct I suppose. But atm it's only used for debugging
+ //  anyway (and I don't intend to use it for anything else)
+ boDebug() << k_funcinfo << "Trying searching sample path" << endl;
+ BosonPathInfo i;
+ i.start = QPoint(5 * BO_TILE_SIZE, 5 * BO_TILE_SIZE);
+ i.dest = QPoint(45 * BO_TILE_SIZE, 35 * BO_TILE_SIZE);
+ boDebug() << k_funcinfo << "Let's go!" << endl;
+ canvas()->pathfinder()->findPath(&i);
+ boDebug() << k_funcinfo << "sample path searching complete" << endl;
+#endif
 }
 
 void BosonWidgetBase::initBigDisplay(BosonBigDisplayBase* b)
@@ -330,38 +308,42 @@ void BosonWidgetBase::initBigDisplay(BosonBigDisplayBase* b)
 	boError() << k_funcinfo << "NULL display" << endl;
 	return;
  }
+ if (b->isInputInitialized()) {
+	// Already initialized
+	return;
+ }
  BO_CHECK_NULL_RET(boGame);
  if (boGame->gameMode()) {
-	b->setDisplayInput(new BosonBigDisplayInput(b));
+	BosonBigDisplayInput* i = new BosonBigDisplayInput(b);
+	b->setDisplayInput(i);
  } else {
-	b->setDisplayInput(new EditorBigDisplayInput(b));
+	EditorBigDisplayInput* i = new EditorBigDisplayInput(b);
+	b->setDisplayInput(i);
  }
  connect(b->displayInput(), SIGNAL(signalLockAction(bool)),
 		mDisplayManager, SIGNAL(signalLockAction(bool)));
  b->setCanvas(canvas());
+
+ // FIXME: this should be done by this->setLocalPlayer(), NOT here!
+ // (setLocalPlayer() is also called when changing player in editor mode)
  b->setLocalPlayer(localPlayer()); // AB: this will also add the mouseIO!
+
  b->setCursor(mCursor);
- b->setKGameChat(d->mChat);
+ b->setKGameChat(d->mChat->chatWidget());
 
  b->show();
  b->makeActive();
+
+ b->setInputInitialized(true);
 }
 
-void BosonWidgetBase::initCommandFrame()
+void BosonWidgetBase::initCommandFrame(KDockWidget* commandFrameDock)
 {
- d->mCommandFrameDock = mTop->createDockWidget("cmdframe_dock", 0, 0, i18n("Command Frame"));
- d->mCommandFrameDock->setEnableDocking(KDockWidget::DockLeft | KDockWidget::DockRight);
- d->mCommandFrameDock->setDockSite(KDockWidget::DockNone);
- d->mCommandFrame = createCommandFrame(d->mCommandFrameDock);
- d->mCommandFrameDock->setWidget(d->mCommandFrame);
- d->mCommandFrame->reparentMiniMap(minimap());
+ d->mCommandFrame = createCommandFrame(commandFrameDock);
+ commandFrameDock->setWidget(d->mCommandFrame);
 
- connect(d->mCommandFrameDock, SIGNAL(iMBeingClosed()), this, SLOT(slotCmdFrameDockHidden()));
- connect(d->mCommandFrameDock, SIGNAL(hasUndocked()), this, SLOT(slotCmdFrameDockHidden()));
-
- // can we use the same input for the editor?
-// d->mCmdInput = new CommandInput;
-// d->mCmdInput->setCommandFrame(d->mCommandFrame);
+ connect(commandFrameDock, SIGNAL(iMBeingClosed()), this, SLOT(slotCmdFrameDockHidden()));
+ connect(commandFrameDock, SIGNAL(hasUndocked()), this, SLOT(slotCmdFrameDockHidden()));
 }
 
 void BosonWidgetBase::initLayout()
@@ -371,21 +353,7 @@ void BosonWidgetBase::initLayout()
  QVBoxLayout* topLayout = new QVBoxLayout(this);
  topLayout->addWidget(displayManager());
 
- d->mCommandFrameDock->manualDock(mTop->getMainDockWidget(), KDockWidget::DockLeft, 30);
- d->mChatDock->manualDock(mTop->getMainDockWidget(), KDockWidget::DockBottom, 80);
-
- if (!kapp->config()->hasGroup("BosonGameDock")) {
-	boDebug() << k_funcinfo << "dock config does not exist" << endl;
-	// Dock config isn't saved (probably first start). Hide chat dock (we only
-	//  show commandframe by default)
-	d->mChatDock->changeHideShowState();
-	displayManager()->updateGeometry();  // Hack? Bug in BoDisplayManager?
- }
- else {
-	boDebug() << k_funcinfo << "dock config exists, loading" << endl;
-	mTop->loadGameDockConfig();
- }
- checkDockStatus();
+ emit signalLoadBosonGameDock();
 }
 
 void BosonWidgetBase::changeCursor(BosonCursor* cursor)
@@ -406,12 +374,16 @@ void BosonWidgetBase::slotHack1()
  displayManager()->activeDisplay()->resize(size);
 }
 
-void BosonWidgetBase::slotAddUnit(Unit* unit, int, int)
+void BosonWidgetBase::slotItemAdded(BosonItem* item)
 {
- if (!unit) {
-	boError() << k_funcinfo << "NULL unit" << endl;
+ if (!item) {
+	boError() << k_funcinfo << "NULL item" << endl;
 	return;
  }
+ if (!RTTI::isUnit(item->rtti())) {
+	return;
+ }
+ Unit* unit = (Unit*)item;
  Player* p = unit->owner();
  if (!p) {
 	boError() << k_funcinfo << "NULL owner" << endl;
@@ -424,7 +396,7 @@ void BosonWidgetBase::slotAddUnit(Unit* unit, int, int)
  slotUnitCountChanged(p);
 }
 
-void BosonWidgetBase::slotRemoveUnit(Unit* unit)
+void BosonWidgetBase::slotUnitRemoved(Unit* unit)
 {
  if (unit->owner() != localPlayer()) {
 	return;
@@ -433,21 +405,6 @@ void BosonWidgetBase::slotRemoveUnit(Unit* unit)
  slotUnitCountChanged(unit->owner());
 }
 
-void BosonWidgetBase::slotFog(int x, int y)
-{
- // very time critical function!!
-
- // slotFog() and slotUnfog() exist here so that we need only a single slot
- // instead of two (on ein minimap and one to actually create/remove the fog)
- // should save some performance (at least I hope)
- minimap()->slotFog(x, y); // FIXME: no need for slot
-}
-
-void BosonWidgetBase::slotUnfog(int x, int y)
-{
- minimap()->slotUnfog(x, y); // FIXME: no need for slot
-}
-
 void BosonWidgetBase::slotPlayerPropertyChanged(KGamePropertyBase* prop, KPlayer* p)
 {
  if (p != localPlayer()) {
@@ -466,48 +423,40 @@ void BosonWidgetBase::slotPlayerPropertyChanged(KGamePropertyBase* prop, KPlayer
  }
 }
 
-void BosonWidgetBase::slotInitFogOfWar()
-{
-// AB: could be placed into BosonWidget, since Editor doesnt have this
- minimap()->initFogOfWar(localPlayer());
-}
-
 bool BosonWidgetBase::sound() const
 {
- return boMusic->sound();
+ return boAudio->sound();
 }
 
 bool BosonWidgetBase::music() const
 {
- return boMusic->music();
+ return boAudio->music();
 }
 
 void BosonWidgetBase::slotToggleSound()
 {
- boMusic->setSound(!boMusic->sound());
- boConfig->setSound(boMusic->sound());
+ boAudio->setSound(!boAudio->sound());
+ boConfig->setSound(boAudio->sound());
 }
 
 void BosonWidgetBase::slotToggleMusic()
 {
- boMusic->setMusic(!boMusic->music());
- boConfig->setMusic(boMusic->music());
+ boAudio->setMusic(!boAudio->music());
+ boConfig->setMusic(boAudio->music());
 }
 
-void BosonWidgetBase::slotAddChatSystemMessage(const QString& fromName, const QString& text)
+void BosonWidgetBase::slotAddChatSystemMessage(const QString& fromName, const QString& text, const Player* forPlayer)
 {
+ if (forPlayer && forPlayer != localPlayer()) {
+	return;
+ }
  // add a chat system-message *without* sending it over network (makes no sense
  // for system messages)
- d->mChat->addSystemMessage(fromName, text);
+ d->mChat->chatWidget()->addSystemMessage(fromName, text);
 
  displayManager()->addChatMessage(i18n("--- %1: %2").arg(fromName).arg(text));
 }
 
-void BosonWidgetBase::slotSetCommandButtonsPerRow(int b)
-{
- d->mCommandFrame->slotSetButtonsPerRow(b);
-}
-
 void BosonWidgetBase::slotUnfogAll(Player* pl)
 {
  if (!boGame) {
@@ -529,8 +478,6 @@ void BosonWidgetBase::slotUnfogAll(Player* pl)
  } else {
 	list.append(pl);
  }
- bool minimapUpdatesEnabled = minimap()->isUpdatesEnabled();
- minimap()->setUpdatesEnabled(false);
  for (unsigned int i = 0; i < list.count(); i++) {
 	Player* p = (Player*)list.at(i);
 	for (unsigned int x = 0; x < map->width(); x++) {
@@ -540,8 +487,6 @@ void BosonWidgetBase::slotUnfogAll(Player* pl)
 	}
 	boGame->slotAddChatSystemMessage(i18n("Debug"), i18n("Unfogged player %1 - %2").arg(p->id()).arg(p->name()));
  }
- minimap()->setUpdatesEnabled(minimapUpdatesEnabled);
- minimap()->update();
 }
 
 void BosonWidgetBase::slotSplitDisplayHorizontal()
@@ -562,16 +507,16 @@ void BosonWidgetBase::slotRemoveActiveDisplay()
 void BosonWidgetBase::slotCmdBackgroundChanged(const QString& file)
 {
  if (file.isNull()) {
-	d->mCommandFrame->unsetPalette();
+	cmdFrame()->unsetPalette();
 	return;
  }
  QPixmap p(file);
  if (p.isNull()) {
 	boError() << k_funcinfo << "Could not load " << file << endl;
-	d->mCommandFrame->unsetPalette();
+	cmdFrame()->unsetPalette();
 	return;
  }
- d->mCommandFrame->setPaletteBackgroundPixmap(p);
+ cmdFrame()->setPaletteBackgroundPixmap(p);
 }
 
 void BosonWidgetBase::initKActions()
@@ -579,145 +524,193 @@ void BosonWidgetBase::initKActions()
  QSignalMapper* scrollMapper = new QSignalMapper(this);
  connect(scrollMapper, SIGNAL(mapped(int)), displayManager(), SLOT(slotScroll(int)));
  KAction* a;
- a = new KAction(i18n("Scroll Up"), Qt::Key_Up, scrollMapper,
+ KShortcut scrollUp(Qt::Key_Up);
+ scrollUp.append(KKeySequence(KKey(Qt::Key_W)));
+ a = new KAction(i18n("Scroll Up"), scrollUp, scrollMapper,
 		SLOT(map()), actionCollection(),
 		"scroll_up");
  scrollMapper->setMapping(a, BoDisplayManager::ScrollUp);
- a = new KAction(i18n("Scroll Down"), Qt::Key_Down, scrollMapper,
+ KShortcut scrollDown(Qt::Key_Down);
+ scrollDown.append(KKeySequence(KKey(Qt::Key_S)));
+ a = new KAction(i18n("Scroll Down"), scrollDown, scrollMapper,
 		SLOT(map()), actionCollection(),
 		"scroll_down");
  scrollMapper->setMapping(a, BoDisplayManager::ScrollDown);
- a = new KAction(i18n("Scroll Left"), Qt::Key_Left, scrollMapper,
+ KShortcut scrollLeft(Qt::Key_Left);
+ scrollLeft.append(KKeySequence(KKey(Qt::Key_A)));
+ a = new KAction(i18n("Scroll Left"), scrollLeft, scrollMapper,
 		SLOT(map()), actionCollection(),
 		"scroll_left");
  scrollMapper->setMapping(a, BoDisplayManager::ScrollLeft);
- a = new KAction(i18n("Scroll Right"), Qt::Key_Right, scrollMapper,
+ KShortcut scrollRight(Qt::Key_Right);
+ scrollRight.append(KKeySequence(KKey(Qt::Key_D)));
+ a = new KAction(i18n("Scroll Right"), scrollRight, scrollMapper,
 		SLOT(map()), actionCollection(),
 		"scroll_right");
  scrollMapper->setMapping(a, BoDisplayManager::ScrollRight);
+ KShortcut rotateLeft(Qt::Key_Q);
+ a = new KAction(i18n("Rotate Left"), rotateLeft, displayManager(),
+		SLOT(slotRotateLeft()), actionCollection(),
+		"rotate_left");
+ KShortcut rotateRight(Qt::Key_E);
+ a = new KAction(i18n("Rotate Right"), rotateRight, displayManager(),
+		SLOT(slotRotateRight()), actionCollection(),
+		"rotate_right");
+ KShortcut zoomIn(Qt::Key_F);
+ a = new KAction(i18n("Zoom In"), zoomIn, displayManager(),
+		SLOT(slotZoomIn()), actionCollection(),
+		"zoom_in");
+ KShortcut zoomOut(Qt::Key_V);
+ a = new KAction(i18n("Zoom out"), zoomOut, displayManager(),
+		SLOT(slotZoomOut()), actionCollection(),
+		"zoom_out");
+
 
  // FIXME: the editor should not have a "game" menu, so what to do with this?
  (void)new KAction(i18n("&Reset View Properties"), KShortcut(Qt::Key_R),
 		displayManager(), SLOT(slotResetViewProperties()), actionCollection(), "game_reset_view_properties");
 
  // Dockwidgets show/hide
- d->mActionChat = new KToggleAction(i18n("Show &Chat"),
-		KShortcut(Qt::CTRL+Qt::Key_C), this, SLOT(slotToggleChatVisible()),
+ d->mActionChat = new KToggleAction(i18n("Show Cha&t"),
+		KShortcut(Qt::CTRL+Qt::Key_C), this, SIGNAL(signalToggleChatVisible()),
 		actionCollection(), "options_show_chat");
  d->mActionCmdFrame = new KToggleAction(i18n("Show C&ommandframe"),
-		KShortcut(Qt::CTRL+Qt::Key_F), this, SLOT(slotToggleCmdFrameVisible()),
+		KShortcut(Qt::CTRL+Qt::Key_F), this, SIGNAL(signalToggleCmdFrameVisible()),
 		actionCollection(), "options_show_cmdframe");
 
  (void)new KAction(i18n("&Grab Screenshot"), KShortcut(Qt::CTRL + Qt::Key_G),
 		this, SLOT(slotGrabScreenshot()), actionCollection(), "game_grab_screenshot");
  (void)new KAction(i18n("Grab &Profiling data"), KShortcut(Qt::CTRL + Qt::Key_P),
 		this, SLOT(slotGrabProfiling()), actionCollection(), "game_grab_profiling");
+ KToggleAction* movie = new KToggleAction(i18n("Grab &Movie"),
+		KShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_M), 0, 0, actionCollection(), "game_grab_movie");
+ movie->setChecked(false);
+ connect(movie, SIGNAL(toggled(bool)),
+		displayManager(), SLOT(slotSetGrabMovie(bool)));
 
  // Debug
+ KToggleAction* resources = new KToggleAction(i18n("Show resources"),
+		KShortcut(), 0, 0, actionCollection(), "show_resources");
+ resources->setChecked(true);
+ connect(resources, SIGNAL(toggled(bool)),
+		this, SLOT(slotSetShowResources(bool)));
  (void)new KAction(i18n("&Unfog"), KShortcut(), this,
 		SLOT(slotUnfogAll()), actionCollection(), "debug_unfog");
  KToggleAction* mapCoordinates = new KToggleAction(i18n("Debug &map coordinates"),
 		KShortcut(), 0, 0, actionCollection(), "debug_map_coordinates");
  mapCoordinates->setChecked(false);
  connect(mapCoordinates, SIGNAL(toggled(bool)),
-		displayManager(), SLOT(slotSetDebugMapCoordinates(bool)));
+		this, SLOT(slotSetDebugMapCoordinates(bool)));
+ KToggleAction* PFData = new KToggleAction(i18n("Debug pathfinder data"),
+		KShortcut(), 0, 0, actionCollection(), "debug_pf_data");
+ PFData->setChecked(false);
+ connect(PFData, SIGNAL(toggled(bool)),
+		this, SLOT(slotSetDebugPFData(bool)));
  KToggleAction* cellGrid = new KToggleAction(i18n("Show Cell &Grid"),
 		KShortcut(), 0, 0, actionCollection(), "debug_cell_grid");
  cellGrid->setChecked(false);
  connect(cellGrid, SIGNAL(toggled(bool)),
-		displayManager(), SLOT(slotSetDebugShowCellGrid(bool)));
+		this, SLOT(slotSetDebugShowCellGrid(bool)));
  KToggleAction* matrices = new KToggleAction(i18n("Debug Ma&trices"),
 		KShortcut(), 0, 0, actionCollection(), "debug_matrices");
  matrices->setChecked(false);
  connect(matrices, SIGNAL(toggled(bool)),
-		displayManager(), SLOT(slotSetDebugMatrices(bool)));
+		this, SLOT(slotSetDebugMatrices(bool)));
  KToggleAction* works = new KToggleAction(i18n("Debug Item works"),
 		KShortcut(), 0, 0, actionCollection(), "debug_works");
  works->setChecked(false);
  connect(works, SIGNAL(toggled(bool)),
-		displayManager(), SLOT(slotSetDebugItemWorks(bool)));
+		this, SLOT(slotSetDebugItemWorks(bool)));
  KToggleAction* camera = new KToggleAction(i18n("Debug camera"),
 		KShortcut(), 0, 0, actionCollection(), "debug_camera");
  camera->setChecked(false);
  connect(camera, SIGNAL(toggled(bool)),
-		displayManager(), SLOT(slotSetDebugCamera(bool)));
+		this, SLOT(slotSetDebugCamera(bool)));
  KToggleAction* rendercounts = new KToggleAction(i18n("Debug Rendering counts"),
 		KShortcut(), 0, 0, actionCollection(), "debug_rendercounts");
  rendercounts->setChecked(false);
  connect(rendercounts, SIGNAL(toggled(bool)),
-		displayManager(), SLOT(slotSetDebugRenderCounts(bool)));
+		this, SLOT(slotSetDebugRenderCounts(bool)));
  KToggleAction* cheating = new KToggleAction(i18n("Enable &Cheating"),
 		KShortcut(), 0, 0, actionCollection(), "debug_enable_cheating");
  connect(cheating, SIGNAL(toggled(bool)), this, SLOT(slotToggleCheating(bool)));
+ KToggleAction* wireFrames = new KToggleAction(i18n("Render &Wireframes"),
+		KShortcut(), 0, 0, actionCollection(), "debug_wireframes");
+ connect(wireFrames, SIGNAL(toggled(bool)), this, SLOT(slotDebugToggleWireFrames(bool)));
+ wireFrames->setChecked(false);
+ slotDebugToggleWireFrames(false);
+ KToggleAction* boundingboxes = new KToggleAction(i18n("Render item's bounding boxes"),
+		KShortcut(), 0, 0, actionCollection(), "debug_boundingboxes");
+ boundingboxes->setChecked(false);
+ connect(boundingboxes, SIGNAL(toggled(bool)),
+		this, SLOT(slotSetDebugBoundingBoxes(bool)));
+ KToggleAction* fps = new KToggleAction(i18n("Debug FPS"),
+		KShortcut(), 0, 0, actionCollection(), "debug_fps");
+ fps->setChecked(false);
+ connect(fps, SIGNAL(toggled(bool)),
+		this, SLOT(slotSetDebugFPS(bool)));
+ (void)new KAction(i18n("&Unfog"), KShortcut(), this,
+		SLOT(slotUnfogAll()), actionCollection(), "debug_unfog");
+ (void)new KAction(i18n("Dump game &log"), KShortcut(), this,
+		SLOT(slotDumpGameLog()), actionCollection(), "debug_gamelog");
+ KToggleAction* enablecolormap = new KToggleAction(i18n("Enable colormap"),
+		KShortcut(), 0, 0, actionCollection(), "debug_colormap_enable");
+ enablecolormap->setChecked(false);
+ connect(enablecolormap, SIGNAL(toggled(bool)),
+		this, SLOT(slotSetEnableColormap(bool)));
 
 
- KSelectAction* debugMode = new KSelectAction("Mode", KShortcut(), actionCollection(), "debug_mode");
+ KSelectAction* debugMode = new KSelectAction(i18n("Mode"), KShortcut(),
+		actionCollection(), "debug_mode");
  connect(debugMode, SIGNAL(activated(int)), this, SLOT(slotDebugMode(int)));
  QStringList l;
- l.append("Normal");
- l.append("Debug Selection");
+ l.append(i18n("Normal"));
+ l.append(i18n("Debug Selection"));
  debugMode->setItems(l);
  debugMode->setCurrentItem(0);
- d->mActionDebugPlayers = new KActionMenu("Players", actionCollection(), "debug_players");
+ d->mActionDebugPlayers = new KActionMenu(i18n("Players"),
+		actionCollection(), "debug_players");
+
+ (void)new KAction(i18n("Show OpenGL states"), KShortcut(), this,
+		SLOT(slotShowGLStates()), actionCollection(),
+		"debug_show_opengl_states");
+ (void)new KAction(i18n("&Reload model textures"), KShortcut(), this,
+		SLOT(slotReloadModelTextures()), actionCollection(), "debug_lazy_reload_model_textures");
+ (void)new KAction(i18n("Light0..."), KShortcut(), displayManager(),
+		SLOT(slotShowLight0Widget()), actionCollection(),
+		"debug_light0");
+ (void)new KAction(i18n("Reload &meshrenderer plugin"), KShortcut(), this,
+		SLOT(slotReloadMeshRenderer()), actionCollection(),
+		"debug_lazy_reload_meshrenderer");
+ (void)new KAction(i18n("Reload &groundrenderer plugin"), KShortcut(), this,
+		SLOT(slotReloadGroundRenderer()), actionCollection(),
+		"debug_lazy_reload_groundrenderer");
+ (void)new KAction(i18n("Crash boson"), KShortcut(), this,
+		SLOT(slotCrashBoson()), actionCollection(),
+		"debug_crash_boson");
 
  cheating->setChecked(DEFAULT_CHEAT_MODE);
  slotToggleCheating(DEFAULT_CHEAT_MODE);
- checkDockStatus();
+ emit signalCheckDockStatus();
 }
 
 void BosonWidgetBase::quitGame()
 {
 // this needs to be done first, before the players are removed
  boDebug() << k_funcinfo << endl;
- canvas()->deleteDestroyed();
  boGame->quitGame();
+ d->mCanvas = 0;
  boDebug() << k_funcinfo << "done" << endl;
 }
 
-bool BosonWidgetBase::isCmdFrameVisible() const
+void BosonWidgetBase::setActionChat(bool chatVisible)
 {
- return d->mCommandFrameDock->isVisible();
+ d->mActionChat->setChecked(chatVisible);
 }
 
-bool BosonWidgetBase::isChatVisible() const
+void BosonWidgetBase::setActionCmdFrame(bool cmdFrameVisible)
 {
- return d->mChatDock->isVisible();
-}
-
-void BosonWidgetBase::setCmdFrameVisible(bool visible)
-{
- if (visible && d->mCommandFrameDock->mayBeShow()) {
-	d->mCommandFrameDock->show();
- } else if (! visible && d->mCommandFrameDock->mayBeHide()) {
-	d->mCommandFrameDock->hide();
- }
-}
-
-void BosonWidgetBase::setChatVisible(bool visible)
-{
- if (visible && d->mChatDock->mayBeShow()) {
-	d->mChatDock->show();
- } else if (! visible && d->mChatDock->mayBeHide()) {
-	d->mChatDock->hide();
- }
-}
-
-void BosonWidgetBase::slotToggleCmdFrameVisible()
-{
- d->mCommandFrameDock->changeHideShowState();
- checkDockStatus();
-}
-
-void BosonWidgetBase::slotToggleChatVisible()
-{
- d->mChatDock->changeHideShowState();
- checkDockStatus();
-}
-
-void BosonWidgetBase::checkDockStatus()
-{
- d->mActionChat->setChecked(isChatVisible());
- d->mActionCmdFrame->setChecked(isCmdFrameVisible());
+ d->mActionCmdFrame->setChecked(cmdFrameVisible);
 }
 
 
@@ -746,14 +739,9 @@ void BosonWidgetBase::startScenarioAndGame()
  //  already loaded as well
  // FIXME: this is hackish but I don't know any other way of checking if game
  //  is loaded or new one here. Feel free to improve
- if (boGame->loadingStatus() != Boson::LoadingCompleted) {
+ if (boGame->loadingStatus() != BosonSaveLoad::LoadingCompleted) {
 	displayManager()->slotCenterHomeBase();
  }
-
- #warning FIXME
- // this is a strange bug: we need to resize the widget once it is shown - otherwise we'll have a VERY slot frame rate.
- // I can't find out where the problem resides :-(
- //QTimer::singleShot(500, this, SLOT(slotHack1()));
 }
 
 void BosonWidgetBase::slotDebugMode(int index)
@@ -828,6 +816,7 @@ void BosonWidgetBase::slotDebugPlayer(int index)
 void BosonWidgetBase::slotChatDockHidden()
 {
  d->mActionChat->setChecked(false);
+ setFocus();
 }
 
 void BosonWidgetBase::slotCmdFrameDockHidden()
@@ -890,27 +879,25 @@ BosonCommandFrameBase* BosonWidgetBase::cmdFrame() const
  return d->mCommandFrame;
 }
 
-void BosonWidgetBase::setLocalPlayer(Player* p, bool init) //AB: probably init = false is obsolete!
+void BosonWidgetBase::setLocalPlayer(Player* p)
 {
- // FIXME: ensure that d->mCmdInput gets removed in BosonWidget before calling
- // this! or maybe while calling this!
- setLocalPlayerRecursively(p);
-
- if (init) {
-	if (!p) {
-		boDebug() << k_funcinfo << "NULL player" << endl;
-		
-		return;
+ if (mLocalPlayer) {
+	KGameIO* oldIO = mLocalPlayer->findRttiIO(BosonLocalPlayerInput::LocalPlayerInputRTTI);
+	if (oldIO) {
+		mLocalPlayer->removeGameIO(oldIO);
 	}
-	initPlayer();
  }
-}
-
-void BosonWidgetBase::setLocalPlayerRecursively(Player* p)
-{
  mLocalPlayer = p;
- if (minimap()) {
-	minimap()->setLocalPlayer(localPlayer());
+ if (mLocalPlayer) {
+	BosonLocalPlayerInput* input = new BosonLocalPlayerInput();
+	connect(input, SIGNAL(signalAction(const BoSpecificAction&)),
+			mDisplayManager, SLOT(slotAction(const BoSpecificAction&)));
+	mLocalPlayer->addGameIO(input);
+	if (!cmdFrame()) {
+		boError() << k_funcinfo << "cmdFrame() is NULL - this should not be possible here!" << endl;
+	} else {
+		input->setCommandFrame(cmdFrame());
+	}
  }
  if (displayManager()) {
 	displayManager()->setLocalPlayer(localPlayer());
@@ -919,29 +906,31 @@ void BosonWidgetBase::setLocalPlayerRecursively(Player* p)
 	boError() << k_funcinfo << "NULL game object" << endl;
 	return;
  }
- boGame->setLocalPlayer(localPlayer());
- if (d->mCommandFrame) {
-	d->mCommandFrame->setLocalPlayer(localPlayer());
+ if (cmdFrame()) {
+	cmdFrame()->setLocalPlayer(localPlayer());
  }
  if (d->mChat) {
-	d->mChat->setFromPlayer(localPlayer());
+	d->mChat->chatWidget()->setFromPlayer(localPlayer());
  }
 }
 
 void BosonWidgetBase::slotGrabScreenshot()
 {
+ BO_CHECK_NULL_RET(displayManager());
+ BO_CHECK_NULL_RET(displayManager()->activeDisplay());
  boDebug() << k_funcinfo << "Taking screenshot!" << endl;
 
-#warning FIXME
- // this should NOT do a reference to mTop!
- // use either parent() or a signal (i.e. move the code to TopWidget)
- QPixmap shot = QPixmap::grabWindow(mTop->winId());
+// QImage image = displayManager()->activeDisplay()->screenShot();
+ QPixmap shot = QPixmap::grabWindow(parentWidget()->winId());
+ if (shot.isNull()) {
+	boError() << k_funcinfo << "NULL image returned" << endl;
+	return;
+ }
  QString file = findSaveFileName("boson", "jpg");
  if (file.isNull()) {
 	boWarning() << k_funcinfo << "Can't find free filename???" << endl;
 	return;
  }
- // TODO: chat message about file location!
  boDebug() << k_funcinfo << "Saving screenshot to " << file << endl;
  bool ok = shot.save(file, "JPEG", 90);
  if (!ok) {
@@ -995,6 +984,11 @@ void BosonWidgetBase::slotToggleCheating(bool on)
  setActionEnabled("debug_players", on);
 }
 
+void BosonWidgetBase::slotDebugToggleWireFrames(bool on)
+{
+ boConfig->setWireFrames(on);
+}
+
 void BosonWidgetBase::setActionEnabled(const char* name, bool on)
 {
  KAction* a = actionCollection()->action(name);
@@ -1048,8 +1042,12 @@ OptionsDialog* BosonWidgetBase::gamePreferences(bool editor)
 		this, SLOT(slotChangeCursor(int, const QString&)));
  connect(dlg, SIGNAL(signalCmdBackgroundChanged(const QString&)),
 		this, SLOT(slotCmdBackgroundChanged(const QString&)));
+ connect(dlg, SIGNAL(signalOpenGLSettingsUpdated()),
+		displayManager(), SLOT(slotUpdateOpenGLSettings()));
  connect(dlg, SIGNAL(signalApply()),
 		this, SLOT(slotApplyOptions()));
+ connect(dlg, SIGNAL(signalFontChanged(const BoFontInfo&)),
+		displayManager(), SLOT(slotChangeFont(const BoFontInfo&)));
 
  return dlg;
 }
@@ -1062,9 +1060,170 @@ void BosonWidgetBase::slotApplyOptions()
  // AB: FIXME: cmdbackground is not yet stored in boConfig! that option should
  // be managed here!
  boDebug() << k_funcinfo << endl;
- minimap()->repaint(); // it automatically uses the new scaling factor
  displayManager()->slotUpdateIntervalChanged(boConfig->updateInterval()); // FIXME: no slot anymore
  displayManager()->setToolTipCreator(boConfig->toolTipCreator());
  displayManager()->setToolTipUpdatePeriod(boConfig->toolTipUpdatePeriod());
 }
 
+void BosonWidgetBase::slotSetDebugMapCoordinates(bool debug)
+{
+ boConfig->setDebugMapCoordinates(debug);
+}
+
+void BosonWidgetBase::slotSetDebugPFData(bool debug)
+{
+ boConfig->setDebugPFData(debug);
+}
+
+void BosonWidgetBase::slotSetDebugShowCellGrid(bool debug)
+{
+ boConfig->setDebugShowCellGrid(debug);
+}
+
+void BosonWidgetBase::slotSetDebugMatrices(bool debug)
+{
+ boConfig->setDebugOpenGLMatrices(debug);
+}
+
+void BosonWidgetBase::slotSetDebugItemWorks(bool debug)
+{
+ boConfig->setDebugItemWorkStatistics(debug);
+}
+
+void BosonWidgetBase::slotSetDebugCamera(bool debug)
+{
+ boConfig->setDebugOpenGLCamera(debug);
+}
+
+void BosonWidgetBase::slotSetDebugRenderCounts(bool debug)
+{
+ boConfig->setDebugRenderCounts(debug);
+}
+
+void BosonWidgetBase::slotSetDebugBoundingBoxes(bool debug)
+{
+ boConfig->setDebugBoundingBoxes(debug);
+}
+
+void BosonWidgetBase::slotSetDebugFPS(bool debug)
+{
+ boConfig->setDebugFPS(debug);
+}
+
+void BosonWidgetBase::slotSetShowResources(bool show)
+{
+ boConfig->setShowResources(show);
+}
+
+void BosonWidgetBase::slotSetEnableColormap(bool enable)
+{
+ boConfig->setEnableColormap(enable);
+}
+
+void BosonWidgetBase::slotRunScriptLine(const QString& line)
+{
+ d->mScript->execLine(line);
+}
+
+void BosonWidgetBase::slotAdvance(unsigned int, bool)
+{
+ d->mScript->advance();
+}
+
+void BosonWidgetBase::initScripts()
+{
+ // Init computer player scripts
+ QPtrListIterator<KPlayer> it(*boGame->playerList());
+ for (; it.current(); ++it) {
+	QPtrListIterator<KGameIO> ioit(*it.current()->ioList());
+	for (; ioit.current(); ++ioit) {
+		if (ioit.current()->rtti() == KGameIO::ComputerIO) {
+			BosonComputerIO* io = (BosonComputerIO*)ioit.current();
+			io->initScript();
+		}
+	}
+ }
+
+ // Init script for local player
+ d->mScript = BosonScript::newScriptParser(BosonScript::Python, localPlayer());
+ // No script will be loaded
+ d->mScript->loadScript(locate("data", "boson/scripts/localplayer-script.py"));
+ d->mScript->init();
+}
+
+void BosonWidgetBase::slotDumpGameLog()
+{
+ boGame->saveGameLogs("boson");
+}
+
+void BosonWidgetBase::slotReloadModelTextures()
+{
+ BO_CHECK_NULL_RET(BosonModelTextures::modelTextures());
+ BosonModelTextures::modelTextures()->reloadTextures();
+}
+
+void BosonWidgetBase::slotReloadMeshRenderer()
+{
+ bool unusable = false;
+ bool r = BoMeshRendererManager::manager()->reloadPlugin(&unusable);
+ if (r) {
+	return;
+ }
+ boError() << "meshrenderer reloading failed" << endl;
+ if (unusable) {
+	KMessageBox::sorry(this, i18n("Reloading meshrenderer failed, library is now unusable. quitting."));
+	exit(1);
+ } else {
+	KMessageBox::sorry(this, i18n("Reloading meshrenderer failed but library should still be usable"));
+ }
+}
+
+void BosonWidgetBase::slotReloadGroundRenderer()
+{
+ bool unusable = false;
+ bool r = BoGroundRendererManager::manager()->reloadPlugin(&unusable);
+ if (r) {
+	return;
+ }
+ boError() << "groundrenderer reloading failed" << endl;
+ if (unusable) {
+	KMessageBox::sorry(this, i18n("Reloading groundrenderer failed, library is now unusable. quitting."));
+	exit(1);
+ } else {
+	KMessageBox::sorry(this, i18n("Reloading groundrenderer failed but library should still be usable"));
+ }
+}
+
+void BosonWidgetBase::slotShowGLStates()
+{
+ boDebug() << k_funcinfo << endl;
+ BoGLStateWidget* w = new BoGLStateWidget(0, 0, WDestructiveClose);
+ w->show();
+}
+
+void BosonWidgetBase::changeToConfigCursor()
+{
+ slotChangeCursor(boConfig->cursorMode(), boConfig->cursorDir());
+}
+
+void BosonWidgetBase::setCanvas(BosonCanvas* canvas)
+{
+ d->mCanvas = canvas;
+ connect(d->mCanvas, SIGNAL(signalUnitRemoved(Unit*)),
+		this, SLOT(slotUnitRemoved(Unit*)));
+ connect(d->mCanvas, SIGNAL(signalItemAdded(BosonItem*)),
+		this, SLOT(slotItemAdded(BosonItem*)));
+
+ BosonScript::setCanvas(d->mCanvas);
+}
+
+void BosonWidgetBase::initMap()
+{
+ // implemented by EditorWidget
+}
+
+void BosonWidgetBase::slotCrashBoson()
+{
+ ((QObject*)0)->name();
+}
+
