@@ -110,6 +110,7 @@ QString BoToolTipCreatorDebug::createToolTip(const BosonItem* item) const
  // too
  QString tip = i18n("Rtti: %1").arg(item->rtti());
  tip += i18n("\nPosition: (%1,%2,%3)").arg(item->x()).arg(item->y()).arg(item->z());
+ tip += i18n("\nRotation: (%1,%2,%3)").arg(item->xRotation()).arg(item->yRotation()).arg(item->rotation());
  tip += i18n("\nSize: (%1,%2)").arg(item->width()).arg(item->height());
 
  QPtrVector<Cell>* cells = item->cellsConst();
@@ -151,6 +152,19 @@ QString BoToolTipCreatorDebug::createToolTip(const BosonItem* item) const
 	}
  }
 
+ QValueList<QPoint> pathpoints = u->pathPointList();
+ if (pathpoints.count() == 0) {
+	tip += i18n("\nNo pathpoints");
+ } else {
+	QValueList<QPoint>::Iterator it;
+	tip += i18n("\nPathpoints: ");
+	for (it = pathpoints.begin(); it != pathpoints.end(); ++it) {
+		if (it != pathpoints.begin()) {
+			tip += i18n(",");
+		}
+		tip += i18n("(%1,%2)").arg((*it).x()).arg((*it).y());
+	}
+ }
 
  if (u->isFacility()) {
 	Facility* fac = (Facility*)u;
