@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #include "editorwidget.h"
 
 #include "defines.h"
-#include "bosonminimap.h"
 #include "boson.h"
 #include "bosonmap.h"
 #include "player.h"
@@ -32,7 +31,6 @@
 #include "bpfdescriptiondialog.h"
 #include "optionsdialog.h"
 #include "boaction.h"
-#include "bosonlocalplayerinput.h"
 #include "botexmapimportdialog.h"
 #include "bosongroundtheme.h"
 #include "commandframe/bosoncommandframebase.h"
@@ -59,15 +57,11 @@ class EditorWidget::EditorWidgetPrivate
 public:
 	EditorWidgetPrivate()
 	{
-		mLocalPlayerInput = 0;
-
 		mPlayerAction = 0;
 		mPlaceAction = 0;
 		mChangeHeight = 0;
 	}
 
-	BosonLocalPlayerInput* mLocalPlayerInput;
-
 	KSelectAction* mPlayerAction;
 	KSelectAction* mPlaceAction;
 	KToggleAction* mChangeHeight;
@@ -126,8 +120,6 @@ void EditorWidget::initMap()
 
  connect(boGame, SIGNAL(signalChangeTexMap(int,int,unsigned int,unsigned int*,unsigned char*)),
 		map, SLOT(slotChangeTexMap(int,int,unsigned int,unsigned int*,unsigned char*)));
- connect(map, SIGNAL(signalCellChanged(int,int)),
-		minimap(), SLOT(slotUpdateCell(int,int)));
 
  // AB: maybe we don't need the connect() above concerning groundtheme. it can't
  // be changed once the map is started
@@ -142,7 +134,8 @@ void EditorWidget::initPlayer()
 {
  BosonWidgetBase::initPlayer();
 
- minimap()->slotShowMap(true);
+ // FIXME: GL minimap!
+// minimap()->slotShowMap(true);
 }
 
 BosonCommandFrameBase* EditorWidget::createCommandFrame(QWidget* parent)
@@ -165,6 +158,8 @@ void EditorWidget::initKActions()
  BosonWidgetBase::initKActions();
 // KStdAction::save(this, SLOT(slotSavePlayField()), actionCollection(), "file_save_playfield");
  KStdAction::saveAs(this, SLOT(slotSavePlayFieldAs()), actionCollection(), "file_save_playfield_as");
+ KAction* close = KStdAction::close(this, SIGNAL(signalEndGame()), actionCollection());
+ close->setText(i18n("&End Editor"));
  KStdAction::quit(this, SIGNAL(signalQuit()), actionCollection());
  (void)KStdAction::preferences(this, SLOT(slotEditorPreferences()), actionCollection());
 
@@ -197,8 +192,8 @@ void EditorWidget::initKActions()
  (void)new KAction(i18n("Edit &Oil"), KShortcut(), this,
 		SLOT(slotEditPlayerOil()), actionCollection(),
 		"editor_player_oil");
- d->mChangeHeight = new KToggleAction(i18n("Edit &Height"), KShortcut(),
-		this, 0, actionCollection(), "editor_height");
+ d->mChangeHeight = new KToggleAction(i18n("Edit &Height"), "bo_edit_height", 
+		KShortcut(),this, 0, actionCollection(), "editor_height"); 
  connect(d->mChangeHeight, SIGNAL(toggled(bool)),
 		this, SLOT(slotEditHeight(bool)));
  (void)new KAction(i18n("&Import height map"), KShortcut(), this,
@@ -326,11 +321,6 @@ void EditorWidget::slotPlayerJoinedGame(KPlayer* player)
  d->mPlayers.append(p);
  players.append(p->name());
  d->mPlayerAction->setItems(players);
-
- // dunno if this makes sense - but currently one cannot add more players so we
- // just activate the player that was added last.
- d->mPlayerAction->setCurrentItem(players.count() - 1);
- slotChangeLocalPlayer(d->mPlayerAction->currentItem());
 }
 
 void EditorWidget::slotPlayerLeftGame(KPlayer* player)
