@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,8 +23,9 @@
 #include "../selectbox.h"
 #include "../bosonmodel.h"
 #include "../cell.h" // for deleteitem. i dont want this. how can we avoid this? don't use qptrvector probably.
-#include "../bosonparticlesystem.h"
+#include "../bosoneffect.h"
 #include "../bosonpropertyxml.h"
+#include "../bo3dtools.h"
 #include "bosonitempropertyhandler.h"
 #include "bodebug.h"
 
@@ -118,6 +119,7 @@ BosonItem::BosonItem(Player* owner, BosonModel* model, BosonCanvas* canvas)
  mCanvas = canvas;
  mModel = model;
 
+ mId = 0;
  mCurrentAnimation = 0;
  mX = mY = mZ = 0.0f;
  mWidth = mHeight = 0;
@@ -127,7 +129,6 @@ BosonItem::BosonItem(Player* owner, BosonModel* model, BosonCanvas* canvas)
  mXRotation = 0.0f;
  mYRotation = 0.0f;
  mGLDepthMultiplier = 1.0f;
- mDisplayList = 0;
  mFrame = 0;
  mGLConstructionStep = 0;
  mAnimationCounter = 0;
@@ -147,7 +148,7 @@ BosonItem::BosonItem(Player* owner, BosonModel* model, BosonCanvas* canvas)
  // 1.732 == sqrt(3) i.e. lenght of vector whose all components are 1
  mBoundingSphereRadius = 1.732f; // TODO: can we extract this from the model? this probably needs to change with different frames!
 
- mIsAnimated = false;
+ mIsAnimated = true; // obsolete! remove!
  mSelectBox = 0;
 
  mCells = new QPtrVector<Cell>();
@@ -155,7 +156,7 @@ BosonItem::BosonItem(Player* owner, BosonModel* model, BosonCanvas* canvas)
  if (mCanvas) {
 	mCanvas->addItem(this);
  } else {
-	boWarning() << k_funcinfo << "NULL canvas" << endl;
+	BO_NULL_ERROR(mCanvas);
  }
 
  if (!mModel) {
@@ -180,9 +181,12 @@ BosonItem::~BosonItem()
  unselect();
  if (canvas()) {
 	canvas()->removeFromCells(this);
-	canvas()->removeAnimation(this);
-	canvas()->removeItem(this);
+	if (canvas()->allItems()->contains(this)) {
+		// must happen BEFORE the item is deleted!
+		boError() << k_funcinfo << "the item has not yet been removed from the canvas!!" << endl;
+	}
  }
+ delete mCells;
 }
 
 
@@ -243,46 +247,68 @@ void BosonItem::makeCells(Cell* allCells, QPtrVector<Cell>* cells, int left, int
 
 bool BosonItem::bosonCollidesWith(BosonItem* item) const
 {
- // First we test z-coordinate
- if (QMAX(z(), item->z()) > QMIN(z() + depth(), item->z() + item->depth())) {
-	// z-coordinates doesn't intersect. Then items doesn't collide either
+ BoVector3 itempos(item->x() + xVelocity(), item->y() + yVelocity(), item->z() + zVelocity());
+ return bosonCollidesWith(itempos, itempos + BoVector3(item->width(), item->height(), item->depth()));
+}
+
+bool BosonItem::bosonCollidesWith(const BoVector3& v1, const BoVector3& v2) const
+{
+// boDebug() << "  ++> " << k_funcinfo << "Item coords: (" << x() << "; " << y() << "; " << z() <<
+//		"); size: (" << width() << "; " << height() << "; " << depth() << ")" << endl;
+
+ // Check z-coord first
+ if (QMAX(z() + zVelocity(), v1.z()) >= QMIN(z() + zVelocity() + depth(), v2.z())) {
+	// z-coordinates don't intersect
 	return false;
  }
 
- // Then bounding rect intersect test
- // Taken from QRect::intersects() but I didn't use this method for speed reasons
- if ((QMAX(x(), item->x()) > QMIN(x() + width() - 1, item->x() + item->width() - 1)) ||
-		(QMAX(y(), item->y()) > QMIN(y() + height() - 1, item->y() + item->height() - 1))) {
-	// Bounding rects does not intersect
-	return false;
- }
+ // Half the width and height of the other box
+ float halfw = (v2.x() - v1.x()) / 2;
+ float halfh = (v2.y() - v1.y()) / 2;
 
+ float centerx = v1.x() + halfw;
+ float centery = v1.y() + halfh;
 
- if (!(RTTI::isUnit(rtti()) && RTTI::isUnit(item->rtti()))) {
-	// At least one item is not unit
-	// Non-unit items collide if their bounding boxes collide
+ // BB 1
+ float minx1, miny1, maxx1, maxy1;
+ minx1 = x() + xVelocity() - halfw;
+ miny1 = y() + xVelocity() - halfh + BO_TILE_SIZE;
+ maxx1 = x() + yVelocity() + width() + halfw;
+ maxy1 = y() + yVelocity() + height() + halfw - BO_TILE_SIZE;
+ if ((centerx > minx1) && (centerx < maxx1) && (centery > miny1) && (centery < maxy1)) {
+	// Box's center is in BB 1
+//	boDebug() << "        " << k_funcinfo << "Items COLLIDE (1)!!!" << endl;
 	return true;
  }
 
- // Both items are units
- // Units are special because they must be able to move diagonally. When moving
- //  diagonally, it's ok if their bounding boxes intersect on xy plane. Then we
- //  check if this item's center isn't inside other item's bounding rect
- // FIXME: maybe we need same for items. ATM, units can't diagonally cross tiles
- //  with mines
- return item->boundingRectAdvanced().contains(boundingRectAdvanced().center(), true);
+ // BB 2
+ float minx2, miny2, maxx2, maxy2;
+ minx2 = x() + xVelocity() - halfw + BO_TILE_SIZE;
+ miny2 = y() + xVelocity() - halfh;
+ maxx2 = x() + yVelocity() + width() + halfw - BO_TILE_SIZE;
+ maxy2 = y() + yVelocity() + height() + halfw;
+ if ((centerx > minx2) && (centerx < maxx2) && (centery > miny2) && (centery < maxy2)) {
+	// Box's center is in BB 2
+//	boDebug() << "        " << k_funcinfo << "Items COLLIDE (2)!!!" << endl;
+	return true;
+ }
+
+ // Check manhattan dist between centers
+ float mycenterx = x() + xVelocity() + width() / 2;
+ float mycentery = y() + yVelocity() + height() / 2;
+ if (QABS(mycenterx - centerx) + QABS(mycentery - centery) < (width() / 2 + halfw + height() / 2 + halfh - BO_TILE_SIZE)) {
+	// Box's center still collides with us
+//	boDebug() << "        " << k_funcinfo << "Items COLLIDE! (3)!!" << endl;
+	return true;
+ }
+
+// boDebug() << "        " << k_funcinfo << "Items don't collide" << endl;
+ return false;
 }
 
 void BosonItem::setAnimated(bool a)
 {
- if (mIsAnimated != a) {
-	mIsAnimated = a;
-	if (a) {
-		canvas()->addAnimation(this);
-	} else {
-		canvas()->removeAnimation(this);
-	}
- }
+	// obsolete!! remove!!
 }
 
 void BosonItem::select(bool markAsLeader)
@@ -380,11 +406,6 @@ void BosonItem::setCurrentFrame(BoFrame* frame)
  // inline functions (or with direct access). otherwise we'd have to #include
  // bosonmodel.h in bosonitem.h (-> bad)
 
-
- // TODO: BoFrame should support display lists with teamcolor! we call
- // frame->displayList(teamColor()) and BoFrame returns the list for the desired
- // color.
- setDisplayList(frame->displayList());
  setGLDepthMultiplier(frame->depthMultiplier());
 }
 
@@ -456,17 +477,8 @@ void BosonItem::renderItem(unsigned int lod)
 {
  BO_CHECK_NULL_RET(mModel);
  BO_CHECK_NULL_RET(mCurrentFrame);
- mModel->enablePointer();
- if (displayList() != 0) {
-#warning FIXME: displaylists and teamcolor!
-	// AB: we should pick the correct displaylist from a map
-	// teamcolor->displaylist!
-	// not here, but in setDisplayList() or friends
-	// FIXME: LOD
-	glCallList(displayList());
- } else {
-	mCurrentFrame->renderFrame(teamColor(), lod);
- }
+ mModel->prepareRendering();
+ mCurrentFrame->renderFrame(teamColor(), lod);
 }
 
 BosonCollisions* BosonItem::collisions() const
@@ -474,37 +486,35 @@ BosonCollisions* BosonItem::collisions() const
  return canvas()->collisions();
 }
 
-void BosonItem::moveParticleSystems(float x, float y, float z)
+void BosonItem::setEffectsPosition(float x, float y, float z)
 {
- if (particleSystems() && particleSystems()->count() > 0) {
+ if (effects() && effects()->count() > 0) {
 	BoVector3 pos(x + width() / 2, y + height() / 2, z);
 	pos.canvasToWorld();
-	QPtrListIterator<BosonParticleSystem> it(*particleSystems());
+	QPtrListIterator<BosonEffect> it(*effects());
 	for (; it.current(); ++it) {
 		it.current()->setPosition(pos);
 	}
  }
 }
 
-void BosonItem::rotateParticleSystems(float angle, float x, float y, float z)
+void BosonItem::setEffectsRotation(float xrot, float yrot, float zrot)
 {
- if (angle == 0.0) {
-	return;
- }
- if (particleSystems() && particleSystems()->count() > 0) {
-	QPtrListIterator<BosonParticleSystem> it(*particleSystems());
+ BoVector3 rot(xrot, yrot, zrot);
+ if (effects() && effects()->count() > 0) {
+	QPtrListIterator<BosonEffect> it(*effects());
 	for (; it.current(); ++it) {
-		it.current()->setRotation(-angle, x, y, z);
+		it.current()->setRotation(rot);
 	}
  }
 }
 
-const QPtrList<BosonParticleSystem>* BosonItem::particleSystems() const
+const QPtrList<BosonEffect>* BosonItem::effects() const
 {
  return 0l;
 }
 
-void BosonItem::clearParticleSystems()
+void BosonItem::clearEffects()
 {
 }
 
@@ -561,3 +571,18 @@ bool BosonItem::loadFromXML(const QDomElement& root)
  return true;
 }
 
+unsigned int BosonItem::lodCount() const
+{
+ if (!mModel) {
+	return 1;
+ }
+ return mModel->lodCount();
+}
+
+unsigned int BosonItem::preferredLod(float dist) const
+{
+ if (!mModel) {
+	return 0;
+ }
+ return mModel->preferredLod(dist);
+}
