@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,12 +21,12 @@
 
 #include "../global.h"
 #include "../bosoncanvas.h"
-#include "../bosonparticlesystem.h"
+#include "../bosoneffect.h"
 #include "../bosonweapon.h"
 #include "../player.h"
 #include "../unitproperties.h"
 #include "../boitemlist.h"
-#include "../bosonparticlesystemproperties.h"
+#include "../bosoneffectproperties.h"
 #include "../boson.h"
 #include "../speciestheme.h"
 #include "../unitproperties.h"
@@ -69,6 +69,10 @@ BosonShot::BosonShot(Player* owner, BosonCanvas* canvas) :
   mProp = 0;
 }
 
+BosonShot::~BosonShot()
+{
+}
+
 void BosonShot::init()
 {
   boDebug(350) << "MISSILE: " << k_funcinfo << "Creating new shot" << endl;
@@ -95,7 +99,6 @@ void BosonShot::init()
 
   // Initialization
   mActive = true;
-  setAnimated(true);
 }
 
 //AB: note this this is primary optimized for SAFETY, NOT for performance. it is
@@ -198,8 +201,6 @@ bool BosonShot::loadFromXML(const QDomElement& root)
   mActive = true; // Inactive shots won't be saved
   move(x, y, z);
 
-  // Is it ok to do this here?
-  setAnimated(true);
   return true;
 }
 
@@ -215,23 +216,23 @@ void BosonShot::explode()
 long int BosonShot::damage() const
 {
   return properties() ? properties()->damage() : 0;
-};
+}
 
 float BosonShot::damageRange() const
 {
   return properties() ? properties()->damageRange() : 0;
-};
+}
 
 float BosonShot::fullDamageRange() const
 {
   return properties() ? properties()->fullDamageRange() : 0;
-};
+}
 
 
 
 /*****  BosonShotBullet  *****/
 
-BosonShotBullet::BosonShotBullet(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 target) :
+BosonShotBullet::BosonShotBullet(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, const BoVector3& target) :
     BosonShot(owner, canvas, prop)
 {
   if(!properties())
@@ -262,16 +263,47 @@ void BosonShotBullet::moveToTarget()
   move(mTarget.x(), mTarget.y(), mTarget.z());
 }
 
+void BosonShotBullet::setTarget(const BoVector3& target)
+{
+  mTarget = target;
+}
+
 
 /*****  BosonShotMissile  *****/
 
-BosonShotMissile::BosonShotMissile(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 pos, BoVector3 target) :
+BosonShotMissile::BosonShotMissile(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, const BoVector3& pos, const BoVector3& target) :
     BosonShot(owner, canvas, prop)
+{
+  if(!properties())
+  {
+    boError(350) << k_funcinfo << "NULL weapon properties!" << endl;
+    setActive(false);
+    return;
+  }
+  mFlyEffects = 0;
+  init(pos, target);
+}
+
+BosonShotMissile::BosonShotMissile(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop) :
+    BosonShot(owner, canvas, prop)
+{
+  if(!properties())
+  {
+    boError(350) << k_funcinfo << "NULL weapon properties!" << endl;
+    setActive(false);
+    return;
+  }
+  mFlyEffects = 0;
+}
+
+BosonShotMissile::~BosonShotMissile()
+{
+ delete mFlyEffects;
+}
+
+void BosonShotMissile::init(const BoVector3& pos, const BoVector3& target)
 {
   boDebug(350) << "MISSILE: " << k_funcinfo << "Creating new shot" << endl;
-  //boDebug(350) << "    pos: (" << pos.x() << "; " << pos.y() << "; " << pos.z() << ");  target: (" <<
-  //    target.x() << "; " << target.y() << "; " << target.z() << "); this=" << this << endl;
-  mFlyParticleSystems = 0;
   mTarget = target;
   if(!properties())
   {
@@ -279,7 +311,7 @@ BosonShotMissile::BosonShotMissile(Player* owner, BosonCanvas* canvas, const Bos
     setActive(false);
     return;
   }
-  if(!canvas->onCanvas((int)pos[0], (int)pos[1]))
+  if(!canvas()->onCanvas((int)pos[0], (int)pos[1]))
   {
     boError(350) << k_funcinfo << "invalid start position" << endl;
     setActive(false);
@@ -303,10 +335,10 @@ BosonShotMissile::BosonShotMissile(Player* owner, BosonCanvas* canvas, const Bos
   mVelo.setZ(target.z() - pos.z());
   mVelo.scale(1 / mTotalDist);
 
-  // Particle systems
-  setParticleSystems(properties()->newFlyParticleSystems(pos, 0.0));
-  // FIXME: name: it has nothing to do with particles anymore
-  mParticleVelo = sqrt(mVelo[0] * mVelo[0] + mVelo[1] * mVelo[1]) / (float)BO_TILE_SIZE;
+  // Effects
+  setEffects(properties()->newFlyEffects(pos, 0.0));
+  // FIXME: name: it has nothing to do with effects anymore
+  mEffectVelo = sqrt(mVelo[0] * mVelo[0] + mVelo[1] * mVelo[1]) / (float)BO_TILE_SIZE;
 
   // Initialization
   move(pos[0], pos[1], pos[2]);
@@ -315,28 +347,11 @@ BosonShotMissile::BosonShotMissile(Player* owner, BosonCanvas* canvas, const Bos
   mPassedDist = 0;
 }
 
-BosonShotMissile::BosonShotMissile(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop) :
-    BosonShot(owner, canvas, prop)
+void BosonShotMissile::setEffects(const QPtrList<BosonEffect>& list)
 {
-  if(!properties())
-  {
-    boError(350) << k_funcinfo << "NULL weapon properties!" << endl;
-    setActive(false);
-    return;
-  }
-  mFlyParticleSystems = 0;
-}
-
-BosonShotMissile::~BosonShotMissile()
-{
- delete mFlyParticleSystems;
-}
-
-void BosonShotMissile::setParticleSystems(const QPtrList<BosonParticleSystem>& list)
-{
-  delete mFlyParticleSystems;
-  mFlyParticleSystems = new QPtrList<BosonParticleSystem>(list);
-  canvas()->addParticleSystems(*mFlyParticleSystems);
+  delete mFlyEffects;
+  mFlyEffects = new QPtrList<BosonEffect>(list);
+  canvas()->addEffects(*mFlyEffects);
 }
 
 // move the shot by one step
@@ -356,7 +371,7 @@ void BosonShotMissile::advanceMoveInternal()
   float zvelo = mVelo[2] * speed() + (addZ - mZ);
   mZ = addZ;
   setVelocity(mVelo[0] * speed(), mVelo[1] * speed(), zvelo);
-  setXRotation(Bo3dTools::rotationToPoint(mParticleVelo * speed(), zvelo) - 90 );
+  setXRotation(Bo3dTools::rotationToPoint(mEffectVelo * speed(), zvelo) - 90 );
 
   // Check if missile is still active
   BoVector3 dist = mTarget - BoVector3(x() + xVelocity(), y() + yVelocity(), z() + zVelocity());
@@ -484,7 +499,7 @@ bool BosonShotMissile::loadFromXML(const QDomElement& root)
 
   mVelo.set(xvelo, yvelo, zvelo);
   mTarget.set(targetx, targety, targetz);
-  mParticleVelo = sqrt(mVelo[0] * mVelo[0] + mVelo[1] * mVelo[1]) / (float)BO_TILE_SIZE;
+  mEffectVelo = sqrt(mVelo[0] * mVelo[0] + mVelo[1] * mVelo[1]) / (float)BO_TILE_SIZE;
   setRotation(Bo3dTools::rotationToPoint(mVelo[0], mVelo[1]));
   setSpeed(speed);
   setAccelerationSpeed(properties()->accelerationSpeed());
@@ -501,7 +516,7 @@ void BosonShotMissile::moveToTarget()
 
 /*****  BosonShotExplosion  *****/
 
-BosonShotExplosion::BosonShotExplosion(Player* owner, BosonCanvas* canvas, BoVector3 pos, long int damage, float damagerange, float fulldamagerange, int delay) :
+BosonShotExplosion::BosonShotExplosion(Player* owner, BosonCanvas* canvas, const BoVector3& pos, long int damage, float damagerange, float fulldamagerange, int delay) :
     BosonShot(owner, canvas)
 {
   activate(pos, damage, damagerange, fulldamagerange, delay);
@@ -590,12 +605,10 @@ void BosonShotExplosion::advanceMoveInternal()
 
 /*****  BosonShotMine  *****/
 
-BosonShotMine::BosonShotMine(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 pos) :
+BosonShotMine::BosonShotMine(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, const BoVector3& pos) :
     BosonShot(owner, canvas, prop)
 {
-  mActivated = false;
-  move(pos.x(), pos.y(), pos.z());
-  setVisible(true);
+  init(pos);
 }
 
 BosonShotMine::BosonShotMine(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop) :
@@ -603,6 +616,13 @@ BosonShotMine::BosonShotMine(Player* owner, BosonCanvas* canvas, const BosonWeap
 {
 }
 
+void BosonShotMine::init(const BoVector3& pos)
+{
+  mActivated = false;
+  move(pos.x(), pos.y(), pos.z());
+  setVisible(true);
+}
+
 bool BosonShotMine::saveAsXML(QDomElement& root)
 {
   if(!BosonShot::saveAsXML(root))
@@ -673,18 +693,14 @@ void BosonShotMine::advanceMoveInternal()
 
 /*****  BosonShotBomb  *****/
 
-BosonShotBomb::BosonShotBomb(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 pos) :
+BosonShotBomb::BosonShotBomb(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, const BoVector3& pos) :
     BosonShot(owner, canvas, prop)
 {
   // Speeds
   setAccelerationSpeed(properties()->accelerationSpeed());
   setMaxSpeed(properties()->speed());
 
-  // TODO: BosonShotBomb and BosonShotMine share quite a lot code. Maybe make
-  //  bomb inherit from mine (bomb is basically a falling mine)
-  mActivated = false;
-  move(pos.x(), pos.y(), pos.z());
-  setVisible(true);
+  init(pos);
 }
 
 BosonShotBomb::BosonShotBomb(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop) :
@@ -695,6 +711,15 @@ BosonShotBomb::BosonShotBomb(Player* owner, BosonCanvas* canvas, const BosonWeap
   setMaxSpeed(properties()->speed());
 }
 
+void BosonShotBomb::init(const BoVector3& pos)
+{
+  // TODO: BosonShotBomb and BosonShotMine share quite a lot code. Maybe make
+  //  bomb inherit from mine (bomb is basically a falling mine)
+  mActivated = false;
+  move(pos.x(), pos.y(), pos.z());
+  setVisible(true);
+}
+
 bool BosonShotBomb::saveAsXML(QDomElement& root)
 {
   if(!BosonShot::saveAsXML(root))
@@ -798,7 +823,7 @@ void BosonShotBomb::advanceMoveInternal()
 #define FRAGMENT_MAX_Z_SPEED 0.1
 #define FRAGMENT_GRAVITY -0.006
 
-BosonShotFragment::BosonShotFragment(Player* owner, BosonCanvas* canvas, BosonModel* model, BoVector3 pos,
+BosonShotFragment::BosonShotFragment(Player* owner, BosonCanvas* canvas, BosonModel* model, const BoVector3& pos,
     const UnitProperties* unitproperties) :
     BosonShot(owner, canvas, model)
 {
@@ -808,20 +833,20 @@ BosonShotFragment::BosonShotFragment(Player* owner, BosonCanvas* canvas, BosonMo
 BosonShotFragment::BosonShotFragment(Player* owner, BosonCanvas* canvas, BosonModel* model) :
     BosonShot(owner, canvas, model)
 {
-  mParticleSystems = 0;
+  mEffects = 0;
   mUnitProperties = 0;
 }
 
 BosonShotFragment::~BosonShotFragment()
 {
- delete mParticleSystems;
+ delete mEffects;
 }
 
-void BosonShotFragment::setParticleSystems(const QPtrList<BosonParticleSystem>& list)
+void BosonShotFragment::setEffects(const QPtrList<BosonEffect>& list)
 {
-  delete mParticleSystems;
-  mParticleSystems = new QPtrList<BosonParticleSystem>(list);
-  canvas()->addParticleSystems(*mParticleSystems);
+  delete mEffects;
+  mEffects = new QPtrList<BosonEffect>(list);
+  canvas()->addEffects(*mEffects);
 }
 
 void BosonShotFragment::activate(const BoVector3& pos, const UnitProperties* unitproperties)
@@ -835,7 +860,7 @@ void BosonShotFragment::activate(const BoVector3& pos, const UnitProperties* uni
   mVelo.setZ(FRAGMENT_MIN_Z_SPEED + (r->getDouble() * (FRAGMENT_MAX_Z_SPEED - FRAGMENT_MIN_Z_SPEED)));
   boDebug() << k_funcinfo << "Velocity is: (" << mVelo.x() << "; " << mVelo.y() << "; " << mVelo.z() << ")" << endl;
 
-  setParticleSystems(mUnitProperties->newExplodingFragmentFlyParticleSystems(BoVector3()));
+  setEffects(mUnitProperties->newExplodingFragmentFlyEffects(pos));
 
   move(pos.x(), pos.y(), pos.z() + 0.2);  // +0.2 prevents immediate contact with the terrain
   setRotation(Bo3dTools::rotationToPoint(mVelo.x(), mVelo.y()));
@@ -942,7 +967,7 @@ void BosonShotFragment::explode()
   BosonShot::explode();
 
   BoVector3 pos(x() + width() / 2, y() + height() / 2, z());
-  canvas()->addParticleSystems(mUnitProperties->newExplodingFragmentHitParticleSystems(pos));
+  canvas()->addEffects(mUnitProperties->newExplodingFragmentHitEffects(pos));
 }
 
 long int BosonShotFragment::damage() const
