@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,8 +27,8 @@
 
 class Unit;
 class Player;
-class BosonParticleSystem;
-class BosonParticleSystemProperties;
+class BosonEffect;
+class BosonEffectProperties;
 class BosonWeaponProperties;
 class UnitProperties;
 
@@ -75,12 +75,8 @@ class BosonShot : public BosonItem
     BosonShot(Player* owner, BosonCanvas* canvas, BosonModel* model);
 
     BosonShot(Player* owner, BosonCanvas* canvas);
+    ~BosonShot();
 
-    /**
-     * @return Owner of this shot, i.e. the player whose unit fired the shot.
-     * Used for statistics
-     **/
-    Player* owner() const { return mOwner; }
     /**
      * @return Weapon properties of this shot if it has one.
      * Note that it's perfectly legal to return NULL pointer here, so you should
@@ -163,7 +159,6 @@ class BosonShot : public BosonItem
     void init();
 
     bool mActive;
-    Player* mOwner;
     const BosonWeaponProperties* mProp;
 };
 
@@ -178,7 +173,7 @@ class BosonShot : public BosonItem
 class BosonShotBullet : public BosonShot
 {
   public:
-    BosonShotBullet(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 target);
+    BosonShotBullet(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, const BoVector3& target);
     BosonShotBullet(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
 
     // Bullets won't be saved becaused they will immediately "explode" and
@@ -190,6 +185,8 @@ class BosonShotBullet : public BosonShot
       return true;
     }
 
+    void setTarget(const BoVector3& target);
+
     inline virtual int type() const { return BosonShot::Bullet; }
 
   protected:
@@ -210,17 +207,21 @@ class BosonShotBullet : public BosonShot
 class BosonShotMissile : public BosonShot
 {
   public:
-    BosonShotMissile(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 pos, BoVector3 target);
+    BosonShotMissile(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, const BoVector3& pos, const BoVector3& target);
     BosonShotMissile(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
+    ~BosonShotMissile();
 
     virtual bool saveAsXML(QDomElement& root);
     virtual bool loadFromXML(const QDomElement& root);
 
-    virtual QPtrList<BosonParticleSystem>* particleSystems() const  { return mFlyParticleSystems; }
+    void init(const BoVector3& pos, const BoVector3& target);
+
+    virtual const QPtrList<BosonEffect>* effects() const  { return mFlyEffects; }
 
     inline virtual int type() const { return BosonShot::Missile; }
 
   protected:
+    void setEffects(const QPtrList<BosonEffect>& list);
     virtual void advanceMoveInternal();
 
     virtual void moveToTarget();
@@ -231,9 +232,9 @@ class BosonShotMissile : public BosonShot
     float mTotalDist;
     float mPassedDist;
     float mZ;
-    float mParticleVelo;
+    float mEffectVelo;
     float mMaxHeight;
-    QPtrList<BosonParticleSystem>* mFlyParticleSystems;
+    QPtrList<BosonEffect>* mFlyEffects;
 };
 
 
@@ -249,7 +250,7 @@ class BosonShotMissile : public BosonShot
 class BosonShotExplosion : public BosonShot
 {
   public:
-    BosonShotExplosion(Player* owner, BosonCanvas* canvas, BoVector3 pos, long int damage, float damagerange, float fulldamagerange, int delay);
+    BosonShotExplosion(Player* owner, BosonCanvas* canvas, const BoVector3& pos, long int damage, float damagerange, float fulldamagerange, int delay);
     BosonShotExplosion(Player* owner, BosonCanvas* canvas);
 
     void activate(const BoVector3& pos, long int damange, float damageRange, float fulldamagerange, int delay);
@@ -285,12 +286,14 @@ class BosonShotExplosion : public BosonShot
 class BosonShotMine : public BosonShot
 {
   public:
-    BosonShotMine(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 pos);
+    BosonShotMine(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, const BoVector3& pos);
     BosonShotMine(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
 
     virtual bool saveAsXML(QDomElement& root);
     virtual bool loadFromXML(const QDomElement& root);
 
+    void init(const BoVector3& pos);
+
     inline virtual int type() const { return BosonShot::Mine; }
 
   protected:
@@ -312,12 +315,14 @@ class BosonShotMine : public BosonShot
 class BosonShotBomb : public BosonShot
 {
   public:
-    BosonShotBomb(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 pos);
+    BosonShotBomb(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, const BoVector3& pos);
     BosonShotBomb(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
 
     virtual bool saveAsXML(QDomElement& root);
     virtual bool loadFromXML(const QDomElement& root);
 
+    void init(const BoVector3& pos);
+
     inline virtual int type() const { return BosonShot::Bomb; }
 
   protected:
@@ -338,9 +343,10 @@ class BosonShotBomb : public BosonShot
 class BosonShotFragment : public BosonShot
 {
   public:
-    BosonShotFragment(Player* owner, BosonCanvas* canvas, BosonModel* model, BoVector3 pos,
+    BosonShotFragment(Player* owner, BosonCanvas* canvas, BosonModel* model, const BoVector3& pos,
         const UnitProperties* unitproperties);
     BosonShotFragment(Player* owner, BosonCanvas* canvas, BosonModel* model);
+    ~BosonShotFragment();
 
     void activate(const BoVector3& pos, const UnitProperties* unitProperties);
 
@@ -353,15 +359,16 @@ class BosonShotFragment : public BosonShot
 
     inline virtual int type() const { return BosonShot::Fragment; }
 
-    virtual QPtrList<BosonParticleSystem>* particleSystems() const  { return mParticleSystems; }
+    virtual const QPtrList<BosonEffect>* effects() const  { return mEffects; }
 
     virtual void explode();
 
   protected:
+    void setEffects(const QPtrList<BosonEffect>& list);
     virtual void advanceMoveInternal();
 
   private:
-    QPtrList<BosonParticleSystem>* mParticleSystems;
+    QPtrList<BosonEffect>* mEffects;
     BoVector3 mVelo;
     const UnitProperties* mUnitProperties;
 };
