@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,12 +25,15 @@
 #include "boversion.h"
 #include "bodebug.h"
 #include "bo3dtools.h"
+#include <config.h>
 
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
 #include <klocale.h>
+#include <kmessagebox.h>
 
-#define HARDCODE_NOSOUND 1
+// sound is enabled by default atm
+#define HARDCODE_NOSOUND 0
 
 static const char *description =
     I18N_NOOP("A realtime strategy game for KDE");
@@ -47,6 +50,7 @@ static KCmdLineOptions options[] =
     { "new", I18N_NOOP("Skip Welcome Widget and display the New Game screen"), 0 },
     { "editor", I18N_NOOP("Skip Welcome Widget and display the Start Editor screen"), 0 },
     { "load", I18N_NOOP("Skip Welcome Widget and display the Load Game screen"), 0 },
+    { "load-from-log <file>", I18N_NOOP(""), 0 },
     { "playfield <identifier>", I18N_NOOP("Playfield identifier for newgame/start editor widget"), 0 },
     { "computer <count>", I18N_NOOP("Add (currently dummy) computer player"), 0 },
     { "start", I18N_NOOP("Start the game"), 0},
@@ -55,6 +59,7 @@ static KCmdLineOptions options[] =
     { "indirect", I18N_NOOP("Use Indirect rendering (sloooow!!). debugging only."), 0 },
     { "ati-depth-workaround", I18N_NOOP("Enable the ATI (proprietary) driver workaround for reading the depth buffer. Will use depth of 0.00390625"), 0 },
     { "ati-depth-workaround-depth <depth>", I18N_NOOP("Use with --ati-depth-workaround. Supply a depth value for your system (default=0.00390625)"), 0 },
+    { "default-lodcount <count>", I18N_NOOP("Use <count> for default level of detail count"), 0 },
     { 0, 0, 0 }
 };
 
@@ -69,7 +74,7 @@ int main(int argc, char **argv)
 		version,
 		description,
 		KAboutData::License_GPL,
-		"(C) 1999-2000,2001-2003 The Boson team",
+		"(C) 1999-2000,2001-2004 The Boson team",
 		0,
 		"http://boson.eu.org");
  about.addAuthor("Thomas Capricelli", I18N_NOOP("Initial Game Design & Coding"),
@@ -86,17 +91,32 @@ int main(int argc, char **argv)
 
  KCmdLineArgs::init(argc, argv, &about);
  KCmdLineArgs::addCmdLineOptions(options);
+#ifdef BOSON_COMPILE_STATIC
+ KApplication::disableAutoDcopRegistration();
+#endif
  BoApplication app;
  KGlobal::locale()->insertCatalogue("libkdegames");
 
     // register ourselves as a dcop client
 //    app.dcopClient()->registerAs(app.name(), false);
 
+ // make sure the data files are installed at the correct location
+ QString errorMessage = TopWidget::checkInstallation();
+ if (!errorMessage.isNull()) {
+	boError() << k_funcinfo << errorMessage << endl;
+	boError() << k_funcinfo << "check your installation!" << endl;
+	KMessageBox::sorry(0, errorMessage, i18n("Check your installation"));
+	return 1;
+ }
+
  TopWidget *top = new TopWidget;
  app.setMainWidget(top);
 
  top->show();
 
+ // pretend an old game was over. here we actually start
+ top->slotGameOver();
+
  if (boConfig->boolValue("EnableATIDepthWorkaround", false)) {
 	double depth = boConfig->doubleValue("ATIDepthWorkaroundValue", 0.00390625);
 	Bo3dTools::enableReadDepthBufferWorkaround((float)depth);
@@ -119,12 +139,26 @@ int main(int argc, char **argv)
 	}
 	Bo3dTools::enableReadDepthBufferWorkaround(depth);
  }
+ if (args->isSet("default-lodcount")) {
+	bool ok = false;
+	unsigned int v = 0;
+	QString s = args->getOption("default-lodcount");
+	v = s.toUInt(&ok);
+	if (!ok) {
+		boError() << k_funcinfo << "default-lodcount was not a valid number" << endl;
+		return 1;
+	}
+	boConfig->setDefaultLodCount(v);
+ }
  if (args->isSet("new")) {
 	top->slotNewGame(args);
  } else if (args->isSet("editor")) {
 	top->slotStartEditor(args);
  } else if (args->isSet("load")) {
 	top->slotLoadGame(args);
+ } else if (args->isSet("load-from-log")) {
+	QString file = args->getOption("load-from-log");
+	top->slotLoadFromLog(file);
  }
  args->clear();
  return app.exec();
