@@ -26,9 +26,15 @@
 #include "defines.h"
 #include "bodebug.h"
 #include "bogltooltip.h"
-#include "bogroundrenderer.h"
+#include "bogroundrenderermanager.h"
 #include "bo3dtools.h"
+#include "bofullscreen.h"
+#include "bomeshrenderermanager.h"
+#include "bosonfont/bosonglfont.h"
+#include "bosonfont/bosonglfontchooser.h"
+#include "info/boinfo.h"
 
+#include <kapplication.h>
 #include <klocale.h>
 #include <knuminput.h>
 #include <ksimpleconfig.h>
@@ -457,6 +463,25 @@ OpenGLOptions::OpenGLOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
  mEnableATIDepthWorkaround->setChecked(false);
  slotEnableATIDepthWorkaround(false);
 
+ QHBox* fontBox = new QHBox(this);
+ (void)new QLabel(i18n("Font: "), fontBox);
+ mFont = new QPushButton(fontBox);
+ mFontChanged = false;
+ mFontInfo = new BoFontInfo();
+ connect(mFont, SIGNAL(clicked()), this, SLOT(slotChangeFont()));
+
+ QHBox* resolutionBox = new QHBox(this);
+ (void)new QLabel(i18n("Resolution:"), resolutionBox);
+ mResolution = new QComboBox(resolutionBox);
+ mResolution->insertItem(i18n("Keep unchanged"));
+ if (BoFullScreen::availableModes().count() != 0) {
+	mResolution->insertStringList(BoFullScreen::availableModes());
+	mResolution->setEnabled(true);
+} else {
+	mResolution->setEnabled(false);
+ }
+ mResolution->setCurrentItem(0);
+
  QPushButton* showDetails = new QPushButton(i18n("Show &Details"), this);
  showDetails->setToggleButton(true);
  connect(showDetails, SIGNAL(toggled(bool)), this, SLOT(slotShowDetails(bool)));
@@ -506,21 +531,22 @@ OpenGLOptions::OpenGLOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
  QToolTip::add(mUseMaterials, i18n("Materials influence the way in which models (like units) are lighted. You can disable them to gain some performance."));
 
  hbox = new QHBox(mAdvanced);
- (void)new QLabel(i18n("Ground rendering method"), hbox);
+ (void)new QLabel(i18n("Ground render:"), hbox);
  mGroundRenderer = new QComboBox(hbox);
- for (int i = 0; i < BoGroundRenderer::Last; i++) {
-	mGroundRenderer->insertItem(BoGroundRenderer::rttiToName(i));
- }
- mGroundRenderer->setCurrentItem(0);
 
  mUseLOD = new QCheckBox(i18n("Use level of detail"), mAdvanced);
  hbox = new QHBox(mAdvanced);
- (void)new QLabel(i18n("Default level of detail"), hbox);
+ (void)new QLabel(i18n("Default level of detail:"), hbox);
  mDefaultLOD = new QComboBox(hbox);
  mDefaultLOD->insertItem(i18n("All details (default)"));
  mDefaultLOD->insertItem(i18n("Lowest details"));
  mDefaultLOD->setCurrentItem(0);
  connect(mUseLOD, SIGNAL(toggled(bool)), hbox, SLOT(setEnabled(bool)));
+ mSmoothShading = new QCheckBox(i18n("Smooth shade model"), mAdvanced);
+ mSmoothShading->setChecked(true);
+ hbox = new QHBox(mAdvanced);
+ (void)new QLabel(i18n("Mesh renderer:"), hbox);
+ mMeshRenderer = new QComboBox(hbox);
 }
 
 OpenGLOptions::~OpenGLOptions()
@@ -581,9 +607,12 @@ void OpenGLOptions::setRenderingSpeed(int speed)
 		setMipmapMinificationFilter(DEFAULT_MIPMAP_MINIFICATION_FILTER);
 		mUseLight->setChecked(DEFAULT_USE_LIGHT);
 		mUseMaterials->setChecked(DEFAULT_USE_MATERIALS);
-		mGroundRenderer->setCurrentItem(DEFAULT_GROUND_RENDERER);
+		setCurrentGroundRenderer(DEFAULT_GROUND_RENDERER);
+		setCurrentMeshRenderer(DEFAULT_MESH_RENDERER);
 		setUseLOD(true);
 		setDefaultLOD(0);
+		mSmoothShading->setChecked(true);
+		break;
 	case BestQuality:
 		// we mostly use defaults here.
 		setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
@@ -593,9 +622,11 @@ void OpenGLOptions::setRenderingSpeed(int speed)
 		setMipmapMinificationFilter(DEFAULT_MIPMAP_MINIFICATION_FILTER);
 		mUseLight->setChecked(DEFAULT_USE_LIGHT);
 		mUseMaterials->setChecked(DEFAULT_USE_MATERIALS);
-		mGroundRenderer->setCurrentItem(DEFAULT_GROUND_RENDERER);
+		setCurrentGroundRenderer(DEFAULT_GROUND_RENDERER);
+		setCurrentMeshRenderer(DEFAULT_MESH_RENDERER);
 		setUseLOD(true);
 		setDefaultLOD(0);
+		mSmoothShading->setChecked(true);
 		break;
 	case Fastest:
 		setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
@@ -605,9 +636,11 @@ void OpenGLOptions::setRenderingSpeed(int speed)
 		setMipmapMinificationFilter(GL_NEAREST);
 		mUseLight->setChecked(false);
 		mUseMaterials->setChecked(false);
-		mGroundRenderer->setCurrentItem(BoGroundRenderer::Fast);
+		setCurrentGroundRenderer("BoFastGroundRenderer");
+		setCurrentMeshRenderer(DEFAULT_MESH_RENDERER);
 		setUseLOD(true);
 		setDefaultLOD(5000);
+		mSmoothShading->setChecked(false);
 		break;
 	
  }
@@ -623,6 +656,17 @@ void OpenGLOptions::slotShowDetails(bool show)
  }
 }
 
+void OpenGLOptions::slotChangeFont()
+{
+ BoFontInfo f = *mFontInfo;
+ int result = BosonGLFontChooser::getFont(f, this);
+ if (result == QDialog::Accepted) {
+	*mFontInfo = f;
+	mFontChanged = true;
+	mFont->setText(mFontInfo->guiName());
+ }
+}
+
 void OpenGLOptions::apply()
 {
  boDebug(210) << k_funcinfo << endl;
@@ -669,14 +713,19 @@ void OpenGLOptions::apply()
 	}
  }
 
- boConfig->setUIntValue("GroundRenderer", mGroundRenderer->currentItem());
- emit signalGroundRendererChanged(mGroundRenderer->currentItem());
-
  boConfig->setAlignSelectionBoxes(mAlignSelectBoxes->isChecked());
  boConfig->setUseLight(mUseLight->isChecked());
  boConfig->setUseMaterials(mUseMaterials->isChecked());
  boConfig->setUseLOD(useLOD());
  boConfig->setUIntValue("DefaultLOD", defaultLOD());
+
+ if (mFontChanged) {
+	boConfig->setStringValue("GLFont", mFontInfo->toString());
+	mFontChanged = false;
+	emit signalFontChanged(*mFontInfo);
+ }
+
+
  boConfig->setBoolValue("EnableATIDepthWorkaround", mEnableATIDepthWorkaround->isChecked());
  if (mEnableATIDepthWorkaround->isChecked()) {
 	bool ok = false;
@@ -689,6 +738,36 @@ void OpenGLOptions::apply()
  } else {
 	Bo3dTools::disableReadDepthBufferWorkaround();
  }
+ boConfig->setBoolValue("SmoothShading", mSmoothShading->isChecked());
+
+ if (mResolution->isEnabled() && mResolution->currentItem() > 0) {
+	// first entry is "do not change", then a list, as provided by
+	// BoFullScreen
+	int index = mResolution->currentItem() - 1;
+	if (!BoFullScreen::enterMode(index)) {
+		boError() << k_funcinfo << "could not enter mode" << index << endl;
+	 }
+ }
+ mResolution->setCurrentItem(0);
+
+ if (!BoMeshRendererManager::manager()->makeRendererCurrent(mMeshRenderer->currentText())) {
+	KMessageBox::sorry(this, i18n("Failed at making mesh renderer %1 current. Trying to use a default renderer (expect a crash, if we fail)...").arg(mMeshRenderer->currentText()));
+	if (!BoMeshRendererManager::manager()->makeRendererCurrent(QString::null)) {
+		KMessageBox::sorry(this, i18n("Failed at setting a default mesh renderer. Quitting now"));
+		kapp->exit(1);
+	}
+ }
+ if (!BoGroundRendererManager::manager()->makeRendererCurrent(mGroundRenderer->currentText())) {
+	KMessageBox::sorry(this, i18n("Failed at making ground renderer %1 current. Trying to use a default renderer (expect a crash, if we fail)...").arg(mGroundRenderer->currentText()));
+	if (!BoGroundRendererManager::manager()->makeRendererCurrent(QString::null)) {
+		KMessageBox::sorry(this, i18n("Failed at setting a default ground renderer. Quitting now"));
+		kapp->exit(1);
+	}
+ }
+
+
+ emit signalOpenGLSettingsUpdated();
+
  boDebug(210) << k_funcinfo << "done" << endl;
 }
 
@@ -705,9 +784,12 @@ void OpenGLOptions::setDefaults()
  setAlignSelectionBoxes(DEFAULT_ALIGN_SELECTION_BOXES);
  mUseLight->setChecked(DEFAULT_USE_LIGHT);
  mUseMaterials->setChecked(DEFAULT_USE_MATERIALS);
- mGroundRenderer->setCurrentItem(DEFAULT_GROUND_RENDERER);
+ setCurrentGroundRenderer(DEFAULT_GROUND_RENDERER);
+ setCurrentMeshRenderer(DEFAULT_MESH_RENDERER);
  setUseLOD(DEFAULT_USE_LOD);
  setDefaultLOD(0);
+ mSmoothShading->setChecked(true);
+ mResolution->setCurrentItem(0);
 }
 
 void OpenGLOptions::load()
@@ -721,11 +803,21 @@ void OpenGLOptions::load()
  setAlignSelectionBoxes(boConfig->alignSelectionBoxes());
  mUseLight->setChecked(boConfig->useLight());
  mUseMaterials->setChecked(boConfig->useMaterials());
- mGroundRenderer->setCurrentItem(boConfig->uintValue("GroundRenderer"));
  setUseLOD(boConfig->useLOD());
  setDefaultLOD(boConfig->uintValue("DefaultLOD", 0));
  mEnableATIDepthWorkaround->setChecked(boConfig->boolValue("EnableATIDepthWorkaround"));
  mATIDepthWorkaroundValue->setText(QString::number(boConfig->doubleValue("ATIDepthWorkaroundValue")));
+ if (!mFontInfo->fromString(boConfig->stringValue("GLFont", QString::null))) {
+	boError() << k_funcinfo << "Could not load font " << boConfig->stringValue("GLFont", QString::null) << endl;
+	*mFontInfo = BoFontInfo();
+ }
+ mFont->setText(mFontInfo->guiName());
+ mSmoothShading->setChecked(boConfig->boolValue("SmoothShading", true));
+ mFontChanged = false;
+ mResolution->setCurrentItem(0);
+
+ setCurrentMeshRenderer(BoMeshRendererManager::manager()->currentRendererName());
+ setCurrentGroundRenderer(BoGroundRendererManager::manager()->currentRendererName());
 }
 
 void OpenGLOptions::setUpdateInterval(int ms)
@@ -881,6 +973,33 @@ void OpenGLOptions::slotATIDepthWorkaroundDefaultValue()
  mATIDepthWorkaroundValue->setText(QString::number(0.00390625));
 }
 
+void OpenGLOptions::setCurrentMeshRenderer(const QString& renderer)
+{
+ mMeshRenderer->clear();
+ QStringList renderers = BoMeshRendererManager::manager()->availableRenderers();
+ mMeshRenderer->insertStringList(renderers);
+
+ for (int i = 0; i < mMeshRenderer->count(); i++) {
+	if (mMeshRenderer->text(i) == renderer) {
+		mMeshRenderer->setCurrentItem(i);
+		return;
+	}
+ }
+}
+
+void OpenGLOptions::setCurrentGroundRenderer(const QString& renderer)
+{
+ mGroundRenderer->clear();
+ QStringList renderers = BoGroundRendererManager::manager()->availableRenderers();
+ mGroundRenderer->insertStringList(renderers);
+ for (int i = 0; i < mGroundRenderer->count(); i++) {
+	if (mGroundRenderer->text(i) == renderer) {
+		mGroundRenderer->setCurrentItem(i);
+		return;
+	}
+ }
+}
+
 //////////////////////////////////////////////////////////////////////
 // Chat Options
 //////////////////////////////////////////////////////////////////////
