@@ -64,6 +64,7 @@ public:
 	QBitArray mFogged; // TODO: use KGameProperty
 	KGameProperty<unsigned long int> mMinerals;
 	KGameProperty<unsigned long int> mOil;
+	KGamePropertyBool mIsNeutralPlayer;
 
 	BosonStatistics* mStatistics;
 
@@ -73,12 +74,11 @@ public:
 	PlayerIO* mPlayerIO;
 };
 
-Player::Player() : KPlayer()
+Player::Player(bool isNeutralPlayer) : KPlayer()
 {
  boDebug() << k_funcinfo << endl;
  mSpecies = 0;
  d = new PlayerPrivate;
- d->mUnits.setAutoDelete(true);
  setAsyncInput(true);
  connect(this, SIGNAL(signalNetworkData(int, const QByteArray&, Q_UINT32, KPlayer*)),
 		this, SLOT(slotNetworkData(int, const QByteArray&, Q_UINT32, KPlayer*)));
@@ -94,6 +94,9 @@ Player::Player() : KPlayer()
 		KGamePropertyBase::PolicyLocal, "MineralCost");
  d->mOil.registerData(IdOil, dataHandler(),
 		KGamePropertyBase::PolicyLocal, "OilCost");
+ d->mIsNeutralPlayer.registerData(IdIsNeutralPlayer, dataHandler(),
+		KGamePropertyBase::PolicyLocal, "IsNeutralPlayer");
+ d->mIsNeutralPlayer = isNeutralPlayer;
  d->mPlayerIO = new PlayerIO(this);
 
  quitGame(); // this will reset some variables
@@ -110,6 +113,11 @@ Player::~Player()
  boDebug() << k_funcinfo << "done" << endl;
 }
 
+bool Player::isNeutralPlayer() const
+{
+ return d->mIsNeutralPlayer;
+}
+
 PlayerIO* Player::playerIO() const
 {
  return d->mPlayerIO;
@@ -201,6 +209,12 @@ void Player::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 sende
 
 void Player::loadTheme(const QString& species, const QColor& teamColor)
 {
+ if (species == QString::fromLatin1("Neutral")) {
+	if (!isNeutralPlayer()) {
+		boError() << k_funcinfo << "only neutral player can have neutral theme" << endl;
+		return;
+	}
+ }
  delete mSpecies;
  mSpecies = new SpeciesTheme(species, teamColor);
 }
@@ -486,6 +500,10 @@ void Player::setMinerals(unsigned long int m)
 
 const QColor& Player::teamColor() const
 {
+ if (!speciesTheme()) {
+	BO_NULL_ERROR(speciesTheme());
+	return Qt::red;
+ }
  return speciesTheme()->teamColor();
 }
 
@@ -494,7 +512,14 @@ bool Player::hasMiniMap() const
  QPtrListIterator<Unit> it(d->mUnits);
  while (it.current()) {
 	if (it.current()->unitProperties()->supportMiniMap()) {
-		return true;
+		if (!it.current()->isFacility()) {
+			return true;
+		} else {
+			Facility* f = (Facility*)it.current();
+			if (f->isConstructionComplete()) {
+				return true;
+			}
+		}
 	}
 	++it;
  }
@@ -517,6 +542,12 @@ bool Player::checkOutOfGame()
 {
  // TODO: make more clever.
  // e.g. currently every small unit, like harvesters, needs to be destroyed...
+ if ((Player*)boGame->playerList()->getLast() == this) {
+	// the last player is per definition the neutral player (which is never
+	// out of the game).
+	mOutOfGame = false;
+	return isOutOfGame();
+ }
  mOutOfGame = (d->mUnits.count() == 0);
  return isOutOfGame();
 }
@@ -526,14 +557,39 @@ BosonStatistics* Player::statistics() const
  return d->mStatistics;
 }
 
-bool Player::isEnemy(Player* p) const
+bool Player::isEnemy(const Player* p) const
 {
- if (p == this) {
+ if (isAllied(p) || isNeutral(p)) {
 	return false;
  }
  return true;
 }
 
+bool Player::isNeutral(const Player* p) const
+{
+ // one day we might implement advanced relationships between players, so that
+ // for two players A,B there may be the following cases:
+ // 1. A and B are allies
+ // 2. A and B are enemies
+ // 3. A and B are neutral to each other
+
+ // atm only the "neutral player" is neutral to another player.
+ if ((const Player*)game()->playerList()->getLast() == p) {
+	return true;
+ }
+ return false;
+}
+
+bool Player::isAllied(const Player* p) const
+{
+ if (p == this) {
+	return true;
+ }
+
+ // allied players are not implemented yet
+ return false;
+}
+
 int Player::mobilesCount()
 {
  return d->mMobilesCount;
@@ -628,6 +684,10 @@ void Player::technologyResearched(ProductionPlugin*, unsigned long int id)
  }
 
  UpgradeProperties* prop = speciesTheme()->technology(id);
+ if (!prop) {
+	boError() << k_funcinfo << "NULL technology " << id << endl;
+	return;
+ }
  prop->setResearched(true);
  prop->apply(this);
 
@@ -643,10 +703,18 @@ bool Player::advanceFlag() const
 bool Player::saveAsXML(QDomElement& root)
 {
  BosonProfiler profiler(BosonProfiling::SavePlayerToXML);
+ if (!game() || !game()->playerList()) {
+	boError() << k_funcinfo << "NULL game/playerlist" << endl;
+	return false;
+ }
 
  // note: we need to save the index in the list, not the actual id()
  root.setAttribute(QString::fromLatin1("Id"), game()->playerList()->findRef(this));
 
+ if (game()->playerList()->findRef(this) == (int)game()->playerList()->count() - 1) {
+	root.setAttribute(QString::fromLatin1("IsNeutral"), 1);
+ }
+
  root.setAttribute(QString::fromLatin1("NetworkPriority"), networkPriority());
 
  QDomDocument doc = root.ownerDocument();
@@ -669,6 +737,21 @@ bool Player::saveAsXML(QDomElement& root)
 	boWarning() << k_funcinfo << "NULL speciestheme while saving into XML" << endl;
  }
 
+ QDomElement upgrades = doc.createElement("Upgrades");
+ if (speciesTheme()) {
+	QIntDict<UpgradeProperties> list = speciesTheme()->technologyList();
+	QIntDictIterator<UpgradeProperties> it(speciesTheme()->technologyList());
+	for (; it.current(); ++it) {
+		if (!it.current()->isResearched()) {
+			continue;
+		}
+		QDomElement e = doc.createElement("Researched");
+		e.setAttribute("Id", QString::number(it.current()->id()));
+		upgrades.appendChild(e);
+	}
+ }
+ root.appendChild(upgrades);
+
  // Save unitpropID
  root.setAttribute(QString::fromLatin1("UnitPropId"), (unsigned int)d->mUnitPropID);
 
@@ -689,6 +772,19 @@ bool Player::loadFromXML(const QDomElement& root)
  // this does NOT load the units!
 
  bool ok = false;
+ if (!game() || !game()->playerList()) {
+	boError() << k_funcinfo << "NULL game/playerlist" << endl;
+	return false;
+ }
+ if (root.hasAttribute("IsNeutral")) {
+	if (root.attribute("IsNeutral").toUInt(&ok) != 1) {
+		boError() << k_funcinfo << "invalid file format: IsNeutral must be 1 if it is present" << endl;
+		return false;
+	} else if (!ok) {
+		boError() << k_funcinfo << "invalid file format: IsNeutral must be a valid number if it is present" << endl;
+		return false;
+	}
+ }
 
  boDebug(260) << k_funcinfo << "load data handler" << endl;
  // load the data handler
@@ -699,6 +795,31 @@ bool Player::loadFromXML(const QDomElement& root)
 	return false;
  }
 
+ QDomElement upgrades = root.namedItem("Upgrades").toElement();
+ if (!upgrades.isNull() && speciesTheme()) {
+	QDomNodeList researched = upgrades.elementsByTagName("Researched");
+	for (unsigned int i = 0; i < researched.count(); i++) {
+		QDomElement e = researched.item(i).toElement();
+		bool ok = false;
+		unsigned int id = e.attribute("Id").toUInt(&ok);
+		if (!ok) {
+			boError() << k_funcinfo << "Id attribute of Researched tag is not a valid number" << endl;
+			continue;
+		}
+		UpgradeProperties* u = speciesTheme()->technology(id);
+		if (!u) {
+			boWarning() << k_funcinfo << "technology " << id << " not found" << endl;
+			continue;
+		}
+		if (u->isResearched()) {
+			boWarning() << k_funcinfo << "technology " << id << " already researched" << endl;
+			continue;
+		}
+		u->setResearched(true);
+		u->apply(this);
+	}
+ }
+
  if (root.hasAttribute(QString::fromLatin1("UnitPropId"))) {
 	d->mUnitPropID = root.attribute(QString::fromLatin1("UnitPropId")).toUInt(&ok);
 	if (!ok) {
