@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 
 #include "unitproperties.h"
 #include "bosonmodel.h"
-#include "bosonparticlesystemproperties.h"
+#include "bosoneffectproperties.h"
 #include "bodebug.h"
 #include "boaction.h"
 
@@ -44,13 +44,13 @@ public:
 	BosonModelFactory() { }
 	~BosonModelFactory() { }
 
-	BosonModel* createUnitModel(const UnitProperties* prop);
+	BosonModel* createUnitModel(const UnitProperties* prop, const QString& file);
 	BosonModel* createObjectModel(const KSimpleConfig* config, const QString& themePath);
 };
 
-BosonModel* BosonModelFactory::createUnitModel(const UnitProperties* prop)
+BosonModel* BosonModelFactory::createUnitModel(const UnitProperties* prop, const QString& file)
 {
- BosonModel* m = new BosonModel(prop->unitPath(), SpeciesData::unitModelFile(),
+ BosonModel* m = new BosonModel(prop->unitPath(), file,
 		((float)prop->unitWidth()) * BO_GL_CELL_SIZE / BO_TILE_SIZE,
 		((float)prop->unitHeight()) * BO_GL_CELL_SIZE / BO_TILE_SIZE);
  m->setLongNames(prop->longTextureNames());
@@ -91,9 +91,9 @@ BosonModel* BosonModelFactory::createObjectModel(const KSimpleConfig* config, co
 
 
 /**
- * By any reason QPixmap uses the alpha mask if existing, even if a custom 
- * mask using setMask() is supplied. We use this hack to delete the alpha mask 
- * if existing, so we can use our custom mask in 
+ * By any reason QPixmap uses the alpha mask if existing, even if a custom
+ * mask using setMask() is supplied. We use this hack to delete the alpha mask
+ * if existing, so we can use our custom mask in
  * BosonCommandWidget::advanceProduction()
  **/
 class OverviewPixmap : public QPixmap
@@ -101,7 +101,7 @@ class OverviewPixmap : public QPixmap
 public:
 	OverviewPixmap() : QPixmap() {}
 	OverviewPixmap(const QImage& image) : QPixmap(image) {}
-	void killAlphaMask() 
+	void killAlphaMask()
 	{
 		delete data->alphapm;
 		data->alphapm = 0;
@@ -120,7 +120,7 @@ public:
 	// we should separate our OpenGL context from BosonBigDisplay first
 	QString mThemePath;
 	QIntDict<TeamColorData> mTeamData;
-	QIntDict<BosonParticleSystemProperties> mParticleProperties;
+	QIntDict<BosonEffectProperties> mEffectProperties;
 	QDict<BoAction> mActions;
 	QDict<QPixmap> mPixmaps;
 
@@ -156,7 +156,7 @@ SpeciesData::SpeciesData(const QString& speciesPath)
 	return;
  }
  d = new SpeciesDataPrivate;
- d->mParticleProperties.setAutoDelete(true);
+ d->mEffectProperties.setAutoDelete(true);
  d->mTeamData.setAutoDelete(true);
  d->mUnitModels.setAutoDelete(true);
  d->mObjectModels.setAutoDelete(true);
@@ -168,7 +168,7 @@ SpeciesData::SpeciesData(const QString& speciesPath)
 SpeciesData::~SpeciesData()
 {
  boDebug() << k_funcinfo << endl;
- d->mParticleProperties.clear();
+ d->mEffectProperties.clear();
  d->mTeamData.clear();
  d->mUnitModels.clear();
  d->mObjectModels.clear();
@@ -200,6 +200,14 @@ SpeciesData* SpeciesData::speciesData(const QString& speciesDir)
  return species;
 }
 
+void SpeciesData::clearSpeciesData()
+{
+ if (!mSpeciesData) {
+	return;
+ }
+ mSpeciesData->clear();
+}
+
 QString SpeciesData::themePath() const
 {
  return d->mThemePath;
@@ -208,23 +216,32 @@ QString SpeciesData::themePath() const
 void SpeciesData::loadUnitModel(const UnitProperties* prop, const QColor& teamColor)
 {
  BO_CHECK_NULL_RET(prop);
- QString fileName = prop->unitPath() + unitModelFile();
- if (!KStandardDirs::exists(fileName)) {
-	boError() << k_funcinfo << "Cannot find " << unitModelFile() << " file for " << prop->typeId() << endl;
-	return;
- }
  BosonModel* m = d->mUnitModels[prop->typeId()];
 
  if (!m) {
+	QStringList fileNames = unitModelFiles();
+	bool found = false;
+	QString file;
+	for (QStringList::Iterator it = fileNames.begin(); it != fileNames.end(); ++it) {
+		if (KStandardDirs::exists(prop->unitPath() + *it)) {
+			file = *it;
+			found = true;
+			break;
+		}
+	}
+	if (!found) {
+		boError() << k_funcinfo << "Cannot find model file file for " << prop->typeId() << endl;
+		return;
+	}
+
 	BosonModelFactory factory;
-	m = factory.createUnitModel(prop);
+	m = factory.createUnitModel(prop, file);
 	d->mUnitModels.insert(prop->typeId(), m);
  } else {
 //	boDebug() << "model already loaded - loading an additional teamcolor only..." << endl;
 	// we only need to load the display lists here, as everything else
 	// doesn't depend on the teamcolor :)
  }
- m->createDisplayLists(&teamColor);
 }
 
 BosonModel* SpeciesData::unitModel(unsigned long int unitType) const
@@ -232,9 +249,12 @@ BosonModel* SpeciesData::unitModel(unsigned long int unitType) const
  return d->mUnitModels[unitType];
 }
 
-QString SpeciesData::unitModelFile()
+QStringList SpeciesData::unitModelFiles()
 {
- return QString::fromLatin1("unit.3ds");
+ QStringList list;
+ list.append("unit.3ds");
+ list.append("unit.ac");
+ return list;
 }
 
 SpeciesData::TeamColorData* SpeciesData::teamColorData(const QColor& color) const
@@ -276,8 +296,11 @@ bool SpeciesData::loadUnitOverview(const UnitProperties* prop, const QColor& tea
 	QImage image;
 	if (!loadUnitImage(teamColor, path + "overview-big.png", image)) {
 		boError() << k_funcinfo << "Can't load " << path + "overview-big.png" << endl;
-		return false;
+		image = QImage(100, 100, 32);
+		image.fill(Qt::red.rgb());
 	}
+	// we use 100x100 images for big overviews.
+	image = image.smoothScale(100, 100, QImage::ScaleMin);
 	QPixmap* p = new QPixmap(image);
 	data->mBigOverview.insert(type, p);
  }
@@ -289,8 +312,10 @@ bool SpeciesData::loadUnitOverview(const UnitProperties* prop, const QColor& tea
 	QImage image;
 	if (!loadUnitImage(teamColor, path + "overview-small.png", image)) {
 		boError() << k_funcinfo << "Can't load " << path + "overview-small.png" << endl;
-		return false;
+		image = QImage(50, 50, 32);
+		image.fill(Qt::red.rgb());
 	}
+	image = image.smoothScale(50, 50, QImage::ScaleMin);
 	OverviewPixmap* p = new OverviewPixmap(image);
 	p->killAlphaMask();
 	data->mSmallOverview.insert(type, p);
@@ -350,7 +375,7 @@ bool SpeciesData::loadUnitImage(const QColor& teamColor, const QString &fileName
  if (withMask) {
 	QBitmap m;
 	m.convertFromImage(*mask);
-	pix.setMask( m ); 
+	pix.setMask( m );
  }
  delete mask;
  _image = pix.convertToImage();
@@ -380,54 +405,67 @@ QPixmap* SpeciesData::smallOverview(unsigned long int unitType, const QColor& te
 }
 
 
-void SpeciesData::loadParticleSystemProperties()
+void SpeciesData::loadEffectProperties()
 {
- if (d->mParticleProperties.count() > 0) {
+ if (d->mEffectProperties.count() > 0) {
 	// already loaded, probably for another player
 	return;
  }
- BosonParticleSystemProperties::initStatic(themePath() + "/particles");
- QString fileName = themePath() + QString::fromLatin1("particles/particles.boson");
+ BosonEffect::initStatic(themePath() + "/particles");
+ QString fileName = themePath() + QString::fromLatin1("effects.boson");
  if (!KStandardDirs::exists(fileName)) {
-	boDebug() << k_funcinfo << "no particles.boson file found" << endl;
-	// We assume that this theme has no particles and don't complain
+	boDebug() << k_funcinfo << "no effects.boson file found" << endl;
+	// We assume that this theme has no effects and don't complain
 	return;
  }
  KSimpleConfig cfg(fileName);
- QStringList particles = cfg.groupList();
- if (particles.isEmpty()) {
-	boWarning() << k_funcinfo << "No particle systems found in particles "
+ QStringList effects = cfg.groupList();
+ if (effects.isEmpty()) {
+	boWarning() << k_funcinfo << "No effects found in effects "
 			<< "file (" << fileName << ")" << endl;
 	return;
  }
- boDebug(150) << k_funcinfo << "Loading " << particles.count()
-		<< " particle systems from config file" << endl;
+ boDebug(150) << k_funcinfo << "Loading " << effects.count()
+		<< " effects from config file" << endl;
  QStringList::Iterator it;
- for (it = particles.begin(); it != particles.end(); ++it) {
-	boDebug(150) << k_funcinfo << "Loading particle system from group " << *it << endl;
-	BosonParticleSystemProperties* particleprop = new BosonParticleSystemProperties(&cfg, *it);
-	if (!d->mParticleProperties.find(particleprop->id())) {
-		d->mParticleProperties.insert(particleprop->id(), particleprop);
-	} else {
-		boError(150) << k_funcinfo << "particle system with id " << particleprop->id() << " already there!" << endl;
+ for (it = effects.begin(); it != effects.end(); ++it) {
+	boDebug(150) << k_funcinfo << "Loading effect from group " << *it << endl;
+	// TODO: Is it possible to remove 'this' from here (used by
+	//  BosonEffectPropertiesCollection)
+	BosonEffectProperties* effectprop = BosonEffectPropertiesFactory::loadEffectProperties(&cfg, *it);
+	if (!effectprop) {
+		// Error has already been given
+		continue;
 	}
+	if (!d->mEffectProperties.find(effectprop->id())) {
+		d->mEffectProperties.insert(effectprop->id(), effectprop);
+	} else {
+		boError(150) << k_funcinfo << "effect with id " << effectprop->id() << " already there!" << endl;
+	}
+ }
+ // BosonEffectProperties (more specifically, collection properties) need 2-wave
+ //  loading
+ QIntDictIterator<BosonEffectProperties> eit(d->mEffectProperties);
+ while (eit.current()) {
+	eit.current()->finishLoading(this);
+	++eit;
  }
 }
 
-const BosonParticleSystemProperties* SpeciesData::particleSystemProperties(unsigned long int id) const
+const BosonEffectProperties* SpeciesData::effectProperties(unsigned long int id) const
 {
  if (id == 0) {
 	// We don't print error here because 0 means "none" in configurations
 	return 0;
  }
- if (!d->mParticleProperties[id]) {
-	boError() << k_funcinfo << "oops - no particle system properties for " << id << endl;
+ if (!d->mEffectProperties[id]) {
+	boError() << k_funcinfo << "oops - no effect properties for " << id << endl;
 	return 0;
  }
- return d->mParticleProperties[id];
+ return d->mEffectProperties[id];
 }
 
-BoAction* SpeciesData::action(const QString& name) const
+const BoAction* SpeciesData::action(const QString& name) const
 {
  BoAction* action = d->mActions[name];
  if (!action) {
@@ -477,10 +515,8 @@ void SpeciesData::loadObjects(const QColor& teamColor)
 		m = factory.createObjectModel(&cfg, themePath());
 		d->mObjectModels.insert(*it, m);
 	} else {
-		// nothing special to do here - just load the additional display
-		// lists for the new teamcolor.
+		// nothing special to do here
 	}
-	m->createDisplayLists(&teamColor);
  }
 }
 
