@@ -411,6 +411,7 @@ void BosonNewEditorWidget::slotEditExistingToggled(bool checked)
 
 QByteArray BosonNewEditorWidget::createNewMap()
 {
+ boDebug() << k_funcinfo << endl;
  unsigned int width = mWidth->value();
  unsigned int height = mHeight->value();
  if (!BosonMap::isValidMapGeo(width, height)) {
@@ -468,7 +469,7 @@ QByteArray BosonNewEditorWidget::createNewMap()
  save->setPlayField(&playField);
  boDebug() << k_funcinfo << "saving to playfield completed" << endl;
  QMap<QString, QByteArray> files;
- if (!save->saveToFiles(files, 0)) {
+ if (!save->saveToFiles(files)) {
 	delete save;
 	boError() << k_funcinfo << "error occured while saving" << endl;
 	KMessageBox::sorry(this, i18n("An error occured while saving the a game to a stream"));
@@ -484,7 +485,7 @@ QByteArray BosonNewEditorWidget::createNewMap()
  playersDoc.appendChild(playersRoot);
  canvasDoc.appendChild(canvasRoot);
  canvasRoot.appendChild(canvasDoc.createElement(QString::fromLatin1("DataHandler")));
- for (int i = 0; i < maxPlayers; i++) {
+ for (int i = 0; i < maxPlayers + 1; i++) {
 	QDomElement p = playersDoc.createElement(QString::fromLatin1("Player"));
 	p.setAttribute("Id", i);
 	playersRoot.appendChild(p);
@@ -492,11 +493,18 @@ QByteArray BosonNewEditorWidget::createNewMap()
 	QDomElement items = canvasDoc.createElement(QString::fromLatin1("Items"));
 	items.setAttribute("Id", i);
 	canvasRoot.appendChild(items);
+
+	if (i == maxPlayers) {
+		p.setAttribute("IsNeutral", 1);
+	}
  }
-// playersRoot.setAttribute("LocalPlayerId", );
  files.insert("players.xml", playersDoc.toCString());
  files.insert("canvas.xml", canvasDoc.toCString());
 
+ BPFDescription desc;
+ desc.setName(mNewMapName->text());
+ files.insert("C/description.xml", desc.toString().utf8());
+
  QByteArray b = BosonPlayField::streamFiles(files);
  boDebug() << k_funcinfo << "files got streamed" << endl;
  return b;
