@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -46,7 +46,6 @@
 #include <klistbox.h>
 #include <ktextbrowser.h>
 #include <kcombobox.h> // can we use qcombobox instead?
-#include <kgamemisc.h>
 
 #include <qcombobox.h>
 #include <qframe.h>
@@ -56,6 +55,7 @@
 #include <qpushbutton.h>
 #include <qlayout.h>
 #include <qpainter.h>
+#include <qguardedptr.h>
 
 
 /**
@@ -114,6 +114,7 @@ class BosonNewGameWidgetPrivate
 public:
 	BosonNewGameWidgetPrivate()
 	{
+		mLocalPlayer = 0;
 	}
 
 	QPtrDict<KPlayer> mItem2Player;
@@ -121,6 +122,8 @@ public:
 
 	QMap<int, QString> mSpeciesIndex2Identifier;
 	QMap<int, QString> mSpeciesIndex2Comment;
+
+	QGuardedPtr<Player> mLocalPlayer;
 };
 
 
@@ -196,9 +199,18 @@ BosonNewGameWidget::~BosonNewGameWidget()
  delete d;
 }
 
-void BosonNewGameWidget::initPlayer()
+void BosonNewGameWidget::setLocalPlayer(Player* p)
 {
- boDebug() << k_funcinfo << "playerCount(): " << boGame->playerCount() << endl;
+ d->mLocalPlayer = p;
+ if (d->mLocalPlayer && !d->mLocalPlayer->speciesTheme()) {
+	initLocalPlayer();
+	mChat->chatWidget()->setFromPlayer(p);
+ }
+}
+
+void BosonNewGameWidget::initLocalPlayer()
+{
+ boDebug() << k_funcinfo << endl;
  if (!localPlayer()) {
 	boWarning() << k_funcinfo << "waiting for local player to join first" << endl;
 	return;
@@ -280,7 +292,6 @@ void BosonNewGameWidget::initPlayFields(BosonCampaign* campaign, QListViewItem*
 	parent->setText(0, campaign->name());
  }
  QStringList list = campaign->playFields();
- boDebug() << list.count() << endl;
  for (unsigned int i = 0; i < list.count(); i++) {
 	QListViewItem* item = new QListViewItem(parent);
 	BosonPlayField* field = boData->playField(list[i]);
@@ -306,11 +317,15 @@ void BosonNewGameWidget::initSpecies()
  for (unsigned int i = 0; i < list.count(); i++) {
 	KSimpleConfig cfg(list[i]);
 	cfg.setGroup("Boson Species");
+	QString identifier = cfg.readEntry("Identifier", "Unknown");
+	if (identifier == QString::fromLatin1("Neutral")) {
+		continue;
+	}
 	mPlayerSpecies->insertItem(cfg.readEntry("Name", i18n("Unknown")), i);
 	// comments aren't used. Maybe remove this dict?
 	d->mSpeciesIndex2Comment.insert(i, cfg.readEntry("Comment", i18n("None")));
-	d->mSpeciesIndex2Identifier.insert(i, cfg.readEntry("Identifier", "Unknown"));
-	if (cfg.readEntry("Identifier", "Unknown") == SpeciesTheme::defaultSpecies()) {
+	d->mSpeciesIndex2Identifier.insert(i, identifier);
+	if (identifier == SpeciesTheme::defaultSpecies()) {
 		defaultIndex = i;
 	}
  }
@@ -424,6 +439,12 @@ void BosonNewGameWidget::slotNetStart()
 void BosonNewGameWidget::slotNetPlayerJoinedGame(KPlayer* p)
 {
  boDebug() << k_funcinfo << endl;
+ BO_CHECK_NULL_RET(p);
+ if (((Player*)p)->isNeutralPlayer()) {
+	// Don't show neutral player in players' list
+	boDebug() << k_funcinfo << "Added player is neutral player. Returning" << endl;
+	return;
+ }
  QListBoxPlayerText* t = new QListBoxPlayerText(p->name());
  t->setColor(((Player*)p)->teamColor());
  d->mItem2Player.insert(t, p);
@@ -473,6 +494,13 @@ void BosonNewGameWidget::slotNetPlayerLeftGame(KPlayer* p)
 {
  boDebug() << k_funcinfo << "there are " << boGame->playerList()->count() << " players in game now" << endl;
 
+ if (((Player*)p)->isNeutralPlayer()) {
+	// Neutral player shouldn't be removed because it's not added until the game
+	//  starts.
+	boError() << k_funcinfo << "Neutral player removed from game?!" << endl;
+	return;
+ }
+
  if (p == (KPlayer*)localPlayer() && boGame->isNetwork()) {
 	// Localplayer was removed and w have a net game.
 	// Either player was kicked by ADMIN or we're connecting to net game
@@ -658,18 +686,6 @@ void BosonNewGameWidget::slotNetPlayerNameChanged(Player* p)
 
 void BosonNewGameWidget::slotNetSetLocalPlayer(Player* p)
 {
- boDebug() << k_funcinfo << endl;
- if (!p) {
-	// AB: a NULL local player is totally valid!
-	// we are meant to unset the player now
-
-	// AB: this is bad... KGameChat doesn't allow NULL players...
-	// how can we unset the player from there?
-	boDebug() << k_funcinfo << "unset local player" << endl;
-	return;
- }
- initPlayer();
- mChat->chatWidget()->setFromPlayer(p);
 }
 
 void BosonNewGameWidget::slotNetSetAdmin(bool admin)
@@ -792,7 +808,11 @@ void BosonNewGameWidget::slotAddComputerPlayer()
 
  Player* p = new Player();
 
- p->setName(KGameMisc::randomName());
+ // AB: we don't have access to libkdegames anymore. one day we will have
+ // computer players with names that fit to the story (if we'll ever have one).
+ // so the randomName() isn't required anyway.
+// p->setName(KGameMisc::randomName());
+ p->setName(i18n("Player %1").arg(boGame->playerCount()));
 
  // the color is dangerous concerning network and so!
  // it'd be better to first add the player and then change the color using a
@@ -824,6 +844,7 @@ void BosonNewGameWidget::slotPlayerSelected(QListBoxItem* item)
 	mRemovePlayer->setEnabled(false);
 	return;
  }
+ BO_CHECK_NULL_RET(mSelectedPlayer->speciesTheme());
 
  // Update name, color, species
  mPlayerName->setText(mSelectedPlayer->name());
@@ -851,10 +872,7 @@ void BosonNewGameWidget::slotPlayerSelected(QListBoxItem* item)
 
 Player* BosonNewGameWidget::localPlayer() const
 {
- // AB: I don't like to using boGame->localPlayer(). i consider this unclean,
- // but it definitely saves a lot of trouble, as we set the localplayer in
- // boGame anyway. no need to duplicate that stuff here.
- return boGame->localPlayer();
+ return d->mLocalPlayer;
 }
 
 void BosonNewGameWidget::slotCancel()
