@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,8 +31,9 @@ class Facility;
 class ProductionPlugin;
 class UnitPlugin;
 class RepairPlugin;
-class BosonParticleSystem;
+class BosonEffect;
 class BosonWeapon;
+class BosonPathInfo;
 class BoVector3;
 template<class T> class QValueList;
 template<class T> class QPtrList;
@@ -56,18 +57,10 @@ public:
 	enum PropertyIds {
 		// properties in Unit
 		IdWaypoints = UnitBase::IdLast + 2,
-		IdMoveDestX = UnitBase::IdLast + 3,
-		IdMoveDestY = UnitBase::IdLast + 4,
-		IdMoveRange = UnitBase::IdLast + 5,
 		IdWantedRotation = UnitBase::IdLast + 6,
-		IdMoveAttacking = UnitBase::IdLast + 7,
-		IdSearchPath = UnitBase::IdLast + 8,
-		IdSlowDownAtDestination = UnitBase::IdLast + 9,
+		IdPathPoints = UnitBase::IdLast + 10,
 
 		// properties in MobileUnit
-		IdMovingFailed = UnitBase::IdLast + 51,
-		IdPathRecalculated = UnitBase::IdLast + 52,
-		IdPathAge = UnitBase::IdLast + 53,
 
 		// properties in Facility
 		IdConstructionStep = UnitBase::IdLast + 100,
@@ -80,7 +73,9 @@ public:
 		IdHarvestingType = UnitBase::IdLast + 304,
 		IdBombingPosX = UnitBase::IdLast + 305,
 		IdBombingPosY = UnitBase::IdLast + 306,
-		IdMinePlacingCounter = UnitBase::IdLast + 307
+		IdMinePlacingCounter = UnitBase::IdLast + 307,
+		IdResourceMineMinerals = UnitBase::IdLast + 308,
+		IdResourceMineOil = UnitBase::IdLast + 309
 	};
 
 	Unit(const UnitProperties* prop, Player* owner, BosonCanvas* canvas);
@@ -97,7 +92,7 @@ public:
 	 * none. Always NULL if @ref work != WorkPlugin
 	 **/
 	inline UnitPlugin* currentPlugin() const { return mCurrentPlugin; }
-	
+
 	/**
 	 * @return The @refUnitPlugin::pluginType of the @ref currentPlugin or
 	 * 0 if there is no plugin.
@@ -112,10 +107,23 @@ public:
 	virtual void setWork(WorkType w);
 	void setPluginWork(int pluginType);
 
+	/**
+	 * We are about to remove @p item from the game. Make sure that this
+	 * unit does not use that item in any way (e.g. as a target)
+	 *
+	 * -> after this method was called @p item will become an invalid
+	 *  pointer, so all references to that pointer must be set to NULL
+	 **/
+	virtual void itemRemoved(BosonItem* item);
+
 
 	virtual void select(bool markAsLeader = false);
 
 	/**
+	 * Moves unit by specified amount and also calculates new z-coordinate and
+	 * rotation.
+	 * This should only be called by BosonCanvas' advance methods.
+	 *
 	 * Note that we use float all over in boson, since mesa uses float
 	 * internally. We won't gain precision by double but we will lose some
 	 * performance
@@ -127,7 +135,7 @@ public:
 	 * simple as possible as it's called for <em>every</em> unit on
 	 * <em>every</em> advance call. An example of what could be done here is
 	 * to increase the realod state.
-	 * 
+	 *
 	 * The really interesting things get done in the advanceXYZ() functions
 	 * below. They are called from @ref BosonCanvas::slotAdvance(). For
 	 * every @ref UnitBase::WorkType there is at least one advance function which
@@ -169,7 +177,7 @@ public:
 		(this->*mAdvanceFunction2)(advanceCount);
 	}
 
-	
+
 	/**
 	 * Synchronize the advance function variable. This needs to be called
 	 * after @ref advanceFunction has been called for <em>all</em> units.
@@ -228,7 +236,7 @@ public:
 	 * default - reimplemented in @ref Facility
 	 **/
 	virtual void advanceConstruction(unsigned int) { }
-	
+
 	/**
 	 * Called when the unit has been destroyed. Maybe compute a destruction
 	 * animation here, or animate the wreckage.
@@ -248,6 +256,12 @@ public:
 	 **/
 	virtual void advanceTurn(unsigned int);
 
+	/**
+	 * @return Target of the unit.
+	 * Note that target is not neccessarily the unit that this unit attacks - it
+	 * is also used for following. In this case, the followed unit will be target
+	 * for the followers.
+	 **/
 	Unit* target() const;
 	virtual void setTarget(Unit* target);
 
@@ -255,37 +269,31 @@ public:
 
 // waypoint stuff: // also in facility - produced units receive this initial waypoint
 	/**
-	 * Add pos to the waypoint list. Please note that waypoints are actually
-	 * added later. addWaypoint() sends the point over network and only as
-	 * soon as it is received from there you can work with it. That means,
-	 * that after calling addWaypoint() @ref waypointCount has not yet been
-	 * increased!
-	 * UPDATE: this documentation is outdated! see source code
+	 * Add pos to the waypoint list.
 	 **/
 	void addWaypoint(const QPoint& pos);
-	
+
 	const QPoint& currentWaypoint() const;
 	unsigned int waypointCount() const;
-
 	/**
-	 * Removes all waypoints from the list. Note that this is done
-	 * <em>immediately</em> - in contrary to @ref addWaypoint this is not
-	 * sent over network as all function calling clearWaypoints() are
-	 * already called on all clients!
-	 * UPDATE: this documentation is outdated! see source code
+	 * Removes all waypoints from the list.
 	 **/
 	void clearWaypoints();
 
 	/**
-	 * Remove the first waypoint from the list. Note that this is done
-	 * <em>immediately</em>. See @ref clearWaypoints for more infos.
+	 * Remove the first waypoint from the list.
 	 **/
 	void waypointDone();
 
 	/**
 	 * @return A list of all waypoints for debugging
 	 **/
-	QValueList<QPoint> waypointList() const;
+	const QValueList<QPoint>& waypointList() const;
+
+	/**
+	 * @return List of path-points
+	 **/
+	const QValueList<QPoint>& pathPointList() const;
 
 	/**
 	 * Move this unit to a specified point. Also make sure that previous
@@ -295,30 +303,32 @@ public:
 	 * @param attack If this is true, unit will stop and attack any enemy units in range while moving
 	 **/
 	virtual void moveTo(const QPoint& pos, bool attack = false);
-	
+
+
+	// TODO: maybe make this protected?
 	/**
-	 * Nearly similar to the above version (actually this is called by the
-	 * above) but any previous @ref work is <em>not</em> cleared. This way
-	 * you can call this while e.g. a unit is being attacked and actually
-	 * start to attack as soon as the unit is in range. We also assume that
-	 * the @ref setAnimated was already called.
-	 * @param x The destination x-coordinate on the canvas
-	 * @param y The destination y-coordinate on the canvas
-	 * @param range Number of tiles around the destination the unit is
-	 * allowed to go to. You must use range > 0 e.g. if destination is
-	 * occupied and you should use e.g. @ref weaponRange if the unit should
-	 * attack. -1 keeps the previously set range.
-	 * @param attack Whether to stop and attack any enemy units in range while moving
-	 * @param slowDownAtDetination Whether to slow down before reaching detination point
-	 * @return true if unit can go to destination, false otherwise
+	 * Internal moving method.
+	 * This is the most central method for having unit starting moving to specific
+	 * point. You shouldn't use this directly, but instead use 'client methods',
+	 * which will then call this one.
+	 *
+	 * Moves unit's _center_ exactly to given position. If range is not 0, unit
+	 * will move until it's less that range _cells_ away from destination.
+	 * Destination is in canvas coords.
 	 **/
-	bool moveTo(float x, float y, int range = 0, bool attack = false, bool slowDownAtDetination = true);
+	bool moveTo(float x, float y, int range = 0);
 
 	/**
 	 * Turns unit smoothly to given degrees
 	 **/
 	virtual void turnTo(int degrees);
 
+	/**
+	 * Updates unit's x- and y-rotation, so that it will be rotated accordingly to
+	 * to the slope of the terrain it's on
+	 **/
+	void updateRotation();
+
 	/**
 	 * Just stop moving. Don't call this if you don't want to stop attacking
 	 * as well! This sets @ref work to @ref WorkNone
@@ -349,10 +359,47 @@ public:
 	 **/
 	virtual void setAdvanceWork(WorkType w);
 
+
+	/**
+	 * @return X-coordinate of unit's current destination (where it's moving).
+	 * If unit isn't moving, returned value is undefined.
+	 * Returned value is in canvas coordinates.
+	 **/
 	int destinationX() const;
+	/**
+	 * @return Y-coordinate of unit's current destination (where it's moving).
+	 * If unit isn't moving, returned value is undefined.
+	 * Returned value is in canvas coordinates.
+	 **/
 	int destinationY() const;
+	/**
+	 * @return Current moving range of the unit (how close to destination point it
+	 *  must move).
+	 * If unit isn't moving, returned value is undefined.
+	 * Returned value is in cell coordinates.
+		**/
 	int moveRange() const;
 
+	/**
+	 * @return Whether unit should attack any enemy units in range while moving.
+	 **/
+	bool moveAttacking() const;
+
+	/**
+	 * @return Whether unit should slow down (instead of immediately stopping)
+	 *  before moving destination is reached.
+	 **/
+	bool slowDownAtDestination() const;
+
+	/**
+	 * @return Current pathinfo structure for this unit.
+	 * Note that this is quite internal and shouldn't be used by anything else
+	 * than Unit and it's inheritants if possible.
+	 * It may be made protected later.
+	 **/
+	BosonPathInfo* pathInfo() const;
+
+
 	/**
 	 * @return TRUE if this unit is next to unit (i.e. less than one cell
 	 * distance) otherwise FALSE
@@ -362,17 +409,17 @@ public:
 	void playSound(UnitSoundEvent event);
 
 	/**
-	 * @return List of active particle systems this unit has.
+	 * @return List of active effects this unit has.
 	 * This may include e.g. smoke for factories.
 	 **/
-	virtual const QPtrList<BosonParticleSystem>* particleSystems() const;
+	virtual const QPtrList<BosonEffect>* effects() const;
 
 	/**
-	 * Clear the particle systems list. Note that the particles are
+	 * Clear the effects list. Note that the effects are
 	 * <em>not</em> deleted - @ref BosonCanvas will take care of this
-	 * anyway. Just the @ref particleSystems list is meant to be cleared.
+	 * anyway. Just the @ref effects list is meant to be cleared.
 	 **/
-	virtual void clearParticleSystems();
+	virtual void clearEffects();
 
 	void loadWeapons();
 
@@ -404,8 +451,11 @@ public:
 	BosonWeapon* weapon(unsigned long int id) const;
 
 
+	virtual void setMovingStatus(MovingStatus m);
+
+
 protected:
-	void setParticleSystems(const QPtrList<BosonParticleSystem>& list);
+	void setEffects(const QPtrList<BosonEffect>& list);
 
 	void shootAt(BosonWeapon* w, Unit* target);
 
@@ -415,9 +465,9 @@ protected:
 	 **/
 	QValueList<Unit*> unitCollisions(bool exact = false);
 
-	/** 
+	/**
 	 * Finds new path to destination.
-	 * Destination must have been set before in variables movedestx and 
+	 * Destination must have been set before in variables movedestx and
 	 * movedesty - usually using @ref moveTo
 	 *
 	 * This is in Unit instead of @ref MobileUnit so that we can apply a
@@ -437,18 +487,35 @@ protected:
 	 **/
 	virtual void advanceMoveCheck() { }
 
-	/**
-	 * @return Whether unit should attack any enemy units in range while moving
-	 **/
-	int moveAttacking() const;
-
-	int slowDownAtDestination() const;
 
 	/**
-	 * @return Whether new path should be searched
+	 * Resets internal pathfinder's info structure
 	 **/
-	int searchPath() const;
-	void setSearchPath(int search);
+	void resetPathInfo();
+
+
+	/**
+	 * Adds so-called path point for this unit.
+	 * Path points are like waypoints, except that they are internal.
+	 **/
+	void addPathPoint(const QPoint& pos);
+	/**
+	 * @return Number of path-points
+	 **/
+	unsigned int pathPointCount() const;
+	/**
+	 * @return Current path-point (where unit should go next)
+	 **/
+	const QPoint& currentPathPoint();
+	/**
+	 * Clears list of path-points
+	 **/
+	void clearPathPoints();
+	/**
+	 * Removes first path-point from the list
+	 **/
+	void pathPointDone();
+
 
 	/**
 	 * Called when the unit wants to move to another position. It may be
@@ -505,7 +572,7 @@ public:
 	 * Turn to direction. This sets a new frame according to the new
 	 * direction.
 	 **/
-	void turnTo(Direction direction);
+	virtual void turnTo(int degrees);
 
 	/**
 	 * Call turnTo according to the current speed (you want to use this!)
@@ -531,13 +598,22 @@ public:
 
 	bool attackEnemyUnitsInRangeWhileMoving();
 
+	/**
+	 * Move towards p, going at most maxdist (in canvas coords).
+	 * How much unit should move, will be added to xspeed and yspeed.
+	 * (x; y) marks unit's current position
+	 *
+	 * @return How much is moved (will be <= maxdist)
+	 **/
+	float moveTowardsPoint(const QPoint& p, float x, float y, float maxdist, float &xspeed, float &yspeed);
+
 	virtual void newPath();
 
 	/**
-	 * Check if waypoint wp marks end of the path. If yes, then it stops unit,
+	 * Check if pathpoint p marks end of the path. If yes, then it stops unit,
 	 * turns to random direction and true, otherwise returns false.
 	 **/
-	bool checkWaypoint(const QPoint& wp);
+	bool checkPathPoint(const QPoint& p);
 
 protected:
 	virtual void advanceMoveInternal(unsigned int advanceCount); // move one step futher to path
@@ -555,7 +631,7 @@ private:
 	// a d pointer is probably not very good here - far too much memory consumption
 	// same apllies to Unit and UnitBase. But it speeds up compiling as we don't
 	// have to change the headers every time...
-	class MobileUnitPrivate; 
+	class MobileUnitPrivate;
 	MobileUnitPrivate* d;
 };
 
