@@ -39,7 +39,6 @@ UnitBase::UnitBase(const UnitProperties* prop, Player* owner, BosonCanvas* canva
  initStatic();
  mWeaponProperties = 0; // created on the fly in weaponDataHandler()
  mUnitProperties = prop; // WARNING: this might be 0 at this point! MUST be != 0 for Unit, but ScenarioUnit uses 0 here
- mIsMoving = false;
 
 // PolicyLocal?
  registerData(&mHealth, IdHealth);
@@ -50,6 +49,7 @@ UnitBase::UnitBase(const UnitProperties* prop, Player* owner, BosonCanvas* canva
  registerData(&mWork, IdWork);
  registerData(&mAdvanceWork, IdAdvanceWork);
  registerData(&mDeletionTimer, IdDeletionTimer);
+ registerData(&mMovingStatus, IdMovingStatus);
 
  // these properties are fully internal (i.e. nothing is displayed in any
  // widget) and they change very often. So we increase speed greatly by not
@@ -59,6 +59,7 @@ UnitBase::UnitBase(const UnitProperties* prop, Player* owner, BosonCanvas* canva
 
  mWork.setLocal((int)WorkNone);
  mAdvanceWork.setLocal((int)WorkNone);
+ mMovingStatus.setLocal((int)Standing);
  mHealth.setLocal(0); // initially destroyed
  mShields.setLocal(0); // doesn't have any shields
  mShieldReloadCounter.setLocal(0);
@@ -93,6 +94,7 @@ void UnitBase::initStatic()
  addPropertyId(IdAdvanceWork, QString::fromLatin1("AdvanceWork"));
  addPropertyId(IdDeletionTimer, QString::fromLatin1("DeletionTimer"));
  addPropertyId(IdShieldReloadCounter, QString::fromLatin1("ShieldReloadCounter"));
+ addPropertyId(IdMovingStatus, QString::fromLatin1("MovingStatus"));
  initialized = true;
 }
 
