@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,11 +21,12 @@
 
 #include "speciestheme.h"
 #include "pluginproperties.h"
-#include "bosonparticlesystem.h"
-#include "bosonparticlesystemproperties.h"
+#include "bosoneffect.h"
+#include "bosoneffectproperties.h"
 #include "bosonweapon.h"
 #include "bosonconfig.h"
 #include "bodebug.h"
+#include "boaction.h"
 
 #include <ksimpleconfig.h>
 #include <klocale.h>
@@ -47,10 +48,16 @@ public:
 	QMap<QString, QString> mTextureNames;
 	QMap<int, QString> mSounds;
 
-	QPtrList<BosonParticleSystemProperties> mDestroyedParticleSystems;
-	QValueList<unsigned long int> mDestroyedParticleSystemIds;
-	QPtrList<BosonParticleSystemProperties> mConstructedParticleSystems;
-	QValueList<unsigned long int> mConstructedParticleSystemIds;
+	QIntDict<BoAction> mActions;
+
+	QPtrList<BosonEffectProperties> mDestroyedEffects;
+	QValueList<unsigned long int> mDestroyedEffectIds;
+	QPtrList<BosonEffectProperties> mConstructedEffects;
+	QValueList<unsigned long int> mConstructedEffectIds;
+	QPtrList<BosonEffectProperties> mExplodingFragmentFlyEffects;
+	QValueList<unsigned long int> mExplodingFragmentFlyEffectIds;
+	QPtrList<BosonEffectProperties> mExplodingFragmentHitEffects;
+	QValueList<unsigned long int> mExplodingFragmentHitEffectIds;
 	BoVector3 mHitPoint;  // FIXME: better name
 };
 
@@ -62,6 +69,9 @@ public:
 	}
 
 	float mSpeed;
+	float mAccelerationSpeed;
+	float mDecelerationSpeed;
+	int mRotationSpeed;
 	bool mCanGoOnLand;
 	bool mCanGoOnWater;
 };
@@ -133,7 +143,7 @@ void UnitProperties::loadUnitType(const QString& fileName, bool fullmode)
  }
  mUnitWidth = (unsigned int)(conf.readDoubleNumEntry("UnitWidth", 1.0) * BO_TILE_SIZE);
  mUnitHeight = (unsigned int)(conf.readDoubleNumEntry("UnitHeight", 1.0) * BO_TILE_SIZE);
- mUnitDepth = (unsigned int)(conf.readDoubleNumEntry("UnitDepth", 1.0) * BO_TILE_SIZE);
+ mUnitDepth = (float)conf.readDoubleNumEntry("UnitDepth", 1.0);
  d->mName = conf.readEntry("Name", i18n("Unknown"));
  mHealth = conf.readUnsignedLongNumEntry("Health", 100);
  mMineralCost = conf.readUnsignedLongNumEntry("MineralCost", 100);
@@ -147,14 +157,24 @@ void UnitProperties::loadUnitType(const QString& fileName, bool fullmode)
  d->mRequirements = BosonConfig::readUnsignedLongNumList(&conf, "Requirements");
  mExplodingDamage = conf.readLongNumEntry("ExplodingDamage", 0);
  mExplodingDamageRange = (float)(conf.readDoubleNumEntry("ExplodingDamageRange", 0));
- d->mHitPoint = BoVector3::load(&conf, "HitPoint");  // FIXME: better name
+ mExplodingFragmentCount = conf.readUnsignedNumEntry("ExplodingFragmentCount", 0);
+ mExplodingFragmentDamage = conf.readLongNumEntry("ExplodingFragmentDamage", 10);
+ mExplodingFragmentDamageRange = (float)conf.readDoubleNumEntry("ExplodingFragmentDamageRange", 0.5);
+ mRemoveWreckageImmediately = conf.readBoolEntry("RemoveWreckageImmediately", false);
+ d->mExplodingFragmentFlyEffectIds = BosonConfig::readUnsignedLongNumList(&conf, "ExplodingFragmentFlyEffects");
+ d->mExplodingFragmentHitEffectIds = BosonConfig::readUnsignedLongNumList(&conf, "ExplodingFragmentHitEffects");
+ if (mFullMode) {
+	d->mExplodingFragmentFlyEffects = BosonEffectProperties::loadEffectProperties(d->mExplodingFragmentFlyEffectIds, mTheme);
+	d->mExplodingFragmentHitEffects = BosonEffectProperties::loadEffectProperties(d->mExplodingFragmentHitEffectIds, mTheme);
+ }
+ d->mHitPoint = BosonConfig::readBoVector3Entry(&conf, "HitPoint");  // FIXME: better name
  d->mHitPoint.cellToCanvas();
 
- d->mDestroyedParticleSystemIds = BosonConfig::readUnsignedLongNumList(&conf, "DestroyedParticles");
- d->mConstructedParticleSystemIds = BosonConfig::readUnsignedLongNumList(&conf, "ConstructedParticles");
+ d->mDestroyedEffectIds = BosonConfig::readUnsignedLongNumList(&conf, "DestroyedEffects");
+ d->mConstructedEffectIds = BosonConfig::readUnsignedLongNumList(&conf, "ConstructedEffects");
  if (mFullMode) {
-	d->mDestroyedParticleSystems = BosonParticleSystemProperties::loadParticleSystemProperties(d->mDestroyedParticleSystemIds, mTheme);
-	d->mConstructedParticleSystems = BosonParticleSystemProperties::loadParticleSystemProperties(d->mConstructedParticleSystemIds, mTheme);
+	d->mDestroyedEffects = BosonEffectProperties::loadEffectProperties(d->mDestroyedEffectIds, mTheme);
+	d->mConstructedEffects = BosonEffectProperties::loadEffectProperties(d->mConstructedEffectIds, mTheme);
  }
 
  if (isFacility) {
@@ -181,7 +201,7 @@ void UnitProperties::saveUnitType(const QString& fileName)
  conf.writeEntry("TerrainType", (int)mTerrain);
  conf.writeEntry("UnitWidth", (double)mUnitWidth / BO_TILE_SIZE);
  conf.writeEntry("UnitHeight", (double)mUnitHeight / BO_TILE_SIZE);
- conf.writeEntry("UnitDepth", (double)mUnitDepth / BO_TILE_SIZE);
+ conf.writeEntry("UnitDepth", (double)mUnitDepth);
  conf.writeEntry("Name", d->mName);
  conf.writeEntry("Health", mHealth);
  conf.writeEntry("MineralCost", mMineralCost);
@@ -197,11 +217,11 @@ void UnitProperties::saveUnitType(const QString& fileName)
  conf.writeEntry("ExplodingDamageRange", mExplodingDamageRange);
  BoVector3 tmpHitPoint(d->mHitPoint);
  tmpHitPoint.canvasToCell();
- tmpHitPoint.save(&conf, "HitPoint");
+ BosonConfig::writeEntry(&conf, "HitPoint", tmpHitPoint);
  conf.writeEntry("Producer", mProducer);
 
- BosonConfig::writeUnsignedLongNumList(&conf, "DestroyedParticles", d->mDestroyedParticleSystemIds);
- BosonConfig::writeUnsignedLongNumList(&conf, "ConstructedParticles", d->mConstructedParticleSystemIds);
+ BosonConfig::writeUnsignedLongNumList(&conf, "DestroyedEffects", d->mDestroyedEffectIds);
+ BosonConfig::writeUnsignedLongNumList(&conf, "ConstructedEffects", d->mConstructedEffectIds);
 
  if (isFacility()) {
 	saveFacilityProperties(&conf);
@@ -224,6 +244,9 @@ void UnitProperties::loadMobileProperties(KSimpleConfig* conf)
 			" for unit " << typeId() << ", defaulting to 0" << endl;
 	mMobileProperties->mSpeed = 0;
  }
+ mMobileProperties->mAccelerationSpeed = (float)conf->readDoubleNumEntry("AccelerationSpeed", 0.1);
+ mMobileProperties->mDecelerationSpeed = (float)conf->readDoubleNumEntry("DecelerationSpeed", 0.2);
+ mMobileProperties->mRotationSpeed = conf->readNumEntry("RotationSpeed", (int)(mMobileProperties->mSpeed * 2));
  mMobileProperties->mCanGoOnLand = conf->readBoolEntry("CanGoOnLand",
 		(isLand() || isAircraft()));
  mMobileProperties->mCanGoOnWater = conf->readBoolEntry("CanGoOnWater",
@@ -252,6 +275,9 @@ void UnitProperties::loadAllPluginProperties(KSimpleConfig* conf)
 	boDebug() << k_funcinfo << "loading refinery plugin properties" << endl;
 	loadPluginProperties(new RefineryProperties(this), conf);
  }
+ if (conf->hasGroup(ResourceMineProperties::propertyGroup())) {
+	loadPluginProperties(new ResourceMineProperties(this), conf);
+ }
 }
 
 void UnitProperties::loadPluginProperties(PluginProperties* prop, KSimpleConfig* conf)
@@ -306,6 +332,9 @@ void UnitProperties::loadWeapons(KSimpleConfig* conf)
 	BosonWeaponProperties* p = new BosonWeaponProperties(this, i + 1);
 	p->loadPlugin(conf, mFullMode);
 	d->mPlugins.append(p);
+	if (!p->autoUse()) {
+		continue;
+	}
 	if (p->canShootAtAirUnits()) {
 		mCanShootAtAirUnits = true;
 		if(p->range() > mMaxAirWeaponRange) {
@@ -322,10 +351,41 @@ void UnitProperties::loadWeapons(KSimpleConfig* conf)
  mMaxWeaponRange = QMAX(mMaxAirWeaponRange, mMaxLandWeaponRange);
 }
 
+void UnitProperties::loadActions()
+{
+ KSimpleConfig conf(unitPath() + "index.unit");
+ // Produce action first
+ // Produce is special because it uses little overview as pixmap and it's
+ //  tooltip text is auto-generated. Only thing we load here is hotkey
+ // TODO: load hotkey
+ conf.setGroup("Actions");
+ mProduceAction = new BoAction(QString("ProduceAction-%1").arg(typeId()), theme()->smallOverview(typeId()),
+		"Produce"/*, hotkey*/);
+ if (canShoot()) {
+	d->mActions.insert(ActionAttack, theme()->action(conf.readEntry("ActionAttack", "ActionAttack")));
+ }
+ if (isMobile()) {
+	d->mActions.insert(ActionMove, theme()->action(conf.readEntry("ActionMove", "ActionMove")));
+	d->mActions.insert(ActionFollow, theme()->action(conf.readEntry("ActionFollow", "ActionFollow")));
+ }
+ if (properties(PluginProperties::Harvester)) {
+	d->mActions.insert(ActionHarvest, theme()->action(conf.readEntry("ActionHarvest", "ActionHarvest")));
+ }
+ if (properties(PluginProperties::Repair)) {
+	d->mActions.insert(ActionHarvest, theme()->action(conf.readEntry("ActionRepair", "ActionRepair")));
+ }
+ if (!d->mActions.isEmpty()) {
+	d->mActions.insert(ActionStop, theme()->action(conf.readEntry("ActionStop", "ActionStop")));
+ }
+}
+
 void UnitProperties::saveMobileProperties(KSimpleConfig* conf)
 {
  conf->setGroup("Boson Mobile Unit");
  conf->writeEntry("Speed", (double)mMobileProperties->mSpeed);
+ conf->writeEntry("AccelerationSpeed", (double)mMobileProperties->mAccelerationSpeed);
+ conf->writeEntry("DecelerationSpeed", (double)mMobileProperties->mDecelerationSpeed);
+ conf->writeEntry("RotationSpeed", mMobileProperties->mRotationSpeed);
  conf->writeEntry("CanGoOnLand", mMobileProperties->mCanGoOnLand);
  conf->writeEntry("CanGoOnWater", mMobileProperties->mCanGoOnWater);
 }
@@ -418,6 +478,16 @@ QMap<int, QString> UnitProperties::sounds() const
  return d->mSounds;
 }
 
+BoAction* UnitProperties::action(UnitAction type) const
+{
+ return d->mActions[type];
+}
+
+const QIntDict<BoAction>* UnitProperties::allActions() const
+{
+ return &d->mActions;
+}
+
 void UnitProperties::setRequirements(QValueList<unsigned long int> requirements)
 {
  d->mRequirements = requirements;
@@ -441,6 +511,30 @@ float UnitProperties::speed() const
  return mMobileProperties->mSpeed;
 }
 
+float UnitProperties::accelerationSpeed() const
+{
+ if (!mMobileProperties) {
+	return 0;
+ }
+ return mMobileProperties->mAccelerationSpeed;
+}
+
+float UnitProperties::decelerationSpeed() const
+{
+ if (!mMobileProperties) {
+	return 0;
+ }
+ return mMobileProperties->mDecelerationSpeed;
+}
+
+int UnitProperties::rotationSpeed() const
+{
+ if (!mMobileProperties) {
+	return 0;
+ }
+ return mMobileProperties->mRotationSpeed;
+}
+
 bool UnitProperties::canGoOnLand() const
 {
  if (!mMobileProperties) {
@@ -476,32 +570,24 @@ const PluginProperties* UnitProperties::properties(int pluginType) const
  return 0;
 }
 
-QPtrList<BosonParticleSystem> UnitProperties::newDestroyedParticleSystems(float x, float y, float z) const
+QPtrList<BosonEffect> UnitProperties::newDestroyedEffects(float x, float y, float z) const
 {
- QPtrList<BosonParticleSystem> list;
- QPtrListIterator<BosonParticleSystemProperties> it(d->mDestroyedParticleSystems);
- while (it.current()) {
-	BosonParticleSystem* s = it.current()->newSystem(BoVector3(x, y, z));
-	if (s) {
-		list.append(s);
-	}
-	++it;
- }
- return list;
+ return BosonEffectProperties::newEffects(&d->mDestroyedEffects, BoVector3(x, y, z));
 }
 
-QPtrList<BosonParticleSystem> UnitProperties::newConstructedParticleSystems(float x, float y, float z) const
+QPtrList<BosonEffect> UnitProperties::newConstructedEffects(float x, float y, float z) const
 {
- QPtrList<BosonParticleSystem> list;
- QPtrListIterator<BosonParticleSystemProperties> it(d->mConstructedParticleSystems);
- while (it.current()) {
-	BosonParticleSystem* s = it.current()->newSystem(BoVector3(x, y, z));
-	if (s) {
-		list.append(s);
-	}
-	++it;
- }
- return list;
+ return BosonEffectProperties::newEffects(&d->mConstructedEffects, BoVector3(x, y, z));
+}
+
+QPtrList<BosonEffect> UnitProperties::newExplodingFragmentFlyEffects(BoVector3 pos) const
+{
+ return BosonEffectProperties::newEffects(&d->mExplodingFragmentFlyEffects, pos);
+}
+
+QPtrList<BosonEffect> UnitProperties::newExplodingFragmentHitEffects(BoVector3 pos) const
+{
+ return BosonEffectProperties::newEffects(&d->mExplodingFragmentHitEffects, pos);
 }
 
 void UnitProperties::clearPlugins(bool deleteweapons)
@@ -522,14 +608,10 @@ void UnitProperties::clearPlugins(bool deleteweapons)
  } else {
 	d->mPlugins.clear();
  }
- if (mMobileProperties) {
-	delete mMobileProperties;
-	mMobileProperties = 0;
- }
- if (mFacilityProperties) {
-	delete mFacilityProperties;
-	mFacilityProperties = 0;
- }
+ delete mMobileProperties;
+ mMobileProperties = 0;
+ delete mFacilityProperties;
+ mFacilityProperties = 0;
 }
 
 void UnitProperties::createMobileProperties()
@@ -561,6 +643,27 @@ void UnitProperties::setSpeed(float speed)
  }
 }
 
+void UnitProperties::setAccelerationSpeed(float speed)
+{
+ if (mMobileProperties) {
+	mMobileProperties->mAccelerationSpeed = speed;
+ }
+}
+
+void UnitProperties::setDecelerationSpeed(float speed)
+{
+ if (mMobileProperties) {
+	mMobileProperties->mDecelerationSpeed = speed;
+ }
+}
+
+void UnitProperties::setRotationSpeed(int speed)
+{
+ if (mMobileProperties) {
+	mMobileProperties->mRotationSpeed = speed;
+ }
+}
+
 void UnitProperties::setCanGoOnLand(bool c)
 {
  if (mMobileProperties) {
@@ -585,24 +688,24 @@ void UnitProperties::addSound(int event, QString filename)
  d->mSounds.insert(event, filename);
 }
 
-void UnitProperties::setDestroyedParticleSystemIds(QValueList<unsigned long int> ids)
+void UnitProperties::setDestroyedEffectIds(QValueList<unsigned long int> ids)
 {
- d->mDestroyedParticleSystemIds = ids;
+ d->mDestroyedEffectIds = ids;
 }
 
-QValueList<unsigned long int> UnitProperties::destroyedParticleSystemIds() const
+QValueList<unsigned long int> UnitProperties::destroyedEffectIds() const
 {
- return d->mDestroyedParticleSystemIds;
+ return d->mDestroyedEffectIds;
 }
 
-void UnitProperties::setConstructedParticleSystemIds(QValueList<unsigned long int> ids)
+void UnitProperties::setConstructedEffectIds(QValueList<unsigned long int> ids)
 {
- d->mConstructedParticleSystemIds = ids;
+ d->mConstructedEffectIds = ids;
 }
 
-QValueList<unsigned long int> UnitProperties::constructedParticleSystemIds() const
+QValueList<unsigned long int> UnitProperties::constructedEffectIds() const
 {
- return d->mConstructedParticleSystemIds;
+ return d->mConstructedEffectIds;
 }
 
 void UnitProperties::reset()
@@ -618,7 +721,7 @@ void UnitProperties::reset()
  mTerrain = (TerrainType)0;
  mUnitWidth = (unsigned int)(1.0 * BO_TILE_SIZE);
  mUnitHeight = (unsigned int)(1.0 * BO_TILE_SIZE);
- mUnitDepth = (unsigned int)(1.0 * BO_TILE_SIZE);
+ mUnitDepth = 1.0;
  d->mName = i18n("Unknown");
  mHealth = 100;
  mMineralCost = 100;
@@ -629,22 +732,23 @@ void UnitProperties::reset()
  mArmor = 0;
  mSupportMiniMap = false;
  d->mRequirements.clear();
- d->mDestroyedParticleSystemIds.clear();
- d->mConstructedParticleSystemIds.clear();
+ d->mDestroyedEffectIds.clear();
+ d->mConstructedEffectIds.clear();
  d->mHitPoint.reset();
  mProducer = 0;
  mExplodingDamage = 0;
  mExplodingDamageRange = 0;
  // Delete old mobile/facility properties
- if (mMobileProperties) {
-	delete mMobileProperties;
- }
- if (mFacilityProperties) {
-	delete mFacilityProperties;
- }
+ delete mMobileProperties;
+ mMobileProperties = 0;
+ delete mFacilityProperties;
+ mFacilityProperties = 0;
  // Mobile stuff (because unit is mobile by default)
  createMobileProperties();
  mMobileProperties->mSpeed = 0; // Hmm, this doesn't make any sense IMO
+ mMobileProperties->mAccelerationSpeed = 0.5;
+ mMobileProperties->mDecelerationSpeed = 1.0;
+ mMobileProperties->mRotationSpeed = (int)(2 * mMobileProperties->mSpeed);
  mMobileProperties->mCanGoOnLand = true;
  mMobileProperties->mCanGoOnWater = false;
  // Sounds
